@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2000-2002 the xine project
+# Copyright (C) 2000-2003 the xine project
 #
 # This file is part of xine, a unix video player.
 # 
@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 #
-# $Id: autogen.sh,v 1.7 2003/01/05 19:32:34 storri Exp $
+# $Id: autogen.sh,v 1.16 2004/10/08 20:22:29 mroi Exp $
 #
 # Maintained by Stephen Torri <storri@users.sourceforge.net>
 #
@@ -27,6 +27,8 @@
 PROG=xine-lib
 
 # Minimum value required to build
+WANT_AUTOMAKE_1_6=1 export WANT_AUTOMAKE_1_6
+WANT_AUTOMAKE=1.6 export WANT_AUTOMAKE
 AUTOMAKE_MIN=1.6.0
 AUTOCONF_MIN=2.53
 LIBTOOL_MIN=1.4.0
@@ -105,13 +107,14 @@ run_autoconf () {
 #--------------------
 # LIBTOOL
 #-------------------
-detect_libtool() {
-  set -- `type libtool 2>/dev/null`
+try_libtool_executable() {
+  libtool=$1
+  set -- `type $libtool 2>/dev/null`
   RETVAL=$?
   NUM_RESULT=$#
   RESULT_FILE=$3
   if [ $RETVAL -eq 0 -a $NUM_RESULT -eq 3 -a -f "$RESULT_FILE" ]; then
-    LT="`libtool --version | awk '{ print $4 }' | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
+    LT="`$libtool --version | awk '{ print $4 }' | sed -e 's/[a-zA-Z\ \.\(\)\-\;]//g'`"
     LIBTOOL_MIN="`echo $LIBTOOL_MIN | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
     if test $LT -lt 100 ; then
       LT=`expr $LT \* 10`
@@ -119,12 +122,21 @@ detect_libtool() {
     if [ `expr $LT` -ge $LIBTOOL_MIN ]; then
       libtool_ok=yes
     fi
-  else
-    echo
-    echo "**Error**: You must have \`libtool' >= $LIBTOOL_MIN installed to" 
-    echo "           compile $PROG. Download the appropriate package"
-    echo "           for your distribution or source from ftp.gnu.org."
-    exit 1
+  fi
+}
+
+detect_libtool() {
+  # try glibtool first, then libtool
+  try_libtool_executable 'glibtool'
+  if [ "x$libtool_ok" != "xyes" ]; then
+    try_libtool_executable 'libtool'
+    if [ "x$libtool_ok" != "xyes" ]; then
+      echo
+      echo "**Error**: You must have \`libtool' >= $LIBTOOL_MIN installed to" 
+      echo "           compile $PROG. Download the appropriate package"
+      echo "           for your distribution or source from ftp.gnu.org."
+      exit 1
+    fi
   fi
 }
 
@@ -137,7 +149,7 @@ run_libtoolize() {
   fi
 
   echo $_echo_n " + Running libtoolize: $_echo_c";
-    libtoolize --force --copy >/dev/null 2>&1;
+    "${libtool}ize" --force --copy >/dev/null 2>&1;
   echo "done."
 }
 
