@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2000-2002 the xine project
+/*
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -16,11 +16,12 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_elem.c,v 1.67 2003/01/04 14:48:11 miguelfreitas Exp $
+ */
+
+/*
+ * $Id: demux_elem.c,v 1.85 2004/06/13 21:28:52 miguelfreitas Exp $
  *
  * demultiplexer for elementary mpeg streams
- * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,84 +34,70 @@
 #include <unistd.h>
 #include <string.h>
 
+#define LOG_MODULE "demux_elem"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "compat.h"
 #include "demux.h"
 
-/*
-#define LOG
-*/
-
 #define NUM_PREVIEW_BUFFERS 50
+#define SCRATCH_SIZE 256
 
 typedef struct {  
-
   demux_plugin_t      demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
-  int                  blocksize;
   int                  status;
-  
-  uint8_t              scratch[4096];
 
-  char                 last_mrl[1024];
+  uint32_t             blocksize;
 } demux_mpeg_elem_t ;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_mpeg_elem_class_t;
 
 static int demux_mpeg_elem_next (demux_mpeg_elem_t *this, int preview_mode) {
   buf_element_t *buf;
-  int n;
-
-#ifdef LOG
-  printf ("demux_elem: next piece\n");
-#endif
+  uint32_t blocksize;
+  off_t done;
 
+  lprintf ("next piece\n");
   buf = this->video_fifo->buffer_pool_alloc(this->video_fifo);
-  buf->content = buf->mem;
-  buf->type = BUF_DEMUX_BLOCK;
+  blocksize = (this->blocksize ? this->blocksize : buf->max_size);
+  done = this->input->read(this->input, buf->mem, blocksize);
+  lprintf ("read size = %lld\n", done);
 
-  n = this->input->read (this->input, buf->mem, this->blocksize);
-
-  if (n<=0) {
+  if (done <= 0) {
     buf->free_buffer (buf);
     this->status = DEMUX_FINISHED;
     return 0;
   }
 
-  buf->size = n;
-
-  buf->pts             = 0;
-  buf->extra_info->input_pos       = this->input->get_current_pos(this->input);
-  buf->type            = BUF_VIDEO_MPEG;
+  buf->size                  = done;
+  buf->content               = buf->mem;
+  buf->pts                   = 0;
+  if( this->input->get_length (this->input) )
+    buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                     65535 / this->input->get_length (this->input) );
+  buf->type                  = BUF_VIDEO_MPEG;
 
   if (preview_mode)
     buf->decoder_flags = BUF_FLAG_PREVIEW;
 
   this->video_fifo->put(this->video_fifo, buf);
   
-  return buf->size>0;
+  return 1;
 }
 
 static int demux_mpeg_elem_send_chunk (demux_plugin_t *this_gen) {
-
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
 
   if (!demux_mpeg_elem_next(this, 0))
@@ -126,23 +113,20 @@ static int demux_mpeg_elem_get_status (demux_plugin_t *this_gen) {
 
 
 static void demux_mpeg_elem_send_headers (demux_plugin_t *this_gen) {
-
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
 
   this->video_fifo  = this->stream->video_fifo;
   this->audio_fifo  = this->stream->audio_fifo;
 
   this->blocksize = this->input->get_blocksize(this->input);
-  if (!this->blocksize)
-    this->blocksize = 2048;
-  
-  xine_demux_control_start(this->stream);
-  
-  if ((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
+
+  _x_demux_control_start(this->stream);
+
+  if (INPUT_IS_SEEKABLE(this->input)) {
     int num_buffers = NUM_PREVIEW_BUFFERS;
-    
+
     this->input->seek (this->input, 0, SEEK_SET);
-    
+
     this->status = DEMUX_OK ;
     while ((num_buffers > 0) && (this->status == DEMUX_OK)) {
       demux_mpeg_elem_next(this, 1);
@@ -152,31 +136,33 @@ static void demux_mpeg_elem_send_headers (demux_plugin_t *this_gen) {
 
   this->status = DEMUX_OK;
 
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 0;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 0);
 }
 
 static int demux_mpeg_elem_seek (demux_plugin_t *this_gen,
-				  off_t start_pos, int start_time) {
+				  off_t start_pos, int start_time, int playing) {
 
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
-  
+  start_pos = (off_t) ( (double) start_pos / 65535 *
+              this->input->get_length (this->input) );
+
   this->status = DEMUX_OK;
 
-  if (this->stream->demux_thread_running) 
-    xine_demux_flush_engine(this->stream);
-  
-  if ((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
-  
+  if (playing)
+    _x_demux_flush_engine(this->stream);
+
+  if (INPUT_IS_SEEKABLE(this->input)) {
+
     /* FIXME: implement time seek */
 
-    this->input->seek (this->input, start_pos, SEEK_SET);
-
-#ifdef LOG
-    printf ("demux_elem: seeking to %d\n", start_pos);
-#endif
+    if (start_pos != this->input->seek (this->input, start_pos, SEEK_SET)) {
+      this->status = DEMUX_FINISHED;
+      return this->status;
+    }
+    lprintf ("seeking to %lld\n", start_pos);
   }
-  
+
   /*
    * now start demuxing
    */
@@ -204,11 +190,58 @@ static int demux_mpeg_elem_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                    input_plugin_t *input_gen) {
+                                    input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_mpeg_elem_t *this;
 
+  switch (stream->content_detection_method) {
+
+  case METHOD_BY_CONTENT: {
+    uint8_t scratch[SCRATCH_SIZE];
+    int i, read, found;
+
+    read = _x_demux_read_header(input, scratch, SCRATCH_SIZE);
+    if (!read)
+      return NULL;
+
+    found = 0;
+    for (i = 0; i < read - 4; i++) {
+      lprintf ("%02x %02x %02x %02x\n", scratch[i], scratch[i+1], scratch[i+2], scratch[i+3]);
+
+      if ((scratch[i] == 0x00) && (scratch[i+1] == 0x00) && (scratch[i+2] == 0x01)) {
+	if (scratch[i+3] == 0xb3) {
+	  found = 1;
+	  lprintf ("found header at offset 0x%x\n", i);
+	  break;
+	} else
+	  return NULL;
+      }
+    }
+
+    if (found == 0)
+      return NULL;
+    lprintf ("input accepted.\n");
+  }
+  break;
+
+  case METHOD_BY_EXTENSION: {
+    char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions))
+      return NULL;
+  }
+  break;
+
+  case METHOD_EXPLICIT:
+  break;
+
+  default:
+    return NULL;
+  }
+
   this         = xine_xmalloc (sizeof (demux_mpeg_elem_t));
   this->stream = stream;
   this->input  = input;
@@ -219,97 +252,12 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_mpeg_elem_dispose;
   this->demux_plugin.get_status        = demux_mpeg_elem_get_status;
   this->demux_plugin.get_stream_length = demux_mpeg_elem_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_mpeg_elem_get_capabilities;
   this->demux_plugin.get_optional_data = demux_mpeg_elem_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
 
   this->status = DEMUX_FINISHED;
 
-  switch (stream->content_detection_method) {
-
-  case METHOD_BY_CONTENT: {
-    int got_sample;
-    int bs = 4;
-    
-    if(!input) {
-      free (this);
-      return NULL;
-    }
-
-    got_sample = 0;
-  
-    if ((input->get_capabilities(input) & INPUT_CAP_SEEKABLE) != 0) {
-      input->seek(input, 0, SEEK_SET);
-
-      bs = input->get_blocksize(input);
-
-      if (bs<4)
-	bs = 4;
-
-      if (input->read(input, this->scratch, bs) == bs) 
-	got_sample = 1;
-    } else if ((input->get_capabilities(input) & INPUT_CAP_PREVIEW) != 0) {
-      input->get_optional_data (input, this->scratch, INPUT_OPTIONAL_DATA_PREVIEW);
-      got_sample = 1;
-    }
-
-    if (!got_sample) {
-      free (this);
-      return NULL;
-    }
-
-#ifdef LOG	
-    printf ("demux_elem: %02x %02x %02x %02x (bs=%d)\n",
-	    this->scratch[0], this->scratch[1], 
-	    this->scratch[2], this->scratch[3], bs);
-#endif
-	
-    if (this->scratch[0] || this->scratch[1] 
-	|| (this->scratch[2] != 0x01) || (this->scratch[3] != 0xb3)) {
-      free (this);
-      return NULL;
-    }
-	
-    this->input = input;
-    
-#ifdef LOG	
-    printf ("demux_elem: input accepted.\n");
-#endif
-
-  }
-  break;
-
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".mpv", 4)) {
-      free (this);
-      return NULL;
-    }
-  }
-  break;
-
-  case METHOD_EXPLICIT:
-  break;
-
-  default:
-    free (this);
-    return NULL;
-  }
-
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
   return &this->demux_plugin;
 }
 
@@ -330,19 +278,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_mpeg_elem_class_t *this = (demux_mpeg_elem_class_t *) this_gen;
 
   free (this);
 }
 
 static void *init_plugin (xine_t *xine, void *data) {
-
   demux_mpeg_elem_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_mpeg_elem_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_mpeg_elem_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -357,9 +301,12 @@ static void *init_plugin (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
+demuxer_info_t demux_info_elem = {
+  0                       /* priority */
+};
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "elem", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 25, "elem", XINE_VERSION_CODE, &demux_info_elem, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
