@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,14 +16,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_mpeg.c,v 1.105 2003/01/16 22:25:53 miguelfreitas Exp $
+ */
+
+/*
+ * $Id: demux_mpeg.c,v 1.145 2004/12/17 20:08:44 miguelfreitas Exp $
  *
  * demultiplexer for mpeg 1/2 program streams
  * reads streams of variable blocksizes
- *
- * currently only used for mpeg-1-files
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,13 +34,19 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
-#include <unistd.h>
+
+#define LOG_MODULE "demux_mpeg"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
 
 #include "xine_internal.h"
 #include "demux.h"
 #include "xineutils.h"
 
 #define NUM_PREVIEW_BUFFERS 150
+#define SCRATCH_SIZE 256
 
 #define WRAP_THRESHOLD       120000
 
@@ -49,48 +54,35 @@
 #define PTS_VIDEO 1
 
 typedef struct demux_mpeg_s {
-
   demux_plugin_t       demux_plugin;
 
+  xine_stream_t	      *stream;
   fifo_buffer_t       *audio_fifo;
   fifo_buffer_t       *video_fifo;
-
-  xine_stream_t	      *stream;
   input_plugin_t      *input;
+  int                  status;
 
   unsigned char        dummy_space[100000];
-
-  int                  status;
   int                  preview_mode;
-
   int                  rate;
 
   int64_t              last_pts[2];
   int                  send_newpts;
   int                  buf_flag_seek;
   int                  has_pts;
-
 } demux_mpeg_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_mpeg_class_t;
 
+    /* code never reached, is it still usefull ?? */
 /*
  * borrow a little knowledge from the Quicktime demuxer
  */
 #include "bswap.h"
 
-#define QT_ATOM( ch0, ch1, ch2, ch3 )                                \
-        ( (long)(unsigned char)(ch3) | ( (long)(unsigned char)(ch2) << 8 ) | \
-        ( (long)(unsigned char)(ch1) << 16 ) | ( (long)(unsigned char)(ch0) << 24 ) )
-
+#define QT_ATOM BE_FOURCC
 /* these are the known top-level QT atoms */
 #define FREE_ATOM QT_ATOM('f', 'r', 'e', 'e')
 #define JUNK_ATOM QT_ATOM('j', 'u', 'n', 'k')
@@ -104,11 +96,11 @@ typedef struct {
 
 /* a little something for dealing with RIFF headers */
 
-#define FOURCC_TAG(ch0, ch1, ch2, ch3) QT_ATOM(ch0, ch1, ch2, ch3)
-
+#define FOURCC_TAG BE_FOURCC
 #define RIFF_TAG FOURCC_TAG('R', 'I', 'F', 'F')
 #define WAVE_TAG FOURCC_TAG('W', 'A', 'V', 'E')
 #define AVI_TAG FOURCC_TAG('A', 'V', 'I', ' ')
+#define FOURXM_TAG FOURCC_TAG('4', 'X', 'M', 'V')
 
 /* arbitrary number of initial file bytes to check for an MPEG marker */
 #define RIFF_CHECK_KILOBYTES 1024
@@ -185,7 +177,7 @@ static void find_mdat_atom(input_plugin_t *input, off_t *mdat_offset,
   }
 }
 
-static uint32_t read_bytes (demux_mpeg_t *this, int n) {
+static uint32_t read_bytes (demux_mpeg_t *this, uint32_t n) {
 
   uint32_t res;
   uint32_t i;
@@ -214,8 +206,8 @@ static uint32_t read_bytes (demux_mpeg_t *this, int n) {
     res = (buf[2]<<8) | buf[3] | (buf[1]<<16) | (buf[0] << 24);
     break;
   default:
-    printf ("demux_mpeg: how how - something wrong in wonderland demux:read_bytes (%d)\n", n);
-    abort();
+    lprintf ("how how - something wrong in wonderland demux:read_bytes (%d)\n", n);
+    _x_abort();
   }
 
   return res;
@@ -225,8 +217,7 @@ static uint32_t read_bytes (demux_mpeg_t *this, int n) {
    i guess llabs may not be available everywhere */
 #define abs(x) ( ((x)<0) ? -(x) : (x) )
 
-static void check_newpts( demux_mpeg_t *this, int64_t pts, int video )
-{
+static void check_newpts( demux_mpeg_t *this, int64_t pts, int video ) {
   int64_t diff;
 
   diff = pts - this->last_pts[video];
@@ -235,10 +226,10 @@ static void check_newpts( demux_mpeg_t *this, int64_t pts, int video )
       (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD) ) ) {
 
     if (this->buf_flag_seek) {
-      xine_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
+      _x_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
       this->buf_flag_seek = 0;
     } else {
-      xine_demux_control_newpts(this->stream, pts, 0);
+      _x_demux_control_newpts(this->stream, pts, 0);
     }
     this->send_newpts = 0;
     this->last_pts[1-video] = 0;
@@ -252,7 +243,7 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
 
   int            len, i;
   uint32_t       w, flags, header_len;
-  int64_t        pts;
+  int64_t        pts, dts;
   buf_element_t *buf = NULL;
 
   len = read_bytes(this, 2);
@@ -272,7 +263,7 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     if ((flags & 0x80) == 0x80) {
 
       w = read_bytes(this, 1);
-      pts = (w & 0x0e) << 29 ;
+      pts = (int64_t)(w & 0x0e) << 29 ;
       w = read_bytes(this, 2);
       pts |= (w & 0xFFFE) << 14;
       w = read_bytes(this, 2);
@@ -281,35 +272,141 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
       header_len -= 5 ;
     }
 
-    /* read rest of header */
-    i = this->input->read (this->input, this->dummy_space, header_len+4);
+    this->input->read (this->input, this->dummy_space, header_len);
 
+    i = this->input->read (this->input, this->dummy_space, 1);
     track = this->dummy_space[0] & 0x0F ;
 
-    /* contents */
+    /* DVD spu/subtitles */
+    if((this->dummy_space[0] & 0xE0) == 0x20) {
+
+      buf = this->input->read_block (this->input, this->video_fifo, len-1);
+
+      track = (this->dummy_space[0] & 0x1f);
+
+      buf->type      = BUF_SPU_DVD + track;
+      buf->decoder_flags |= BUF_FLAG_SPECIAL;
+      buf->decoder_info[1] = BUF_SPECIAL_SPU_DVD_SUBTYPE;
+      buf->decoder_info[2] = SPU_DVD_SUBTYPE_PACKAGE;
+      buf->pts       = pts;
+
+      this->video_fifo->put (this->video_fifo, buf);
 
-    if(this->audio_fifo)
-      buf = this->input->read_block (this->input, this->audio_fifo, len-4);
-    else {
-      this->input->read (this->input, this->dummy_space, len-4);
       return;
     }
 
-    if (buf == NULL) {
-      this->status = DEMUX_FINISHED;
-      return ;
+    /* SVCD OGT subtitles are in stream 0x70 */
+    if((this->dummy_space[0] & 0xFF) == 0x70) {
+      int spu_id = this->dummy_space[1] & 0x03;
+
+      buf = this->input->read_block (this->input, this->video_fifo, len-1);
+
+      buf->type      = BUF_SPU_SVCD + spu_id;
+      buf->pts       = pts;
+
+      /* There is a bug in WinSubMux doesn't redo PACK headers in 
+	 the private stream 1. This might cause the below to mess up.
+      if( !preview_mode )
+        check_newpts( this, this->pts, PTS_VIDEO );
+      */
+      this->video_fifo->put (this->video_fifo, buf);
+      lprintf ("SPU SVCD PACK (pts: %lld, spu id: %d) put on FIFO\n", 
+	       buf->pts, spu_id);
+
+      return;
     }
 
-    buf->type      = BUF_AUDIO_A52 + track;
-    buf->pts       = pts;
-    check_newpts( this, pts, PTS_AUDIO );
+    /* CVD subtitles are in stream 0x00-0x03 */
+    if((this->dummy_space[0] & 0xfc) == 0x00) {
 
-    if (this->preview_mode)
-      buf->decoder_flags = BUF_FLAG_PREVIEW;
+      buf = this->input->read_block (this->input, this->video_fifo, len-1);
 
-    buf->extra_info->input_pos = this->input->get_current_pos (this->input);
+      buf->type      = BUF_SPU_CVD + (this->dummy_space[0] & 0x03);
+      buf->pts       = pts;
 
-    this->audio_fifo->put (this->audio_fifo, buf);
+      this->video_fifo->put (this->video_fifo, buf);
+
+      return;
+    }
+
+    if((this->dummy_space[0] & 0xf0) == 0x80) {
+
+      /* read rest of header - AC3 */
+      i = this->input->read (this->input, this->dummy_space+1, 3);
+
+      /* contents */
+      for (i = len - 4; i > 0; i -= (this->audio_fifo)
+           ? this->audio_fifo->buffer_pool_buf_size : this->video_fifo->buffer_pool_buf_size) {
+        if(this->audio_fifo) {
+          buf = this->input->read_block (this->input, this->audio_fifo,
+            (i > this->audio_fifo->buffer_pool_buf_size) ? this->audio_fifo->buffer_pool_buf_size : i);
+
+          if (buf == NULL) {
+            this->status = DEMUX_FINISHED;
+            return;
+          }
+          if (track & 0x8) {
+            buf->type      = BUF_AUDIO_DTS + (track & 0x07); /* DVDs only have 8 tracks */
+          } else {
+            buf->type      = BUF_AUDIO_A52 + track;
+          }
+          buf->pts       = pts;
+          check_newpts( this, pts, PTS_AUDIO );
+          pts = 0;
+
+          if (this->preview_mode)
+            buf->decoder_flags = BUF_FLAG_PREVIEW;
+
+          if( this->input->get_length (this->input) )
+            buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                             65535 / this->input->get_length (this->input) );
+          
+          if (this->rate)
+            buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
+                                                  * 1000 / (this->rate * 50));
+
+          this->audio_fifo->put (this->audio_fifo, buf);
+
+        } else
+          this->input->read (this->input, this->dummy_space, i);
+
+      }
+      return;
+
+    } else if((this->dummy_space[0] & 0xf0) == 0xa0) {
+      i = this->input->read (this->input, this->dummy_space+1, 6);
+
+      buf = this->input->read_block (this->input, this->video_fifo, len-7);
+
+      buf->type      = BUF_AUDIO_LPCM_BE + track;
+      buf->decoder_flags |= BUF_FLAG_SPECIAL;
+      buf->decoder_info[1] = BUF_SPECIAL_LPCM_CONFIG;
+      buf->decoder_info[2] = this->dummy_space[5];
+      buf->pts       = pts;
+
+      check_newpts( this, pts, PTS_AUDIO );
+
+      if (this->preview_mode)
+        buf->decoder_flags |= BUF_FLAG_PREVIEW;
+
+      if( this->input->get_length (this->input) )
+        buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                         65535 / this->input->get_length (this->input) );
+      if (this->rate)
+        buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
+                                              * 1000 / (this->rate * 50));
+
+      if(this->audio_fifo)
+        this->audio_fifo->put (this->audio_fifo, buf);
+      else
+        buf->free_buffer(buf);
+
+      return;
+
+    } else {
+      for (i = len-1; i > 0; i -= 10000)
+        this->input->read (this->input, this->dummy_space, (i > 10000) ? 10000 : i);
+    }
 
   } else if ((stream_id & 0xe0) == 0xc0) {
     int track = stream_id & 0x1f;
@@ -325,7 +422,7 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     if ((flags & 0x80) == 0x80) {
 
       w = read_bytes(this, 1);
-      pts = (w & 0x0e) << 29 ;
+      pts = (int64_t)(w & 0x0e) << 29 ;
       w = read_bytes(this, 2);
       pts |= (w & 0xFFFE) << 14;
       w = read_bytes(this, 2);
@@ -334,33 +431,41 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
       header_len -= 5 ;
     }
 
-    /* read rest of header */
     i = this->input->read (this->input, this->dummy_space, header_len);
 
-    if(this->audio_fifo)
-      buf = this->input->read_block (this->input, this->audio_fifo, len);
-    else {
-      this->input->read (this->input, this->dummy_space, len);
-      return;
+    for (i = len; i > 0; i -= (this->audio_fifo) 
+	   ? this->audio_fifo->buffer_pool_buf_size : this->video_fifo->buffer_pool_buf_size) {
+      if(this->audio_fifo) {
+	buf = this->input->read_block (this->input, this->audio_fifo,
+	  (i > this->audio_fifo->buffer_pool_buf_size) ? this->audio_fifo->buffer_pool_buf_size : i);
+
+	if (buf == NULL) {
+	  this->status = DEMUX_FINISHED;
+	  return;
+	}
+
+	buf->type      = BUF_AUDIO_MPEG + track;
+	buf->pts       = pts;
+	check_newpts( this, pts, PTS_AUDIO );
+	pts = 0;
+
+	if (this->preview_mode)
+	  buf->decoder_flags = BUF_FLAG_PREVIEW;
+
+        if( this->input->get_length (this->input) )
+          buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                          65535 / this->input->get_length (this->input) );
+        if (this->rate)
+          buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
+                                                * 1000 / (this->rate * 50));
+
+	this->audio_fifo->put (this->audio_fifo, buf);
+
+      } else
+	this->input->read (this->input, this->dummy_space, i);
+
     }
 
-    if (buf == NULL) {
-      this->status = DEMUX_FINISHED;
-      return ;
-    }
-
-    buf->type      = BUF_AUDIO_MPEG + track;
-    buf->pts       = pts;
-    check_newpts( this, pts, PTS_AUDIO );
-
-    if (this->preview_mode)
-      buf->decoder_flags = BUF_FLAG_PREVIEW;
-    
-    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
-
-    if(this->audio_fifo)
-      this->audio_fifo->put (this->audio_fifo, buf);
-
   } else if ((stream_id >= 0xbc) && ((stream_id & 0xf0) == 0xe0)) {
 
     w = read_bytes(this, 1);
@@ -370,11 +475,12 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     len -= header_len + 3;
 
     pts = 0;
+    dts = 0;
 
     if ((flags & 0x80) == 0x80) {
 
       w = read_bytes(this, 1);
-      pts = (w & 0x0e) << 29 ;
+      pts = (int64_t)(w & 0x0e) << 29 ;
       w = read_bytes(this, 2);
       pts |= (w & 0xFFFE) << 14;
       w = read_bytes(this, 2);
@@ -382,33 +488,56 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
 
       header_len -= 5 ;
     }
+    
+    if ((flags & 0x40) == 0x40) {
+    
+      w = read_bytes(this, 1);
+      dts = (int64_t)(w & 0x0e) << 29 ;
+      w = read_bytes(this, 2);
+      dts |= (w & 0xFFFE) << 14;
+      w = read_bytes(this, 2);
+      dts |= (w & 0xFFFE) >> 1;
+      
+      header_len -= 5 ;
+    } 
 
     /* read rest of header */
     i = this->input->read (this->input, this->dummy_space, header_len);
 
     /* contents */
 
-    buf = this->input->read_block (this->input, this->video_fifo, len);
+    for (i = len; i > 0; i -= this->video_fifo->buffer_pool_buf_size) {
+      buf = this->input->read_block (this->input, this->video_fifo,
+	(i > this->video_fifo->buffer_pool_buf_size) ? this->video_fifo->buffer_pool_buf_size : i);
 
-    if (buf == NULL) {
-      this->status = DEMUX_FINISHED;
-      return ;
+      if (buf == NULL) {
+	this->status = DEMUX_FINISHED;
+	return;
+      }
+
+      buf->type = BUF_VIDEO_MPEG;
+      buf->pts  = pts;
+      buf->decoder_info[0] = pts - dts;
+      check_newpts( this, pts, PTS_VIDEO );
+      pts = 0;
+
+      if (this->preview_mode)
+	buf->decoder_flags = BUF_FLAG_PREVIEW;
+
+      if( this->input->get_length (this->input) )
+        buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                         65535 / this->input->get_length (this->input) );
+      if (this->rate)
+        buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
+                                              * 1000 / (this->rate * 50));
+
+      this->video_fifo->put (this->video_fifo, buf);
     }
-    buf->type = BUF_VIDEO_MPEG;
-    buf->pts  = pts;
-    check_newpts( this, pts, PTS_VIDEO );
-
-    if (this->preview_mode)
-      buf->decoder_flags = BUF_FLAG_PREVIEW;
-    
-    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
-
-    this->video_fifo->put (this->video_fifo, buf);
 
   } else {
 
-    i = this->input->read (this->input, this->dummy_space, len);
-    /* (*this->input->seek) (len,SEEK_CUR); */
+    for (i = len; i > 0; i -= 10000) 
+      this->input->read (this->input, this->dummy_space, (i > 10000) ? 10000 : i);
   }
 
 }
@@ -418,12 +547,13 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
   int             len;
   uint32_t        w;
   int             i;
-  int64_t         pts;
+  int64_t         pts, dts;
   buf_element_t  *buf = NULL;
 
   len = read_bytes(this, 2);
 
   pts=0;
+  dts=0;
 
   if (stream_id != 0xbf) {
 
@@ -453,7 +583,7 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
       if (this->status != DEMUX_OK)
         return;
 
-      pts = (w & 0xe) << 29 ;
+      pts = (int64_t)(w & 0xe) << 29 ;
       w = read_bytes(this, 2); len -= 2;
 
       pts |= (w & 0xFFFE) << 14;
@@ -468,103 +598,116 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
       if (this->status != DEMUX_OK)
         return;
 
-      pts = (w & 0x0e) << 29 ;
+      pts = (int64_t)(w & 0x0e) << 29 ;
       w = read_bytes(this, 2); len -= 2;
 
       pts |= (w & 0xFFFE) << 14;
 
       w = read_bytes(this, 2); len -= 2;
-
+      
       pts |= (w & 0xFFFE) >> 1;
 
-/*       printf ("pts2=%lld\n",pts); */
-
-      /* Decoding Time Stamp */
-      w = read_bytes(this, 3); len -= 3;
+      w = read_bytes(this, 1); len -= 1;
+      dts = (int64_t)(w & 0x0e) << 29 ;
       w = read_bytes(this, 2); len -= 2;
+      dts |= (w & 0xFFFE) << 14;
+      w = read_bytes(this, 2); len -= 2;
+      dts |= (w & 0xFFFE) >> 1;
+
     } else {
 
       /*
       if (w != 0x0f)
-        xprintf (VERBOSE|DEMUX, " ERROR w (%02x) != 0x0F ",w);
+        lprintf("ERROR w (%02x) != 0x0F ",w);
       */
     }
 
   }
 
   if (pts && !this->has_pts) {
-#ifdef LOG
-    printf("demux_mpeg: this stream has pts\n");
-#endif
+    lprintf("this stream has pts\n");
     this->has_pts = 1;
   } else if (scr && !this->has_pts) {
-#ifdef LOG
-    printf("demux_mpeg: use scr\n");
-#endif
+    lprintf("use scr\n");
     pts = scr;
   }
   
   if ((stream_id & 0xe0) == 0xc0) {
     int track = stream_id & 0x1f;
 
-    if(this->audio_fifo) {
-      buf = this->input->read_block (this->input, this->audio_fifo, len);
-    } else {
-      this->input->read (this->input, this->dummy_space, len);
-      return;
+    for (i = len; i > 0; i -= (this->audio_fifo) 
+	   ? this->audio_fifo->buffer_pool_buf_size : this->video_fifo->buffer_pool_buf_size) {
+      if(this->audio_fifo) {
+	buf = this->input->read_block (this->input, this->audio_fifo,
+	  (i > this->audio_fifo->buffer_pool_buf_size) ? this->audio_fifo->buffer_pool_buf_size : i);
+
+	if (buf == NULL) {
+	  this->status = DEMUX_FINISHED;
+	  return;
+	}
+
+	buf->type      = BUF_AUDIO_MPEG + track ;
+	buf->pts       = pts;
+	check_newpts( this, pts, PTS_AUDIO );
+	pts = 0;
+
+	if (this->preview_mode)
+	  buf->decoder_flags = BUF_FLAG_PREVIEW;
+
+        if( this->input->get_length (this->input) )
+          buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                          65535 / this->input->get_length (this->input) );
+        if (this->rate)
+          buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
+                                                * 1000 / (this->rate * 50));
+
+	this->audio_fifo->put (this->audio_fifo, buf);
+
+      } else
+	this->input->read (this->input, this->dummy_space, i);
+
     }
 
-    if (buf == NULL) {
-      this->status = DEMUX_FINISHED;
-      return ;
-    }
-    buf->type      = BUF_AUDIO_MPEG + track ;
-    buf->pts       = pts;
-    
-    check_newpts( this, pts, PTS_AUDIO );
-
-    if (this->preview_mode)
-      buf->decoder_flags = BUF_FLAG_PREVIEW;
-    
-    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
-    if (this->rate)
-      buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
-                                          * 1000 / (this->rate * 50));
-
-    if(this->audio_fifo)
-      this->audio_fifo->put (this->audio_fifo, buf);
-
   } else if ((stream_id & 0xf0) == 0xe0) {
 
-    buf = this->input->read_block (this->input, this->video_fifo, len);
+    for (i = len; i > 0; i -= this->video_fifo->buffer_pool_buf_size) {
+      buf = this->input->read_block (this->input, this->video_fifo,
+	(i > this->video_fifo->buffer_pool_buf_size) ? this->video_fifo->buffer_pool_buf_size : i);
 
-    if (buf == NULL) {
-      this->status = DEMUX_FINISHED;
-      return ;
+      if (buf == NULL) {
+	this->status = DEMUX_FINISHED;
+	return;
+      }
+
+      buf->type = BUF_VIDEO_MPEG;
+      buf->pts  = pts;
+      buf->decoder_info[0] = pts - dts;
+      check_newpts( this, pts, PTS_VIDEO );
+      pts = 0;
+
+      if (this->preview_mode)
+	buf->decoder_flags = BUF_FLAG_PREVIEW;
+
+      if( this->input->get_length (this->input) )
+        buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                         65535 / this->input->get_length (this->input) );
+      if (this->rate)
+        buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
+                                              * 1000 / (this->rate * 50));
+
+      this->video_fifo->put (this->video_fifo, buf);
     }
-    buf->type = BUF_VIDEO_MPEG;
-    buf->pts  = pts;
-    
-    check_newpts( this, pts, PTS_VIDEO );
-
-    if (this->preview_mode)
-      buf->decoder_flags = BUF_FLAG_PREVIEW;
-    
-    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
-    if (this->rate)
-      buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
-                                          * 1000 / (this->rate * 50));
-
-    this->video_fifo->put (this->video_fifo, buf);
 
   } else if (stream_id == 0xbd) {
 
-    i = this->input->read (this->input, this->dummy_space, len);
+    for (i = len; i > 0; i -= 10000) 
+      this->input->read (this->input, this->dummy_space, (i > 10000) ? 10000 : i);
+
   } else {
 
-    this->input->read (this->input, this->dummy_space, len);
+    for (i = len; i > 0; i -= 10000) 
+      this->input->read (this->input, this->dummy_space, (i > 10000) ? 10000 : i);
   }
-
 }
 
 static uint32_t parse_pack(demux_mpeg_t *this) {
@@ -576,7 +719,7 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
 
   buf = read_bytes (this, 1);
 
-  if ((buf>>4) == 4) {
+  if ((buf>>6) == 0x01) {
 
     int stuffing, i;
 
@@ -584,8 +727,8 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
 
     /* system_clock_reference */
 
-    scr  = (buf & 0x08) << 27;
-    scr  = (buf & 0x03) << 28;
+    scr  = (int64_t)(buf & 0x38) << 27;
+    scr |= (buf & 0x03) << 28;
     buf  = read_bytes (this, 1);
     scr |= buf << 20;
     buf  = read_bytes (this, 1);
@@ -599,9 +742,15 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
 
     /* mux_rate */
 
-    buf = read_bytes(this,3);
     if (!this->rate) {
-      this->rate = (buf & 0xFFFFFC) >> 2;
+      buf  = read_bytes (this, 1);
+      this->rate = (buf << 14);
+      buf  = read_bytes (this, 1);
+      this->rate |= (buf << 6);
+      buf  = read_bytes (this, 1);
+      this->rate |= (buf >> 2);
+    } else {
+      read_bytes(this,3);
     }
 
     /* stuffing bytes */
@@ -616,7 +765,7 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
 
      /* system_clock_reference */
 
-     scr = (buf & 0x2) << 30;
+     scr = (int64_t)(buf & 0x2) << 30;
      buf = read_bytes (this, 2);
      scr |= (buf & 0xFFFE) << 14;
      buf = read_bytes (this, 2);
@@ -632,7 +781,7 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
        buf = read_bytes (this,1);
        this->rate |= (buf >> 1);
 
-       /* printf ("demux_mpeg: mux_rate = %d\n",this->rate);  */
+       lprintf ("mux_rate = %d\n",this->rate);
 
      } else
        buf = read_bytes (this, 3) ;
@@ -642,7 +791,7 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
 
   buf = read_bytes (this, 4) ;
 
-  /* printf ("  code = %08x\n",buf);*/
+  /* lprintf("code = %08x\n",buf);*/
 
   if (buf == 0x000001bb) {
     buf = read_bytes (this, 2);
@@ -652,7 +801,7 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
     buf = read_bytes (this, 4) ;
   }
 
-  /* printf ("  code = %08x\n",buf); */
+  /* lprintf("code = %08x\n",buf); */
 
   while ( ((buf & 0xFFFFFF00) == 0x00000100)
           && ((buf & 0xff) != 0xba) ) {
@@ -673,16 +822,15 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
 
 }
 
-static uint32_t parse_pack_preview (demux_mpeg_t *this, int *num_buffers)
-{
+static uint32_t parse_pack_preview (demux_mpeg_t *this, int *num_buffers) {
   uint32_t buf ;
   int mpeg_version;
 
   /* system_clock_reference */
   buf = read_bytes (this, 1);
 
-  if ((buf>>4) == 4) {
-     buf = read_bytes(this, 2);
+  if ((buf>>6) == 0x01) {
+     buf = read_bytes(this, 1);
      mpeg_version = 2;
   } else {
      mpeg_version = 1;
@@ -693,18 +841,38 @@ static uint32_t parse_pack_preview (demux_mpeg_t *this, int *num_buffers)
   /* mux_rate */
 
   if (!this->rate) {
-    buf = read_bytes (this,1);
-    this->rate = (buf & 0x7F) << 15;
-    buf = read_bytes (this,1);
-    this->rate |= (buf << 7);
-    buf = read_bytes (this,1);
-    this->rate |= (buf >> 1);
-
-    /* printf ("demux_mpeg: mux_rate = %d\n",this->rate); */
+    if (mpeg_version == 2) {
+      buf  = read_bytes (this, 1);
+      this->rate = (buf << 14);
+      buf  = read_bytes (this, 1);
+      this->rate |= (buf << 6);
+      this->rate |= (buf >> 2);
+      read_bytes (this, 1);
+    } else {
+      buf = read_bytes (this,1);
+      this->rate = (buf & 0x7F) << 15;
+      buf = read_bytes (this,1);
+      this->rate |= (buf << 7);
+      buf = read_bytes (this,1);
+      this->rate |= (buf >> 1);
+    }
+    /* lprintf("mux_rate = %d\n",this->rate); */
 
   } else
     buf = read_bytes (this, 3) ;
 
+  if( mpeg_version == 2 )
+  {
+    int i, stuffing;
+
+    buf = read_bytes(this,1);
+
+    /* stuffing bytes */
+    stuffing = buf &0x03;
+    for (i=0; i<stuffing; i++)
+      read_bytes (this, 1);
+  }
+
   /* system header */
 
   buf = read_bytes (this, 4) ;
@@ -737,14 +905,37 @@ static uint32_t parse_pack_preview (demux_mpeg_t *this, int *num_buffers)
 
 static void demux_mpeg_resync (demux_mpeg_t *this, uint32_t buf) {
 
-  while ((buf !=0x000001ba) && (this->status == DEMUX_OK)) {
+  if (INPUT_IS_SEEKABLE(this->input)) {
+    uint8_t dummy_buf[4096];
+    off_t len, pos;
 
-    buf = (buf << 8) | read_bytes (this, 1);
+    /* fast resync, read 4K block at once */
+    pos = 0;
+    len = 0;
+    while ((buf != 0x000001ba) && (this->status == DEMUX_OK)) {
+      if (pos == len) {
+	len = this->input->read(this->input, dummy_buf, sizeof(dummy_buf));
+        pos = 0;
+        if (len == 0) {
+          this->status = DEMUX_FINISHED;
+	  break;
+	}
+      }
+      buf = (buf << 8) | dummy_buf[pos];
+      pos++;
+    }
+    /* seek back to the pos of the 0x00001ba tag */
+    this->input->seek(this->input, pos-len, SEEK_CUR);
+
+  } else {
+    /* slow resync */
+    while ((buf !=0x000001ba) && (this->status == DEMUX_OK)) {
+      buf = (buf << 8) | read_bytes (this, 1);
+    }
   }
 }
 
 static int demux_mpeg_send_chunk (demux_plugin_t *this_gen) {
-
   demux_mpeg_t *this = (demux_mpeg_t *) this_gen;
 
   uint32_t w=0;
@@ -775,14 +966,14 @@ static void demux_mpeg_send_headers (demux_plugin_t *this_gen) {
   this->last_pts[0]   = 0;
   this->last_pts[1]   = 0;
   
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /*
    * send preview buffers for stream/meta_info
    */
   
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
 
   this->preview_mode = 1;
     
@@ -803,18 +994,24 @@ static void demux_mpeg_send_headers (demux_plugin_t *this_gen) {
     
   this->status = DEMUX_OK ;
 
-  this->stream->stream_info[XINE_STREAM_INFO_BITRATE] = this->rate * 50 * 8;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_BITRATE, this->rate * 50 * 8);
 }
 
 static int demux_mpeg_seek (demux_plugin_t *this_gen,
-			     off_t start_pos, int start_time) {
+			     off_t start_pos, int start_time, int playing) {
 
   demux_mpeg_t   *this = (demux_mpeg_t *) this_gen;
+  start_time /= 1000;
+  start_pos = (off_t) ( (double) start_pos / 65535 *
+              this->input->get_length (this->input) );
 
-  if ((this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE) != 0 ) {
+  if (INPUT_IS_SEEKABLE(this->input)) {
 
-    if ( (!start_pos) && (start_time))
-      start_pos = start_time * this->rate * 50;
+    if ( (!start_pos) && (start_time)) {
+      start_pos = start_time;
+      start_pos *= this->rate;
+      start_pos *= 50;
+    }
 
     this->input->seek (this->input, start_pos+4, SEEK_SET);
 
@@ -827,13 +1024,12 @@ static int demux_mpeg_seek (demux_plugin_t *this_gen,
   this->send_newpts = 1;
   this->status = DEMUX_OK ;
 
-  if( !this->stream->demux_thread_running ) {
+  if( !playing ) {
     this->preview_mode = 0;
     this->buf_flag_seek = 0;
-  }
-  else {
+  } else {
     this->buf_flag_seek = 1;
-    xine_demux_flush_engine(this->stream);
+    _x_demux_flush_engine(this->stream);
   }
 
   return this->status;
@@ -845,11 +1041,10 @@ static void demux_mpeg_dispose (demux_plugin_t *this_gen) {
 }
 
 static int demux_mpeg_get_stream_length (demux_plugin_t *this_gen) {
-
   demux_mpeg_t *this = (demux_mpeg_t *) this_gen;
 
   if (this->rate)
-    return (int)((int64_t) 1000 * this->input->get_length (this->input) / 
+    return (int)((int64_t) 1000 * this->input->get_length (this->input) /
                  (this->rate * 50));
   else
     return 0;
@@ -879,8 +1074,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_mpeg_dispose;
   this->demux_plugin.get_status        = demux_mpeg_get_status;
   this->demux_plugin.get_stream_length = demux_mpeg_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_mpeg_get_capabilities;
   this->demux_plugin.get_optional_data = demux_mpeg_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -891,82 +1084,44 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   switch (stream->content_detection_method) {
 
   case METHOD_BY_CONTENT: {
-    uint8_t buf[4096];
     off_t mdat_atom_offset = -1;
     int64_t mdat_atom_size = -1;
     unsigned int fourcc_tag;
-    int i, j;
+    int i, j, read;
     int ok = 0;
+    uint8_t buf[SCRATCH_SIZE];
 
+    /* use demux_mpeg_block for block devices */
     if (input->get_capabilities(input) & INPUT_CAP_BLOCK ) {
       free (this);
       return NULL;
     }
 
-    if ((input->get_capabilities(input) & INPUT_CAP_SEEKABLE) == 0) {
- 
-      /* try preview */
-
-      if ((input->get_capabilities(input) & INPUT_CAP_PREVIEW) == 0) {
-	free (this);
-	return NULL;
-      }
-
-      input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW);
-
-#ifdef LOG
-      printf ("demux_mpeg: %02x %02x %02x %02x\n",
-	      buf[0], buf[1], buf[2], buf[3]);
-#endif      
-
-      /*
-       * look for mpeg header
-       */
-      
-      if (!buf[0] && !buf[1] && (buf[2] == 0x01) 
-	  && (buf[3] == 0xba)) /* if so, take it */
-	break;
-
+    /* look for mpeg header */
+    read = _x_demux_read_header(input, buf, SCRATCH_SIZE);
+    if (!read) {
       free (this);
       return NULL;
     }
 
-    input->seek(input, 0, SEEK_SET);
-    if (input->read(input, buf, 16) == 16) {
-
-      if(!buf[0] && !buf[1] && (buf[2] == 0x01))
-
-	switch(buf[3]) {
-	case 0xba:
-          if((buf[4] & 0xf0) == 0x20) {
-            uint32_t pckbuf ;
-
-	    pckbuf = read_bytes (this, 1);
-	    if ((pckbuf>>4) != 4) {
-	      ok = 1;
-	      break;
-	    }
-	  }
-	  break;
-#if 0
-	case 0xe0:
-	  if((buf[6] & 0xc0) != 0x80) {
-	    uint32_t pckbuf ;
-
-	    pckbuf = read_bytes (this, 1);
-	    if ((pckbuf>>4) != 4) {
-	      ok = 1;
-	      break;
-	    }
-	  }
-	  break;
-#endif
-        }
+    for (i = 0; i < read - 4; i++) {
+      lprintf ("%02x %02x %02x %02x\n", buf[i], buf[i+1], buf[i+2], buf[i+3]);
+      if (!buf[i] && !buf[i+1] && (buf[i+2] == 0x01)
+          && (buf[i+3] == 0xba)) /* if so, take it */ {
+        ok = 1;
+        break;
+      }
     }
 
-    if (ok)
+    if (ok == 1)
       break;
 
+    /* the special cases need seeking */
+    if (!INPUT_IS_SEEKABLE(input)) {
+      free (this);
+      return NULL;
+    }
+
     /* special case for MPEG streams hidden inside QT files; check
      * is there is an mdat atom  */
     find_mdat_atom(input, &mdat_atom_offset, &mdat_atom_size);
@@ -979,39 +1134,40 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 	input->seek(input, mdat_atom_offset + 8, SEEK_SET);
 
       /* go through the same MPEG detection song and dance */
-      if (input->read(input, buf, 6)) {
-	if (!buf[0] && !buf[1] && buf[2] == 0x01) {
-	  switch (buf[3]) {
-	  case 0xba:
-	    if ((buf[4] & 0xf0) == 0x20) {
-	      uint32_t pckbuf ;
+      if (input->read(input, buf, 4) == 4) {
 
-	      pckbuf = read_bytes (this, 1);
-	      if ((pckbuf>>4) != 4) {
-		ok = 1;
-	      }
-	    }
-	    break;
-	  }
-	}
-      }
-      if (ok)
+        if (!buf[0] && !buf[1] && (buf[2] == 0x01) 
+	    && (buf[3] == 0xba)) /* if so, take it */
 	  break;
+      }
 
       free (this);
       return NULL;
     }
+    
+    /* reset position for next check */
+    if (input->seek(input, 0, SEEK_SET) != 0) {
+      free (this);
+      return NULL;
+    }
 
     /* special case for MPEG streams with a RIFF header */
     fourcc_tag = BE_32(&buf[0]);
     if (fourcc_tag == RIFF_TAG) {
-      fourcc_tag = BE_32(&buf[8]);
+      uint8_t large_buf[1024];
+      
+      if (input->read(input, large_buf, 12) != 12) {
+        free(this);
+        return NULL;
+      }
+      fourcc_tag = BE_32(&large_buf[8]);
       /* disregard the RIFF file if it is certainly a better known
        * format like AVI or WAVE */
       if ((fourcc_tag == WAVE_TAG) ||
-	  (fourcc_tag == AVI_TAG)) {
+	  (fourcc_tag == AVI_TAG) ||
+	  (fourcc_tag == FOURXM_TAG)) {
 	free (this);
-	return DEMUX_CANNOT_HANDLE;
+	return NULL;
       }
 
       /* Iterate through first n kilobytes of RIFF file searching for
@@ -1021,10 +1177,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
        * be extra lazy and do not bother skipping over the data 
        * header. */
       for (i = 0; i < RIFF_CHECK_KILOBYTES && !ok; i++) {
-	if (input->read(input, buf, 1024) != 1024)
+	if (input->read(input, large_buf, 1024) != 1024)
 	  break;
 	for (j = 0; j < 1024 - 4; j++) {
-	  if (BE_32(&buf[j]) == MPEG_MARKER) {
+	  if (BE_32(&large_buf[j]) == MPEG_MARKER) {
 	    ok = 1;
 	    break;
 	  }
@@ -1038,24 +1194,17 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   }
 
   case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
+    char *extensions, *mrl;
 
     mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
 
-    ending = strrchr(mrl, '.');
-    
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp(ending, ".MPEG", 5)
-	&& strncasecmp (ending, ".mpg", 4)) {
+    if (!_x_demux_check_extension (mrl, extensions)) {
       free (this);
       return NULL;
     }
   }
-    break;
+  break;
 
   case METHOD_EXPLICIT:
     break;
@@ -1086,19 +1235,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_mpeg_class_t *this = (demux_mpeg_class_t *) this_gen;
 
   free (this);
  }
 
 static void *init_plugin (xine_t *xine, void *data) {
-
   demux_mpeg_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_mpeg_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_mpeg_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -1113,9 +1258,12 @@ static void *init_plugin (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
+demuxer_info_t demux_info_mpeg = {
+  9                        /* priority */
+};
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "mpeg", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 25, "mpeg", XINE_VERSION_CODE, &demux_info_mpeg, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
