@@ -1,5 +1,6 @@
-/* 
- * Copyright (C) 2003 the xine project
+/*
+ * Copyright (C) 2003-2004 the xine project
+ * Copyright (C) 2003 Jeroen Asselman <j.asselman@itsec-ps.nl>
  * 
  * This file is part of xine, a free video player.
  * 
@@ -16,11 +17,12 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_yuv_frames.c,v 1.3 2003/01/26 15:56:21 tmmm Exp $
+ */
+
+/*
+ * $Id: demux_yuv_frames.c,v 1.18 2004/10/14 23:02:29 miguelfreitas Exp $
  *
  * dummy demultiplexer for raw yuv frames (delivered by v4l)
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,36 +35,33 @@
 #include <string.h>
 #include <stdlib.h>
 
+#define LOG_MODULE "demux_yuv_frames"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "demux.h"
 
-/*
-#define LOG
-*/
+#define WRAP_THRESHOLD       20000
 
 typedef struct demux_yuv_frames_s {
   demux_plugin_t        demux_plugin;
 
   xine_stream_t        *stream;
-  
   fifo_buffer_t        *audio_fifo;
   fifo_buffer_t        *video_fifo;
-
   input_plugin_t       *input;
-
   int                   status;
-  
+
+  int                   seek_flag;
+  int64_t               last_pts;
 } demux_yuv_frames_t ;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_yuv_frames_class_t;
 
 
@@ -72,57 +71,89 @@ static int demux_yuv_frames_get_status (demux_plugin_t *this_gen) {
   return this->status;
 }
 
-static int demux_yuv_frames_send_chunk (demux_plugin_t *this_gen) {
+static int switch_buf(demux_yuv_frames_t *this , buf_element_t *buf){
+  int result = 0;
 
+  if (!buf)
+    return 0;
+
+  if (this->seek_flag) {
+    this->seek_flag = 0;
+    _x_demux_control_newpts(this->stream, buf->pts, BUF_FLAG_SEEK);
+  } else if (abs(this->last_pts - buf->pts) > WRAP_THRESHOLD) {
+    _x_demux_control_newpts(this->stream, buf->pts, 0);
+  }
+
+  this->last_pts = buf->pts;
+
+  switch (buf->type) {
+    case BUF_VIDEO_I420:
+    case BUF_VIDEO_YUY2:
+      this->video_fifo->put(this->video_fifo, buf);
+      result = 1;	/* 1, we still should read audio */
+      break;
+    case BUF_AUDIO_LPCM_LE:
+      if (!_x_stream_info_get(this->stream, XINE_STREAM_INFO_HAS_VIDEO))
+        _x_demux_control_newpts(this->stream, buf->pts, 0);
+      this->audio_fifo->put(this->audio_fifo, buf);
+      break;
+    default:
+      lprintf ("dhelp, unknown buffer type %08x\n", buf->type);
+      buf->free_buffer(buf);
+  }
+
+  return result;
+}
+
+static int demux_yuv_frames_send_chunk (demux_plugin_t *this_gen){
   demux_yuv_frames_t *this = (demux_yuv_frames_t *) this_gen;
   buf_element_t      *buf;
 
-  buf = this->input->read_block (this->input, NULL, 0);
-
-  if (!buf)
-    this->status = DEMUX_FINISHED;
-  else {
-
-    switch (buf->type) {
-    case BUF_VIDEO_YUV_FRAMES:
-      this->video_fifo->put (this->video_fifo, buf);
-      break;
-    default:
-      printf ("demux_yuv_frames: help, unknown buffer type %08x\n",
-	      buf->type);
-
-      buf->free_buffer (buf);
-    }
-  }
+  do {
+    if ( _x_stream_info_get(this->stream, XINE_STREAM_INFO_HAS_VIDEO) )
+      buf = this->input->read_block (this->input, this->video_fifo, 0);
+    else
+      buf = this->input->read_block (this->input, this->audio_fifo, 0);
+  } while (switch_buf(this, buf));
 
   return this->status;
 }
 
-static void demux_yuv_frames_send_headers (demux_plugin_t *this_gen) {
-
+static void demux_yuv_frames_send_headers (demux_plugin_t *this_gen){
   demux_yuv_frames_t *this = (demux_yuv_frames_t *) this_gen;
+  buf_element_t      *buf;
 
   this->video_fifo  = this->stream->video_fifo;
   this->audio_fifo  = this->stream->audio_fifo;
+ 
+  _x_demux_control_start(this->stream);
 
+  if(_x_stream_info_get(this->stream, XINE_STREAM_INFO_HAS_AUDIO)) {
+    buf = this->input->read_block(this->input, this->audio_fifo, 0);
+    
+    this->audio_fifo->put(this->audio_fifo, buf);
+  }
+  
+  if(_x_stream_info_get(this->stream, XINE_STREAM_INFO_HAS_VIDEO)) {
+    buf = this->input->read_block(this->input, this->video_fifo, 0);
+    
+    this->video_fifo->put(this->video_fifo, buf);
+  }
+  
   this->status = DEMUX_OK;
-
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
 }
 
 static int demux_yuv_frames_seek (demux_plugin_t *this_gen,
-			    off_t start_pos, int start_time) {
+			    off_t start_pos, int start_time, int playing) {
+  demux_yuv_frames_t *this = (demux_yuv_frames_t *) this_gen;
 
-  demux_yuv_frames_t *this = (demux_yuv_frames_t *) this_gen; 
-  
+  this->seek_flag = 1;
+  this->last_pts = 0;
   return this->status;
 }
 
 static int demux_yuv_frames_get_stream_length (demux_plugin_t *this_gen) {
-
   /* demux_yuv_frames_t *this = (demux_yuv_frames_t *) this_gen;  */
-
   return 0;
 }
 
@@ -136,16 +167,15 @@ static int demux_yuv_frames_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static void demux_yuv_frames_dispose (demux_plugin_t *this_gen) {
-
-  demux_yuv_frames_t *this = (demux_yuv_frames_t *) this_gen;  
+  demux_yuv_frames_t *this = (demux_yuv_frames_t *) this_gen;
 
   free (this);
 }
 
-static demux_plugin_t *open_plugin (demux_class_t *class_gen, 
-				    xine_stream_t *stream, 
+static demux_plugin_t *open_plugin (demux_class_t *class_gen,
+				    xine_stream_t *stream,
 				    input_plugin_t *input) {
-  
+
   demux_yuv_frames_t *this;
 
   switch (stream->content_detection_method) {
@@ -155,9 +185,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
     break;
 
   case METHOD_BY_EXTENSION: {
-
     char *mrl;
-    
+
     mrl = input->get_mrl (input);
 
     if (strncmp (mrl, "v4l:/", 5))
@@ -172,8 +201,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   default:
     return NULL;
   }
-
-  printf ("demux_yuv_frames: input accepted.\n");
+  lprintf ("input accepted.\n");
 
   /*
    * if we reach this point, the input has been accepted.
@@ -189,27 +217,25 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   this->demux_plugin.dispose           = demux_yuv_frames_dispose;
   this->demux_plugin.get_status        = demux_yuv_frames_get_status;
   this->demux_plugin.get_stream_length = demux_yuv_frames_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_yuv_frames_get_capabilities;
   this->demux_plugin.get_optional_data = demux_yuv_frames_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
-  
+
   this->status = DEMUX_FINISHED;
-  
+
   return &this->demux_plugin;
 }
 
 
 
 /*
- * ogg demuxer class
+ * demuxer class
  */
 
 static char *get_description (demux_class_t *this_gen) {
   return "YUV frames dummy demux plugin";
 }
- 
+
 static char *get_identifier (demux_class_t *this_gen) {
   return "YUV_FRAMES";
 }
@@ -223,19 +249,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_yuv_frames_class_t *this = (demux_yuv_frames_class_t *) this_gen;
 
   free (this);
 }
 
 static void *init_class (xine_t *xine, void *data) {
-  
   demux_yuv_frames_class_t     *this;
-  
-  this         = xine_xmalloc (sizeof (demux_yuv_frames_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+
+  this = xine_xmalloc (sizeof (demux_yuv_frames_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -250,9 +272,16 @@ static void *init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
+demuxer_info_t demux_info_yuv_frames = {
+  0                       /* priority */
+};
 
 plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "ogg", XINE_VERSION_CODE, NULL, init_class },
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_DEMUX, 25, "yuv_frames", XINE_VERSION_CODE, &demux_info_yuv_frames, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
+
+/*
+ * vim:sw=3:sts=3:
+ */
