@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.74 2003/01/29 18:53:59 miguelfreitas Exp $
+ * $Id: input_file.c,v 1.98 2004/12/20 21:22:20 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +34,12 @@
 #include <string.h>
 #include <errno.h>
 
+#define LOG_MODULE "input_file"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "compat.h"
@@ -41,6 +47,12 @@
 
 #define MAXFILES      65535
 
+#ifndef WIN32
+/* MS needs O_BINARY to open files, for everyone else,
+ * make sure it doesn't get in the way */
+#  define O_BINARY  0
+#endif
+
 typedef struct {
 
   input_class_t     input_class;
@@ -75,6 +87,10 @@ static uint32_t file_plugin_get_capabilities (input_plugin_t *this_gen) {
   if (this->fh <0)
     return 0;
 
+#ifdef _MSC_VER
+    /*return INPUT_CAP_SEEKABLE | INPUT_CAP_GET_DIR;*/
+	return INPUT_CAP_SEEKABLE;
+#else
   if (fstat (this->fh, &buf) == 0) {
     if (S_ISREG(buf.st_mode))
       return INPUT_CAP_SEEKABLE;
@@ -83,6 +99,7 @@ static uint32_t file_plugin_get_capabilities (input_plugin_t *this_gen) {
   } else
     perror ("system call fstat");
   return 0;
+#endif /* _MSC_VER */
 }
 
 
@@ -92,36 +109,25 @@ static off_t file_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
   return read (this->fh, buf, len);
 }
 
-/*
- * helper function to release buffer
- * in case demux thread is cancelled
- */
-static void pool_release_buffer (void *arg) {
-  buf_element_t *buf = (buf_element_t *) arg;
-  if( buf != NULL )
-    buf->free_buffer(buf);
-}
-
 static buf_element_t *file_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
 
   off_t                 num_bytes, total_bytes;
   file_input_plugin_t  *this = (file_input_plugin_t *) this_gen;
   buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
 
-  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
-  pthread_cleanup_push( pool_release_buffer, buf );
-
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
   total_bytes = 0;
 
   while (total_bytes < todo) {
-    pthread_testcancel();
     num_bytes = read (this->fh, buf->mem + total_bytes, todo-total_bytes);
     if (num_bytes <= 0) {
-      if (num_bytes < 0) 
+      if (num_bytes < 0) {
 	xine_log (this->stream->xine, XINE_LOG_MSG,
 		  _("input_file: read error (%s)\n"), strerror(errno));
+	_x_message(this->stream, XINE_MSG_READ_ERROR,
+                     this->mrl, NULL);
+      }
       buf->free_buffer (buf);
       buf = NULL;
       break;
@@ -132,9 +138,6 @@ static buf_element_t *file_plugin_read_block (input_plugin_t *this_gen, fifo_buf
   if( buf != NULL )
     buf->size = total_bytes;
 
-  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
-  pthread_cleanup_pop(0);
-
   return buf;
 }
 
@@ -198,7 +201,8 @@ static int file_plugin_get_optional_data (input_plugin_t *this_gen,
 static void file_plugin_dispose (input_plugin_t *this_gen ) {
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
-  close(this->fh);
+  if (this->fh != -1)
+    close(this->fh);
 
   free (this->mrl);
 
@@ -209,6 +213,8 @@ static char *decode_uri (char *uri) {
 
   int len = strlen (uri);
   int i;
+  
+  uri = strdup(uri);
 
   for (i=0; i<len; i++) {
 
@@ -232,23 +238,71 @@ static char *decode_uri (char *uri) {
   return uri;
 }
 
-static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
+static int file_plugin_open (input_plugin_t *this_gen ) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+  char                *filename;
+
+  lprintf("file_plugin_open\n");
+
+  if (strncasecmp (this->mrl, "file:", 5) == 0)
+  {
+    if (strncasecmp (this->mrl, "file://localhost/", 16) == 0)
+      filename = decode_uri(&(this->mrl[16]));
+    else if (strncasecmp (this->mrl, "file://127.0.0.1/", 16) == 0)
+      filename = decode_uri(&(this->mrl[16]));
+    else
+      filename = decode_uri(&(this->mrl[5]));
+  }
+  else
+    filename = decode_uri(this->mrl);
+
+  this->fh = open (filename, O_RDONLY|O_BINARY);
+  
+  free(filename);
+
+  if (this->fh == -1) {
+    /* try again without unescaping; such MRLs might be invalid,
+     * but we are a nice software */
+    if (strncasecmp (this->mrl, "file:", 5) == 0)
+    {
+      if (strncasecmp (this->mrl, "file://localhost/", 16) == 0)
+        this->fh = open(&this->mrl[16], O_RDONLY|O_BINARY);
+      else if (strncasecmp (this->mrl, "file://127.0.0.1/", 16) == 0)
+        this->fh = open(&this->mrl[16], O_RDONLY|O_BINARY);
+      else
+        this->fh = open(&this->mrl[5], O_RDONLY|O_BINARY);
+    }
+    else
+      this->fh = open(this->mrl, O_RDONLY|O_BINARY);
+    
+    if (this->fh == -1) {
+      if (errno == EACCES) {
+        _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
+	xine_log (this->stream->xine, XINE_LOG_MSG,
+		  _("input_file: Permission denied: >%s<\n"), this->mrl);
+      } else if (errno == ENOENT) {
+        _x_message(this->stream, XINE_MSG_FILE_NOT_FOUND, this->mrl, NULL);
+	xine_log (this->stream->xine, XINE_LOG_MSG,
+		  _("input_file: File not found: >%s<\n"), this->mrl);
+      }
+
+      return 0;
+    }
+  }
+
+  return 1;
+}
+
+static input_plugin_t *file_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, 
 				    const char *data) {
 
   /* file_input_class_t  *cls = (file_input_class_t *) cls_gen; */
   file_input_plugin_t *this;
   char                *mrl = strdup(data);
-  char                *filename;
-  int                  fh;
 
-  if (!strncasecmp (mrl, "file:", 5)) 
-    filename = decode_uri (&mrl[5]);
-  else
-    filename = mrl;
+  lprintf("file_class_get_instance\n");
 
-  fh = open (filename, O_RDONLY);
-
-  if (fh == -1) {
+  if ((strncasecmp (mrl, "file:", 5)) && strstr (mrl, ":/") && (strstr (mrl, ":/") < strchr(mrl, '/'))) {
     free (mrl);
     return NULL;
   }
@@ -256,8 +310,9 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   this = (file_input_plugin_t *) xine_xmalloc (sizeof (file_input_plugin_t));
   this->stream = stream;
   this->mrl    = mrl;
-  this->fh     = fh;
+  this->fh     = -1;
 
+  this->input_plugin.open               = file_plugin_open;
   this->input_plugin.get_capabilities   = file_plugin_get_capabilities;
   this->input_plugin.read               = file_plugin_read;
   this->input_plugin.read_block         = file_plugin_read_block;
@@ -324,7 +379,7 @@ static void origin_change_cb(void *data, xine_cfg_entry_t *cfg) {
 #define CMP          2
 #define LEN          3
 #define ISDIGIT(c)   ((unsigned) (c) - '0' <= 9)
-static int strverscmp(const char *s1, const char *s2) {
+static int _strverscmp(const char *s1, const char *s2) {
   const unsigned char *p1 = (const unsigned char *) s1;
   const unsigned char *p2 = (const unsigned char *) s2;
   unsigned char c1, c2;
@@ -381,10 +436,10 @@ static int strverscmp(const char *s1, const char *s2) {
 }
 
 /*
- * Wrapper to strverscmp() for qsort() calls, which sort mrl_t type array.
+ * Wrapper to _strverscmp() for qsort() calls, which sort mrl_t type array.
  */
 static int _sortfiles_default(const xine_mrl_t *s1, const xine_mrl_t *s2) {
-  return(strverscmp(s1->mrl, s2->mrl));
+  return(_strverscmp(s1->mrl, s2->mrl));
 }
 
 /*
@@ -397,11 +452,9 @@ static uint32_t get_file_type(char *filepathname, char *origin, xine_t *xine) {
   char         buf[XINE_PATH_MAX + XINE_NAME_MAX + 1];
 
   if((lstat(filepathname, &pstat)) < 0) {
-    sprintf(buf, "%s/%s", origin, filepathname);
+    snprintf(buf, sizeof(buf), "%s/%s", origin, filepathname);
     if((lstat(buf, &pstat)) < 0) {
-#ifdef LOG
-      printf ("lstat failed for %s{%s}\n", filepathname, origin);
-#endif
+      lprintf ("lstat failed for %s{%s}\n", filepathname, origin);
       file_type |= mrl_unknown;
       return file_type;
     }
@@ -445,7 +498,7 @@ static off_t get_file_size(char *filepathname, char *origin) {
   char         buf[XINE_PATH_MAX + XINE_NAME_MAX + 1];
 
   if((lstat(filepathname, &pstat)) < 0) {
-    sprintf(buf, "%s/%s", origin, filepathname);
+    snprintf(buf, sizeof(buf), "%s/%s", origin, filepathname);
     if((lstat(buf, &pstat)) < 0)
       return (off_t) 0;
   }
@@ -457,7 +510,7 @@ static char *file_class_get_description (input_class_t *this_gen) {
   return _("file input plugin");
 }
 
-static char *file_class_get_identifier (input_class_t *this_gen) {
+static const char *file_class_get_identifier (input_class_t *this_gen) {
   return "file";
 }
 
@@ -498,12 +551,12 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
   }
 
   /* Store new origin path */
- __try_again_from_home:
+ try_again_from_home:
   
-  this->config->update_string(this->config, "input.file_origin_path", current_dir);
+  this->config->update_string(this->config, "media.files.origin_path", current_dir);
 
   if(strcasecmp(current_dir, "/"))
-    sprintf(current_dir_slashed, "%s/", current_dir);
+    snprintf(current_dir_slashed, sizeof(current_dir_slashed), "%s/", current_dir);
   else
     sprintf(current_dir_slashed, "/");
   
@@ -516,7 +569,7 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
       /* Try one more time with user homedir */
       snprintf(current_dir, XINE_PATH_MAX, "%s", xine_get_homedir());
       already_tried++;
-      goto __try_again_from_home;
+      goto try_again_from_home;
     }
 
     return NULL;
@@ -529,7 +582,7 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
   while((pdirent = readdir(pdir)) != NULL) {
     
     memset(fullfilename, 0, sizeof(fullfilename));
-    sprintf(fullfilename, "%s/%s", current_dir, pdirent->d_name);
+    snprintf(fullfilename, sizeof(fullfilename), "%s/%s", current_dir, pdirent->d_name);
     
     if(is_a_dir(fullfilename)) {
       
@@ -560,7 +613,8 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
 	  linksize = readlink(fullfilename, linkbuf, XINE_PATH_MAX + XINE_NAME_MAX);
 	  
 	  if(linksize < 0) 
-	    printf ("input_file: readlink() failed: %s\n", strerror(errno));
+	    xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+		     "input_file: readlink() failed: %s\n", strerror(errno));
 	  else {
 	    dir_files[num_dir_files].link = (char *) xine_xmalloc(linksize + 1);
 	    strncpy(dir_files[num_dir_files].link, linkbuf, linksize);
@@ -597,7 +651,8 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
 	  linksize = readlink(fullfilename, linkbuf, XINE_PATH_MAX + XINE_NAME_MAX);
 	  
 	  if(linksize < 0) {
-	    printf ("input_file: readlink() failed: %s\n", strerror(errno));
+	    xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+		     "input_file: readlink() failed: %s\n", strerror(errno));
 	  }
 	  else {
 	    hide_files[num_hide_files].link = (char *) 
@@ -632,7 +687,8 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
 	linksize = readlink(fullfilename, linkbuf, XINE_PATH_MAX + XINE_NAME_MAX);
 	
 	if(linksize < 0) {
-	  printf ("input_file: readlink() failed: %s\n", strerror(errno));
+	  xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+		   "input_file: readlink() failed: %s\n", strerror(errno));
 	}
 	else {
 	  norm_files[num_norm_files].link = (char *) 
@@ -794,7 +850,7 @@ static void *init_plugin (xine_t *xine, void *data) {
   this->config = xine->config;
   config       = xine->config;
   
-  this->input_class.open_plugin        = open_plugin;
+  this->input_class.get_instance       = file_class_get_instance;
   this->input_class.get_identifier     = file_class_get_identifier;
   this->input_class.get_description    = file_class_get_description;
   this->input_class.get_dir            = file_class_get_dir;
@@ -811,18 +867,20 @@ static void *init_plugin (xine_t *xine, void *data) {
     if(getcwd(current_dir, sizeof(current_dir)) == NULL)
       strcpy(current_dir, ".");
 
-    this->origin_path = config->register_string(config, "input.file_origin_path",
+    this->origin_path = config->register_string(config, "media.files.origin_path",
 						current_dir, 
 						_("file browsing start location"),
-						NULL, 0, origin_change_cb, 
-						(void *) this);
+						_("The browser to select the file to play will "
+						  "start at this location."),
+						0, origin_change_cb, (void *) this);
   }
   
   this->show_hidden_files = config->register_bool(config, 
-						  "input.file_hidden_files", 
-						  1, _("list hidden files"),
-						  NULL, 10, hidden_bool_cb, 
-						  (void *) this);
+						  "media.files.show_hidden_files", 
+						  0, _("list hidden files"),
+						  _("If enabled, the browser to select the file to "
+						    "play will also show hidden files."),
+						  10, hidden_bool_cb, (void *) this);
   
   return this;
 }
@@ -833,7 +891,6 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 11, "file", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_INPUT, 15, "FILE", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
-
