@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 2002 Bastien Nocera <hadess@hadess.net>
  * 
  * This file is part of totem,
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_gnome_vfs.c,v 1.2 2003/01/13 20:10:23 hadess Exp $
+ * $Id: input_gnome_vfs.c,v 1.23 2004/10/30 16:14:33 hadess Exp $
  */
 
 
@@ -28,16 +28,18 @@
 
 #include "xine_internal.h"
 #include "xineutils.h"
-#include "compat.h"
 #include "input_plugin.h"
+#include "net_buf_ctrl.h"
 
 #include <libgnomevfs/gnome-vfs.h>
 
-/* #define D(x...) */
-#define D(x...) g_message (x)
-#define LOG
-
-#define PREVIEW_SIZE 16384
+#ifdef  __GNUC__
+#define D(...)
+#else
+#define D(__VA_ARGS__)
+#endif
+/* #define D(...) g_message (__VA_ARGS__) */
+/* #define LOG */
 
 typedef struct {
 	input_class_t input_class;
@@ -47,16 +49,16 @@ typedef struct {
 typedef struct {
 	input_plugin_t input_plugin;
 	xine_stream_t *stream;
+	nbc_t *nbc;
 
 	/* File */
 	GnomeVFSHandle *fh;
 	off_t curpos;
 	char *mrl;
-	/* Subtitle */
-	GnomeVFSHandle *sub;
+	GnomeVFSURI *uri;
 
 	/* Preview */
-	char preview[PREVIEW_SIZE];
+	char preview[MAX_PREVIEW_SIZE];
 	off_t preview_size;
 	off_t preview_pos;
 } gnomevfs_input_t;
@@ -70,6 +72,8 @@ gnomevfs_plugin_get_capabilities (input_plugin_t *this_gen)
 	return INPUT_CAP_SEEKABLE | INPUT_CAP_SPULANG;
 }
 
+#define SSH_BUFFER_SIZE 256 * 1024
+
 static off_t
 gnomevfs_plugin_read (input_plugin_t *this_gen, char *buf, off_t len)
 {
@@ -85,17 +89,19 @@ gnomevfs_plugin_read (input_plugin_t *this_gen, char *buf, off_t len)
 		GnomeVFSResult res;
 
 		res = gnome_vfs_read (this->fh, &buf[num_bytes],
-				(GnomeVFSFileSize) (len - num_bytes),
+				(GnomeVFSFileSize) MIN (len - num_bytes, SSH_BUFFER_SIZE),
 				(GnomeVFSFileSize *)&n);
 
 		D("gnomevfs_plugin_read: read %ld from gnome-vfs",
 				(long int) n);
-
-		if (res != GNOME_VFS_OK)
+		if (res != GNOME_VFS_OK && res != GNOME_VFS_ERROR_EOF)
 		{
 			D("gnomevfs_plugin_read: gnome_vfs_read returns %s",
 					gnome_vfs_result_to_string (res));
-			return 0;
+			return -1;
+		} else if (res == GNOME_VFS_ERROR_EOF) {
+			D("gnomevfs_plugin_read: GNOME_VFS_ERROR_EOF");
+			return num_bytes;
 		}
 
 		if (n <= 0)
@@ -110,18 +116,6 @@ gnomevfs_plugin_read (input_plugin_t *this_gen, char *buf, off_t len)
 	return num_bytes;
 }
 
-/*
- * helper function to release buffer
- * in case demux thread is cancelled
- */
-static void
-pool_release_buffer (void *arg)
-{
-	buf_element_t *buf = (buf_element_t *) arg;
-	if( buf != NULL )
-		buf->free_buffer(buf);
-}
-
 static buf_element_t*
 gnomevfs_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo,
 		off_t todo)
@@ -129,26 +123,18 @@ gnomevfs_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo,
 	off_t total_bytes;
 	buf_element_t *buf = fifo->buffer_pool_alloc (fifo);
 
-	pthread_setcancelstate (PTHREAD_CANCEL_ENABLE,NULL);
-	pthread_cleanup_push (pool_release_buffer, buf);
-
 	buf->content = buf->mem;
 	buf->type = BUF_DEMUX_BLOCK;
 
 	total_bytes = gnomevfs_plugin_read (this_gen, buf->content, todo);
 
-	while (total_bytes != todo)
+	if (total_bytes == todo) buf->size = todo;
+	else
 	{
 		buf->free_buffer (buf);
 		buf = NULL;
 	}
 
-	if (buf != NULL)
-		buf->size = total_bytes;
-
-	pthread_setcancelstate (PTHREAD_CANCEL_DISABLE, NULL);
-	pthread_cleanup_pop (0);
-
 	return buf;
 }
 
@@ -189,7 +175,8 @@ static off_t
 gnomevfs_plugin_get_length (input_plugin_t *this_gen)
 {
 	gnomevfs_input_t *this = (gnomevfs_input_t *) this_gen;
-	GnomeVFSFileInfo info;
+	GnomeVFSFileInfo *info;
+	off_t length;
 
 	if (this->fh == NULL)
 	{
@@ -197,14 +184,18 @@ gnomevfs_plugin_get_length (input_plugin_t *this_gen)
 		return 0;
 	}
 
-	if (gnome_vfs_get_file_info_from_handle (this->fh,
-				&info,
+	info = gnome_vfs_file_info_new ();
+	if (gnome_vfs_get_file_info (this->mrl, info,
 				GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK)
 	{
-		D ("gnomevfs_plugin_get_length: %d", (int) info.size);
-		return (off_t) info.size;
-	} else
-		return 0;
+		length = info->size;
+		gnome_vfs_file_info_unref (info);
+		D ("gnomevfs_plugin_get_length: %lld", length);
+		return length;
+	}
+
+	gnome_vfs_file_info_unref (info);
+	return 0;
 }
 
 static uint32_t
@@ -213,12 +204,6 @@ gnomevfs_plugin_get_blocksize (input_plugin_t *this_gen)
 	return 0;
 }
 
-static int
-gnomevfs_klass_eject_media (input_class_t *this_gen)
-{
-	return 1; /* doesn't make sense */
-}
-
 static char*
 gnomevfs_plugin_get_mrl (input_plugin_t *this_gen)
 {
@@ -233,7 +218,7 @@ static char
 	return _("gnome-vfs input plugin as shipped with xine");
 }
 
-static char
+static const char
 *gnomevfs_klass_get_identifier (input_class_t *this_gen)
 {
 	return "gnomevfs";
@@ -243,37 +228,8 @@ static int
 gnomevfs_plugin_get_optional_data (input_plugin_t *this_gen, 
 		void *data, int data_type)
 {
-	gnomevfs_input_t *this = (gnomevfs_input_t *) this_gen;
+	D ("input_gnomevfs: get optional data, type %08x\n", data_type);
 
-	D ("input_gnomevfs: get optional data, type %08x, sub %p\n",
-			data_type, this->sub);
-/* FIXME */
-#if 0
-	switch (data_type) {
-	case INPUT_OPTIONAL_DATA_TEXTSPU0:
-		if(this->sub)
-		{
-			GnomeVFSHandle **tmp;
-      
-			/* dirty hacks... */
-			tmp = data;
-			*tmp = this->sub;
-
-			return INPUT_OPTIONAL_SUCCESS;
-		}
-		break;
-	case INPUT_OPTIONAL_DATA_SPULANG:
-		sprintf(data, "%3s", (this->sub) ? "on" : "off");
-		return INPUT_OPTIONAL_SUCCESS;
-		break;
-	case INPUT_OPTIONAL_DATA_PREVIEW:
-		memcpy (data, this->preview, this->preview_size);
-		return this->preview_size;
-	default:
-		return INPUT_OPTIONAL_UNSUPPORTED;
-		break;
-	}
-#endif
 	return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
@@ -282,16 +238,35 @@ gnomevfs_plugin_dispose (input_plugin_t *this_gen )
 {
 	gnomevfs_input_t *this = (gnomevfs_input_t *) this_gen;
 
+	if (this->nbc)
+	{
+		nbc_close (this->nbc);
+		this->nbc = NULL;
+	}
 	if (this->fh)
 		gnome_vfs_close (this->fh);
-	if (this->sub)
-		gnome_vfs_close (this->sub);
 	if (this->mrl)
 		g_free (this->mrl);
-
+	if (this->uri)
+		gnome_vfs_uri_unref (this->uri);
 	g_free (this);
 }
 
+static int
+gnomevfs_plugin_open (input_plugin_t *this_gen )
+{
+	gnomevfs_input_t *this = (gnomevfs_input_t *) this_gen;
+
+	D("gnomevfs_klass_open: opening '%s'", this->mrl);
+	if (gnome_vfs_open_uri (&this->fh, this->uri, GNOME_VFS_OPEN_READ) != GNOME_VFS_OK)
+	{
+		D("gnomevfs_klass_open: failed to open '%s'", this->mrl);
+		return 0;
+	}
+
+	return 1;
+}
+
 static void
 gnomevfs_klass_dispose (input_class_t *this_gen)
 {
@@ -300,62 +275,41 @@ gnomevfs_klass_dispose (input_class_t *this_gen)
 	g_free (this);
 }
 
+static char * ignore_scheme[] = { "cdda", "http", "file" };
+
 static input_plugin_t *
-gnomevfs_klass_open (input_class_t *klass_gen, xine_stream_t *stream,
+gnomevfs_klass_get_instance (input_class_t *klass_gen, xine_stream_t *stream,
 		const char *mrl)
 {
 	gnomevfs_input_t *this;
-	GnomeVFSHandle *fh, *sub;
-	const char *subtitle_file;
-	char *subtitle_path, *subtitle;
 	GnomeVFSURI *uri;
+	int i;
 
-	D("gnomevfs_klass_open: %s", mrl);
+	if (mrl == NULL)
+		return NULL;
+	else if (strstr (mrl, "://") == NULL)
+		return NULL;
+
+	D("gnomevfs_klass_get_instance: %s", mrl);
+
+	for (i = 0; i < G_N_ELEMENTS (ignore_scheme); i++) {
+		if (strncmp (ignore_scheme[i], mrl, strlen (ignore_scheme[i])) == 0)
+				return NULL;
+	}
 
 	uri = gnome_vfs_uri_new (mrl);
 	if (uri == NULL)
 		return NULL;
 
-	/* local files should be handled by the file input */
-	if (gnome_vfs_uri_is_local (uri) == TRUE)
-	{
-		gnome_vfs_uri_unref (uri);
-		return NULL;
-	}
-
-	subtitle_file = gnome_vfs_uri_get_fragment_identifier (uri);
-	if (subtitle_file != NULL)
-	{
-		subtitle_path = gnome_vfs_uri_extract_dirname (uri);
-		subtitle = g_strdup_printf ("%s%s", subtitle_path,
-				subtitle_file);
-		g_free (subtitle_path);
-
-		D("input_file: trying to open subtitle file '%s'\n",
-				subtitle);
-
-		if (gnome_vfs_open (&sub, subtitle, GNOME_VFS_OPEN_READ)
-				!= GNOME_VFS_OK)
-			D("input_file: failed to open subtitle file '%s'\n",
-					subtitle);
-	} else {
-		sub = NULL;
-	}
-
-	if (gnome_vfs_open_uri (&fh, uri, GNOME_VFS_OPEN_READ)
-			!= GNOME_VFS_OK)
-	{
-		if (sub != NULL)
-			gnome_vfs_close (sub);
-		return NULL;
-	}
-
+	D("Creating the structure for stream '%s'", mrl);
 	this = g_new0 (gnomevfs_input_t, 1);
 	this->stream = stream;
-	this->fh = fh;
+	this->fh = NULL;
 	this->mrl = g_strdup (mrl);
-	this->sub = sub;
+	this->uri = uri;
+	this->nbc = nbc_init (this->stream);
 
+	this->input_plugin.open              = gnomevfs_plugin_open;
 	this->input_plugin.get_capabilities  = gnomevfs_plugin_get_capabilities;
 	this->input_plugin.read              = gnomevfs_plugin_read;
 	this->input_plugin.read_block        = gnomevfs_plugin_read_block;
@@ -377,27 +331,32 @@ static void
 {
 	gnomevfs_input_class_t *this;
 
+	xprintf (xine, XINE_VERBOSITY_DEBUG, "gnome_vfs init_input_class\n");
+
 	if (gnome_vfs_initialized () == FALSE)
-		gnome_vfs_init ();
+		if (gnome_vfs_init () == FALSE) {
+			xprintf (xine, XINE_VERBOSITY_DEBUG, "Couldn't initialise gnome-vfs\n");
+			return NULL;
+		}
+
 	if (!g_thread_supported ())
 		g_thread_init (NULL);
 
 	this = g_new0 (gnomevfs_input_class_t, 1);
 	this->xine = xine;
 
-	this->input_class.open_plugin        = gnomevfs_klass_open;
+	this->input_class.get_instance       = gnomevfs_klass_get_instance;
 	this->input_class.get_identifier     = gnomevfs_klass_get_identifier;
 	this->input_class.get_description    = gnomevfs_klass_get_description;
 	this->input_class.get_dir            = NULL;
 	this->input_class.get_autoplay_list  = NULL;
 	this->input_class.dispose            = gnomevfs_klass_dispose;
-	this->input_class.eject_media        = gnomevfs_klass_eject_media;
 
 	return (input_class_t *) this;
 }
 
 plugin_info_t xine_plugin_info[] = {
-	{ PLUGIN_INPUT, 9, "gnomevfs", XINE_VERSION_CODE, NULL,
+	{ PLUGIN_INPUT, 15, "gnomevfs", XINE_VERSION_CODE, NULL,
 		init_input_class },
 	{ PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
