@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -20,7 +20,7 @@
  * Read from a tcp network stream over a lan (put a tweaked mp1e encoder the
  * other end and you can watch tv anywhere in the house ..)
  *
- * $Id: input_net.c,v 1.41 2002/12/27 16:47:11 miguelfreitas Exp $
+ * $Id: input_net.c,v 1.60 2004/09/17 19:21:46 valtri Exp $
  *
  * how to set up mp1e for use with this plugin:
  * 
@@ -48,47 +48,45 @@
 #include <string.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+
+#ifdef ENABLE_IPV6
+#include <sys/types.h>
+#include <netdb.h>
+#endif
+
+#ifndef WIN32
 #include <arpa/inet.h>
 #include <netdb.h>
+#endif
+
 #include <errno.h>
 #include <sys/time.h>
 
+#define LOG_MODULE "input_net"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
 #include "net_buf_ctrl.h"
 
-#ifdef __GNUC__
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_MSG, message, ##args);                   \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                       \
-    printf(__VA_ARGS__);                                             \
-  }
-#endif
-
-/*
-#define LOG
-*/
-
 #define NET_BS_LEN 2324
-#define PREVIEW_SIZE            2200
 #define BUFSIZE                 1024
 
 typedef struct {
   input_plugin_t   input_plugin;
 
-  xine_stream_t          *stream;
+  xine_stream_t   *stream;
   
   int              fh;
   char            *mrl;
-
-  char             preview[PREVIEW_SIZE];
+  char            *host_port;
+  
+  char             preview[MAX_PREVIEW_SIZE];
   off_t            preview_size;
-  off_t            preview_pos;
 
   off_t            curpos;
 
@@ -113,14 +111,16 @@ typedef struct {
 /*                       Private functions                          */
 /* **************************************************************** */
 
-static int host_connect_attempt(struct in_addr ia, int port, xine_t *xine) {
+#ifndef ENABLE_IPV6
+static int host_connect_attempt_ipv4(struct in_addr ia, int port, xine_t *xine) {
 
   int                s;
   struct sockaddr_in sin;
 
   s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
   if (s==-1) {
-    LOG_MSG(xine, _("input_net: socket(): %s\n"), strerror(errno));
+    xine_log(xine, XINE_LOG_MSG,
+             _("input_net: socket(): %s\n"), strerror(errno));
     return -1;
   }
 
@@ -128,37 +128,123 @@ static int host_connect_attempt(struct in_addr ia, int port, xine_t *xine) {
   sin.sin_addr   = ia;
   sin.sin_port   = htons(port);
   
-  if (connect(s, (struct sockaddr *)&sin, sizeof(sin))==-1 && errno != EINPROGRESS) {
-    LOG_MSG (xine, _("input_net: connect(): %s\n"), strerror(errno));
+#ifndef WIN32
+  if (connect(s, (struct sockaddr *)&sin, sizeof(sin))==-1 && errno != EINPROGRESS) 
+#else
+  if (connect(s, (struct sockaddr *)&sin, sizeof(sin))==-1 && WSAGetLastError() != WSAEINPROGRESS) 
+#endif
+  {
+    xine_log(xine, XINE_LOG_MSG,
+             _("input_net: connect(): %s\n"), strerror(errno));
     close(s);
     return -1;
   }	
 
   return s;
 }
+#else
+static int host_connect_attempt(int family, struct sockaddr* sin, int addrlen, xine_t *xine) {
 
-static int host_connect(const char *host, int port, xine_t *xine) {
+  int                s;
+  
+  s = socket(family, SOCK_STREAM, IPPROTO_TCP);
+  if (s==-1) {
+    xine_log(xine, XINE_LOG_MSG,
+             _("input_net: socket(): %s\n"), strerror(errno));
+    return -1;
+  }
+
+#ifndef WIN32
+  if (connect(s, sin, addrlen)==-1 && errno != EINPROGRESS)  
+#else
+  if (connect(s, sin, addrlen)==-1 && WSAGetLastError() != WSAEINPROGRESS) 
+#endif
+  {
+    xine_log(xine, XINE_LOG_MSG,
+             _("input_net: connect(): %s\n"), strerror(errno));
+    close(s);
+    return -1;
+  }	
+
+  return s;
+}
+#endif
+
+#ifndef ENABLE_IPV6
+static int host_connect_ipv4(const char *host, int port, xine_t *xine) {
   struct hostent *h;
   int             i;
   int             s;
 	
   h = gethostbyname(host);
   if (h==NULL) {
-    LOG_MSG (xine, _("input_net: unable to resolve '%s'.\n"), host);
+    xine_log(xine, XINE_LOG_MSG,
+             _("input_net: unable to resolve '%s'.\n"), host);
     return -1;
   }
 	
   for (i=0; h->h_addr_list[i]; i++) {
     struct in_addr ia;
     memcpy (&ia, h->h_addr_list[i],4);
-    s = host_connect_attempt (ia, port, xine);
+    s = host_connect_attempt_ipv4 (ia, port, xine);
     if (s != -1)
       return s;
   }
 
-  LOG_MSG (xine, _("input_net: unable to connect to '%s'.\n"), host);
+  xine_log(xine, XINE_LOG_MSG,
+           _("input_net: unable to connect to '%s'.\n"), host);
   return -1;
 }
+#endif
+
+static int host_connect(const char *host, int port, xine_t *xine) {
+
+#ifndef ENABLE_IPV6
+  return host_connect_ipv4(host, port, xine);
+#else
+
+  struct addrinfo hints, *res, *tmpaddr;
+  int error;
+  char strport[16];
+  int             i;
+  int             s;
+	
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_family = PF_UNSPEC; 
+  
+  snprintf(strport, sizeof(strport), "%d", port);
+
+  lprintf("Resolving host '%s' at port '%s'\n", host, strport);
+
+  error = getaddrinfo(host, strport, &hints, &res);
+  
+  if (error) {
+      
+    xine_log(xine, XINE_LOG_MSG,
+             _("input_net: unable to resolve '%s'.\n"), host);
+    return -1;
+  }
+  
+  /* We loop over all addresses and try to connect */
+  tmpaddr = res;
+  while (tmpaddr) {
+      
+      s = host_connect_attempt (tmpaddr->ai_family, 
+				tmpaddr->ai_addr, tmpaddr->ai_addrlen, xine);
+      if (s != -1)
+	  return s;
+
+      tmpaddr = tmpaddr->ai_next;
+  }
+  
+  xine_log(xine, XINE_LOG_MSG,
+           _("input_net: unable to connect to '%s'.\n"), host);
+  return -1;
+
+#endif
+
+}
 
 #define LOW_WATER_MARK  50
 #define HIGH_WATER_MARK 100
@@ -168,54 +254,43 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
   off_t n, total;
 
-#ifdef LOG
-  printf ("input_net: reading %d bytes...\n", len);
-#endif
-
-  nbc_check_buffers (this->nbc);
+  lprintf("reading %d bytes...\n", len);
 
   total=0;
-  while (total<len){
+  if (this->curpos < this->preview_size) {
+    n = this->preview_size - this->curpos;
+    if (n > (len - total))
+      n = len - total;
 
-    if (this->preview_pos < this->preview_size) {
-      n = this->preview_size - this->preview_pos;
-      if (n > (len - total))
-        n = len - total;
-#ifdef LOG
-      printf ("input_net: %lld bytes from preview (which has %lld bytes)\n",
-	      n, this->preview_size);
-#endif
+    lprintf("%lld bytes from preview (which has %lld bytes)\n", n, this->preview_size);
 
-      memcpy (&buf[total], &this->preview[this->preview_pos], n);
-      this->preview_pos += n;
-    } else
-    {
-      n = read (this->fh, &buf[total], len-total);
-    }
+    memcpy (&buf[total], &this->preview[this->curpos], n);
+    this->curpos += n;
+    total += n;
+  }
 
-#ifdef LOG
-    printf ("input_net: got %lld bytes (%lld/%lld bytes read)\n",
-	    n,total,len);
-#endif
+  if( (len-total) > 0 ) {
+    n = _x_read_abort (this->stream, this->fh, &buf[total], len-total);
+
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_net: got %" PRIiMAX " bytes (%" PRIiMAX "/%" PRIiMAX " bytes read)\n", (intmax_t)n, (intmax_t)total, (intmax_t)len);
   
-    if (n > 0){
-      this->curpos += n;
-      total += n;
+    if (n < 0) {
+      _x_message(this->stream, XINE_MSG_READ_ERROR, this->host_port, NULL);
+      return 0;
     }
-    else if (n<0 && errno!=EAGAIN) 
-      return total;
+
+    this->curpos += n;
+    total += n;
   }
   return total;
 }
 
 static buf_element_t *net_plugin_read_block (input_plugin_t *this_gen, 
 					     fifo_buffer_t *fifo, off_t todo) {
-  net_input_plugin_t   *this = (net_input_plugin_t *) this_gen;
+  /* net_input_plugin_t   *this = (net_input_plugin_t *) this_gen; */
   buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
   off_t                 total_bytes;
 
-  nbc_check_buffers (this->nbc);
-  
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
   
@@ -256,18 +331,37 @@ static off_t net_plugin_get_current_pos (input_plugin_t *this_gen){
 static off_t net_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
 
-  printf ("input_net: seek %lld bytes, origin %d\n",
-	  offset, origin);
-
-  /* only relative forward-seeking is implemented */
-
   if ((origin == SEEK_CUR) && (offset >= 0)) {
 
     for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
-      this->curpos += net_plugin_read (this_gen, this->seek_buf, BUFSIZE);
+      if( !this_gen->read (this_gen, this->seek_buf, BUFSIZE) )
+        return this->curpos;
     }
 
-    this->curpos += net_plugin_read (this_gen, this->seek_buf, offset);
+    this_gen->read (this_gen, this->seek_buf, offset);
+  }
+
+  if (origin == SEEK_SET) {
+
+    if (offset < this->curpos) {
+
+      if( this->curpos <= this->preview_size )
+        this->curpos = offset;
+      else
+        xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                "input_net: cannot seek back! (%" PRIiMAX " > %" PRIiMAX ")\n",
+                (intmax_t)this->curpos, (intmax_t)offset);
+
+    } else {
+      offset -= this->curpos;
+
+      for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
+        if( !this_gen->read (this_gen, this->seek_buf, BUFSIZE) )
+          return this->curpos;
+      }
+
+      this_gen->read (this_gen, this->seek_buf, offset);
+    }
   }
 
   return this->curpos;
@@ -299,10 +393,13 @@ static int net_plugin_get_optional_data (input_plugin_t *this_gen,
 static void net_plugin_dispose (input_plugin_t *this_gen ) {
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
 
-  close(this->fh);
-  this->fh = -1;
-  
+  if (this->fh != -1) {
+    close(this->fh);
+    this->fh = -1;
+  }
+    
   free (this->mrl);
+  free (this->host_port);
   
   if (this->nbc) {
     nbc_close (this->nbc);
@@ -312,32 +409,13 @@ static void net_plugin_dispose (input_plugin_t *this_gen ) {
   free (this_gen);
 }
 
-
-static input_plugin_t *net_plugin_open (input_class_t *cls_gen, xine_stream_t *stream, const char *mrl) {
-  /* net_input_plugin_t *this = (net_input_plugin_t *) this_gen; */
-  net_input_plugin_t *this = xine_xmalloc(sizeof(net_input_plugin_t));
+static int net_plugin_open (input_plugin_t *this_gen ) {
+  net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
   char *filename;
   char *pptr;
   int port = 7658;
 
-  this->mrl = strdup(mrl);
-  this->stream = stream;
-
-  if (!strncasecmp (mrl, "tcp://", 6)) {
-    filename = (char *) &this->mrl[6];
-    
-    if((!filename) || (strlen(filename) == 0)) {
-      free (this->mrl);
-      free (this);
-      return NULL;
-    }
-    
-  } else {
-    free (this->mrl);
-    free (this);
-    return NULL;
-  }
-    
+  filename = this->host_port;
   pptr=strrchr(filename, ':');
   if(pptr) {
     *pptr++ = 0;
@@ -348,24 +426,67 @@ static input_plugin_t *net_plugin_open (input_class_t *cls_gen, xine_stream_t *s
   this->curpos = 0;
 
   if (this->fh == -1) {
-    free (this->mrl);
-    free (this);
-    return NULL;
+    return 0;
   }
 
-  this->nbc = nbc_init (this->stream);
-
   /*
    * fill preview buffer
    */
-  this->preview_pos  = 0;
+#ifndef WIN32
+  this->preview_size = read (this->fh, this->preview, MAX_PREVIEW_SIZE);
+#else
+  this->preview_size = recv (this->fh, this->preview, MAX_PREVIEW_SIZE, 0);
+#endif
+
+  this->curpos       = 0;
+
+  return 1;
+}
+
+static input_plugin_t *net_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, const char *mrl) {
+  /* net_input_plugin_t *this = (net_input_plugin_t *) this_gen; */
+  net_input_plugin_t *this;
+  nbc_t *nbc = NULL;
+  char *filename;
+
+  if (!strncasecmp (mrl, "tcp://", 6)) {
+    filename = (char *) &mrl[6];
+    
+    if((!filename) || (strlen(filename) == 0)) {
+      return NULL;
+    }
+    
+    nbc = nbc_init (stream);
+    
+  } else if (!strncasecmp (mrl, "slave://", 8)) {
+ 
+    filename = (char *) &mrl[8];
+    
+    if((!filename) || (strlen(filename) == 0)) {
+      return NULL;
+    }
+    
+    /* the only difference for slave:// is that network buffering control
+     * is not used. otherwise, dvd still menus are not displayed (it freezes
+     * with "buffering..." all the time)
+     */
+    
+    nbc = NULL;
+    
+  } else {
+    return NULL;
+  }
+
+  this = xine_xmalloc(sizeof(net_input_plugin_t));
+  this->mrl           = strdup(mrl);
+  this->host_port     = strdup(filename);
+  this->stream        = stream;
+  this->fh            = -1;
+  this->curpos        = 0;
+  this->nbc           = nbc;
   this->preview_size  = 0;
 
-  this->preview_size = read (this->fh, this->preview, PREVIEW_SIZE);
-  
-  this->preview_pos  = 0;
-  this->curpos  = 0;
-
+  this->input_plugin.open              = net_plugin_open;
   this->input_plugin.get_capabilities  = net_plugin_get_capabilities;
   this->input_plugin.read              = net_plugin_read;
   this->input_plugin.read_block        = net_plugin_read_block;
@@ -378,7 +499,6 @@ static input_plugin_t *net_plugin_open (input_class_t *cls_gen, xine_stream_t *s
   this->input_plugin.dispose           = net_plugin_dispose;
   this->input_plugin.input_class       = cls_gen;
 
-
   return &this->input_plugin;
 }
 
@@ -391,7 +511,7 @@ static char *net_class_get_description (input_class_t *this_gen) {
 	return _("net input plugin as shipped with xine");
 }
 
-static char *net_class_get_identifier (input_class_t *this_gen) {
+static const char *net_class_get_identifier (input_class_t *this_gen) {
   return "TCP";
 }
 
@@ -409,7 +529,7 @@ static void *init_class (xine_t *xine, void *data) {
   this->config = xine->config;
   this->xine   = xine;
 
-  this->input_class.open_plugin       = net_plugin_open;
+  this->input_class.get_instance      = net_class_get_instance;
   this->input_class.get_description   = net_class_get_description;
   this->input_class.get_identifier    = net_class_get_identifier;
   this->input_class.get_dir           = NULL;
@@ -426,7 +546,7 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 11, "tcp", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 15, "tcp", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
