@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2000-2002 the xine project
+/*
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,19 +17,27 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_plugin.h,v 1.43 2002/12/29 16:48:35 mroi Exp $
+ * $Id: input_plugin.h,v 1.58 2004/09/26 22:54:52 valtri Exp $
  */
 
 #ifndef HAVE_INPUT_PLUGIN_H
 #define HAVE_INPUT_PLUGIN_H
 
-#include <inttypes.h>
 #include <sys/types.h>
-#include <assert.h>
-#include "buffer.h"
-#include "configfile.h"
 
-#define INPUT_PLUGIN_IFACE_VERSION   11
+#ifdef XINE_COMPILE
+#  include <inttypes.h>
+#  include "xineutils.h"
+#  include "buffer.h"
+#  include "configfile.h"
+#else
+#  include <xine/os_types.h>
+#  include <xine/xineutils.h>
+#  include <xine/buffer.h>
+#  include <xine/configfile.h>
+#endif
+
+#define INPUT_PLUGIN_IFACE_VERSION   15
  
 typedef struct input_class_s input_class_t ;
 typedef struct input_plugin_s input_plugin_t;
@@ -37,14 +45,16 @@ typedef struct input_plugin_s input_plugin_t;
 struct input_class_s {
 
   /*
-   * open a new instance of this plugin class
+   * create a new instance of this plugin class
+   * return NULL if the plugin does'nt handle the given mrl
    */
-  input_plugin_t* (*open_plugin) (input_class_t *this, xine_stream_t *stream, const char *mrl);
+  input_plugin_t* (*get_instance) (input_class_t *this, xine_stream_t *stream, const char *mrl);
 
   /*
    * return short, human readable identifier for this plugin class
+   * The returned string should not be malloc'd as it is not freed.
    */
-  char* (*get_identifier) (input_class_t *this);
+  const char* (*get_identifier) (input_class_t *this);
 
   /*
    * return human readable (verbose = 1 line) description for 
@@ -79,6 +89,12 @@ struct input_class_s {
 
 struct input_plugin_s {
 
+  /*
+   * open the stream
+   * return 0 if an error occured
+   */
+  int (*open) (input_plugin_t *this);
+
   /*
    * return capabilities of the current playable entity. See
    * get_current_pos below for a description of a "playable entity"
@@ -219,6 +235,8 @@ struct input_plugin_s {
  *   input plugin knows something about audio/spu languages, 
  *   e.g. knows that audio stream #0 is english, 
  *   audio stream #1 is german, ...
+ *   *((int *)data) will provide the requested channel number
+ *   and awaits the language back in (char *)data
  */
 
 #define INPUT_CAP_AUDIOLANG            0x00000008
@@ -232,6 +250,8 @@ struct input_plugin_s {
  *   they can handle the stream or not. the preview data must 
  *   be buffered and delivered again through subsequent 
  *   read() calls.
+ *   caller must provide a buffer allocated with at least 
+ *   MAX_PREVIEW_SIZE bytes.
  */
 
 #define INPUT_CAP_PREVIEW              0x00000040  
@@ -247,6 +267,15 @@ struct input_plugin_s {
 
 #define INPUT_CAP_CHAPTERS             0x00000080
 
+/*
+ * INPUT_CAP_RIP_FORBIDDEN:
+ *   means that rip/disk saving must not be used.
+ *   (probably at author's request)
+ */
+
+#define INPUT_CAP_RIP_FORBIDDEN        0x00000100
+
+#define INPUT_IS_SEEKABLE(input) (((input)->get_capabilities(input) & INPUT_CAP_SEEKABLE) != 0)
 
 #define INPUT_OPTIONAL_UNSUPPORTED    0
 #define INPUT_OPTIONAL_SUCCESS        1
@@ -256,6 +285,7 @@ struct input_plugin_s {
 #define INPUT_OPTIONAL_DATA_PREVIEW   7
 
 #define MAX_MRL_ENTRIES 255
+#define MAX_PREVIEW_SIZE 4096
 
 /* Types of mrls returned by get_dir() */
 #define mrl_unknown        (0 << 0)
@@ -300,8 +330,8 @@ struct input_plugin_s {
  * Duplicate two mrls entries (s = source, d = destination).
  */
 #define MRL_DUPLICATE(s, d) {                                                 \
-  assert((s) != NULL);                                                        \
-  assert((d) != NULL);                                                        \
+  _x_assert((s) != NULL);                                                     \
+  _x_assert((d) != NULL);                                                     \
                                                                               \
   if((s)->origin) {                                                           \
     if((d)->origin) {                                                         \
@@ -346,11 +376,11 @@ struct input_plugin_s {
 #define MRLS_DUPLICATE(s, d) {                                                \
   int i = 0;                                                                  \
                                                                               \
-  assert((s) != NULL);                                                        \
-  assert((d) != NULL);                                                        \
+  _x_assert((s) != NULL);                                                     \
+  _x_assert((d) != NULL);                                                     \
                                                                               \
   while((s) != NULL) {                                                        \
-    d[i] = (xine_mrl_t *) malloc(sizeof(xine_mrl_t));                                   \
+    d[i] = (xine_mrl_t *) malloc(sizeof(xine_mrl_t));                         \
     MRL_DUPLICATE(s[i], d[i]);                                                \
     i++;                                                                      \
   }                                                                           \
