@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine project
+ * Copyright (C) 2002-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -26,7 +26,6 @@
 
 #include <unistd.h>
 #include <stdio.h>
-#include <assert.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -34,8 +33,14 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include "bswap.h"
 
+#define LOG_MODULE "input_pnm"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
+#include "bswap.h"
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
@@ -43,10 +48,6 @@
 #include "pnm.h"
 #include "net_buf_ctrl.h"
 
-/*
-#define LOG
-*/
-
 #define BUFSIZE 4096
 
 #if !defined(NDELAY) && defined(O_NDELAY)
@@ -63,6 +64,8 @@ typedef struct {
 typedef struct {
   input_plugin_t   input_plugin;
 
+  xine_stream_t   *stream;
+  
   pnm_t           *pnm;
 
   char            *mrl;
@@ -81,10 +84,7 @@ static off_t pnm_plugin_read (input_plugin_t *this_gen,
   pnm_input_plugin_t *this = (pnm_input_plugin_t *) this_gen;
   off_t               n;
 
-#ifdef LOG
-  printf ("pnm_plugin_read: %lld bytes ...\n",
-          len);
-#endif
+  lprintf ("pnm_plugin_read: %lld bytes ...\n", len);
 
   nbc_check_buffers (this->nbc);
 
@@ -100,10 +100,7 @@ static buf_element_t *pnm_plugin_read_block (input_plugin_t *this_gen,
   buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
   int                   total_bytes;
 
-#ifdef LOG
-  printf ("pnm_plugin_read_block: %lld bytes...\n",
-          todo);
-#endif
+  lprintf ("pnm_plugin_read_block: %lld bytes...\n", todo);
 
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
@@ -123,8 +120,9 @@ static buf_element_t *pnm_plugin_read_block (input_plugin_t *this_gen,
 static off_t pnm_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   pnm_input_plugin_t *this = (pnm_input_plugin_t *) this_gen;
 
-  printf ("input_pnm: seek %lld bytes, origin %d\n",
-	  offset, origin);
+  xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+           "input_pnm: seek %" PRIiMAX " bytes, origin %d\n", (intmax_t)offset,
+           origin);
 
   /* only realtive forward-seeking is implemented */
 
@@ -151,7 +149,7 @@ static off_t pnm_plugin_get_length (input_plugin_t *this_gen) {
 }
 
 static uint32_t pnm_plugin_get_capabilities (input_plugin_t *this_gen) {
-  return INPUT_CAP_PREVIEW;
+  return INPUT_CAP_PREVIEW | INPUT_CAP_RIP_FORBIDDEN;
 }
 
 static uint32_t pnm_plugin_get_blocksize (input_plugin_t *this_gen) {
@@ -200,7 +198,7 @@ static int pnm_plugin_get_optional_data (input_plugin_t *this_gen,
   switch (data_type) {
   case INPUT_OPTIONAL_DATA_PREVIEW:
 
-    return pnm_peek_header(this->pnm, data);
+    return pnm_peek_header(this->pnm, data, MAX_PREVIEW_SIZE);
 
     break;
   }
@@ -208,38 +206,43 @@ static int pnm_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
-static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
+static int pnm_plugin_open (input_plugin_t *this_gen) {
+  pnm_input_plugin_t *this = (pnm_input_plugin_t *) this_gen;
+  pnm_t              *pnm;
+
+  lprintf ("trying to open '%s'\n", this->mrl);
+
+  pnm = pnm_connect (this->stream, this->mrl);
+
+  if (!pnm) {
+    return 0;
+  }
+
+  this->pnm    = pnm;
+  
+  return 1;
+}
+
+static input_plugin_t *pnm_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, 
 				    const char *data) {
 
   /* pnm_input_class_t  *cls = (pnm_input_class_t *) cls_gen; */
   pnm_input_plugin_t *this;
-  pnm_t              *pnm;
   char               *mrl = strdup(data);
 
-#ifdef LOG
-  printf ("input_pnm: trying to open '%s'\n", mrl);
-#endif
-
   if (strncasecmp (mrl, "pnm://", 6)) {
     free (mrl);
     return NULL;
   }
 
-  pnm = pnm_connect (mrl);
-
-  if (!pnm) {
-    free (mrl);
-    return NULL;
-  }
-
   this = (pnm_input_plugin_t *) xine_xmalloc (sizeof (pnm_input_plugin_t));
 
-  this->pnm    = pnm;
+  this->stream = stream;
+  this->pnm    = NULL;
   this->mrl    = mrl; 
-  this->nbc    = nbc_init (stream);
-  
-  nbc_set_high_water_mark(this->nbc, 50);
+  this->nbc    = nbc_init (this->stream);
   
+  this->input_plugin.open              = pnm_plugin_open;
   this->input_plugin.get_capabilities  = pnm_plugin_get_capabilities;
   this->input_plugin.read              = pnm_plugin_read;
   this->input_plugin.read_block        = pnm_plugin_read_block;
@@ -263,7 +266,7 @@ static char *pnm_class_get_description (input_class_t *this_gen) {
   return _("pnm streaming input plugin");
 }
 
-static char *pnm_class_get_identifier (input_class_t *this_gen) {
+static const char *pnm_class_get_identifier (input_class_t *this_gen) {
   return "pnm";
 }
 
@@ -281,7 +284,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   this->xine   = xine;
 
-  this->input_class.open_plugin        = open_plugin;
+  this->input_class.get_instance       = pnm_class_get_instance;
   this->input_class.get_identifier     = pnm_class_get_identifier;
   this->input_class.get_description    = pnm_class_get_description;
   this->input_class.get_dir            = NULL;
@@ -298,7 +301,7 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 11, "pnm", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 15, "pnm", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
