@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine project
+ * Copyright (C) 2002-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -26,7 +26,6 @@
 
 #include <unistd.h>
 #include <stdio.h>
-#include <assert.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -34,8 +33,14 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include "bswap.h"
 
+#define LOG_MODULE "input_rtsp"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
+#include "bswap.h"
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
@@ -43,10 +48,6 @@
 #include "librtsp/rtsp_session.h"
 #include "net_buf_ctrl.h"
 
-/*
-#define LOG
-*/
-
 #define BUFSIZE 1025
 
 #if !defined(NDELAY) && defined(O_NDELAY)
@@ -63,7 +64,9 @@ typedef struct {
 typedef struct {
   input_plugin_t   input_plugin;
 
-  rtsp_session_t           *rtsp;
+  rtsp_session_t  *rtsp;
+
+  xine_stream_t   *stream;
 
   char            *mrl;
   char            *public_mrl;
@@ -82,10 +85,7 @@ static off_t rtsp_plugin_read (input_plugin_t *this_gen,
   rtsp_input_plugin_t *this = (rtsp_input_plugin_t *) this_gen;
   off_t               n;
 
-#ifdef LOG
-  printf ("rtsp_plugin_read: %lld bytes ...\n",
-          len);
-#endif
+  lprintf ("rtsp_plugin_read: %lld bytes ...\n", len);
 
   nbc_check_buffers (this->nbc);
 
@@ -101,10 +101,7 @@ static buf_element_t *rtsp_plugin_read_block (input_plugin_t *this_gen,
   buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
   int                   total_bytes;
 
-#ifdef LOG
-  printf ("rtsp_plugin_read_block: %lld bytes...\n",
-          todo);
-#endif
+  lprintf ("rtsp_plugin_read_block: %lld bytes...\n", todo);
 
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
@@ -125,8 +122,7 @@ static off_t rtsp_plugin_seek (input_plugin_t *this_gen, off_t offset, int origi
 
   rtsp_input_plugin_t *this = (rtsp_input_plugin_t *) this_gen;
 
-  printf ("input_rtsp: seek %lld bytes, origin %d\n",
-	  offset, origin);
+  lprintf ("seek %lld bytes, origin %d\n", offset, origin);
 
   /* only realtive forward-seeking is implemented */
 
@@ -153,7 +149,7 @@ static off_t rtsp_plugin_get_length (input_plugin_t *this_gen) {
 }
 
 static uint32_t rtsp_plugin_get_capabilities (input_plugin_t *this_gen) {
-  return INPUT_CAP_PREVIEW;
+  return INPUT_CAP_PREVIEW | INPUT_CAP_RIP_FORBIDDEN;
 }
 
 static uint32_t rtsp_plugin_get_blocksize (input_plugin_t *this_gen) {
@@ -205,7 +201,7 @@ static int rtsp_plugin_get_optional_data (input_plugin_t *this_gen,
   switch (data_type) {
   case INPUT_OPTIONAL_DATA_PREVIEW:
 
-    return rtsp_session_peek_header(this->rtsp, data);
+    return rtsp_session_peek_header(this->rtsp, data, MAX_PREVIEW_SIZE);
 
     break;
   }
@@ -213,46 +209,52 @@ static int rtsp_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
-static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
+static int rtsp_plugin_open (input_plugin_t *this_gen) {
+  rtsp_input_plugin_t *this = (rtsp_input_plugin_t *) this_gen;
+
+  rtsp_session_t      *rtsp;
+
+  lprintf ("trying to open '%s'\n", this->mrl);
+
+  rtsp = rtsp_session_start(this->stream,this->mrl);
+
+  if (!rtsp) {
+    lprintf ("returning null.\n");
+
+    return 0;
+  }
+
+  this->rtsp = rtsp;
+  
+  return 1;
+}
+
+static input_plugin_t *rtsp_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, 
 				    const char *data) {
 
   /* rtsp_input_class_t  *cls = (rtsp_input_class_t *) cls_gen; */
   rtsp_input_plugin_t *this;
-  rtsp_session_t              *rtsp;
-  char               *mrl = strdup(data);
-
-#ifdef LOG
-  printf ("input_rtsp: trying to open '%s'\n", mrl);
-#endif
+  char                *mrl = strdup(data);
 
   if (strncasecmp (mrl, "rtsp://", 6)) {
     free (mrl);
     return NULL;
   }
 
-  rtsp = rtsp_session_start(mrl);
-
-  if (!rtsp) {
-    free (mrl);
-#ifdef LOG
-    printf ("input_rtsp: returning null.\n");
-#endif
-    return NULL;
-  }
-
   this = (rtsp_input_plugin_t *) xine_xmalloc (sizeof (rtsp_input_plugin_t));
 
-  this->rtsp    = rtsp;
-  this->mrl    = mrl;
+  this->stream  = stream;
+  this->rtsp    = NULL;
+  this->mrl     = mrl;
   /* since we handle only real streams yet, we can savely add
    * an .rm extention to force handling by demux_real.
    */
   this->public_mrl = xine_xmalloc (sizeof (char)*(strlen(this->mrl)+10));
   sprintf(this->public_mrl, "%s.rm", this->mrl);
   
-  this->nbc    = nbc_init (stream);
-  nbc_set_high_water_mark(this->nbc, 50);
+  this->nbc     = nbc_init (stream);
 
+  this->input_plugin.open              = rtsp_plugin_open;
   this->input_plugin.get_capabilities  = rtsp_plugin_get_capabilities;
   this->input_plugin.read              = rtsp_plugin_read;
   this->input_plugin.read_block        = rtsp_plugin_read_block;
@@ -276,7 +278,7 @@ static char *rtsp_class_get_description (input_class_t *this_gen) {
   return _("rtsp streaming input plugin");
 }
 
-static char *rtsp_class_get_identifier (input_class_t *this_gen) {
+static const char *rtsp_class_get_identifier (input_class_t *this_gen) {
   return "rtsp";
 }
 
@@ -294,7 +296,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   this->xine   = xine;
 
-  this->input_class.open_plugin        = open_plugin;
+  this->input_class.get_instance       = rtsp_class_get_instance;
   this->input_class.get_identifier     = rtsp_class_get_identifier;
   this->input_class.get_description    = rtsp_class_get_description;
   this->input_class.get_dir            = NULL;
@@ -311,7 +313,7 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 11, "rtsp", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 15, "rtsp", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
