@@ -1,6 +1,6 @@
-/* 
- * Copyright (C) 2000-2002 the xine project
- * 
+/*
+ * Copyright (C) 2000-2003 the xine project
+ *
  * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_stdin_fifo.c,v 1.41 2003/01/17 17:50:20 mroi Exp $
+ * $Id: input_stdin_fifo.c,v 1.60 2004/09/17 19:21:46 valtri Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,37 +32,36 @@
 #include <sys/stat.h>
 #include <errno.h>
 
+#define LOG_MODULE "input_stdin_fifo"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
 #include "net_buf_ctrl.h"
 
-/*
-#define LOG
-*/
-
-#define PREVIEW_SIZE            2200
-
-#define DEFAULT_LOW_WATER_MARK     1
-#define DEFAULT_HIGH_WATER_MARK    5
+#define BUFSIZE                 1024
 
 typedef struct {
   input_plugin_t   input_plugin;
 
   xine_stream_t   *stream;
-  
+
   int              fh;
   char            *mrl;
   off_t            curpos;
 
-  char             preview[PREVIEW_SIZE];
+  char             preview[MAX_PREVIEW_SIZE];
   off_t            preview_size;
-  off_t            preview_pos;
 
-  nbc_t           *nbc; 
+  nbc_t           *nbc;
 
-  char             scratch[1025];
+  char             seek_buf[BUFSIZE];
 
+  xine_t          *xine;
 } stdin_input_plugin_t;
 
 typedef struct {
@@ -72,64 +71,44 @@ typedef struct {
   xine_t           *xine;
 } stdin_input_class_t;
 
+static off_t stdin_plugin_get_current_pos (input_plugin_t *this_gen);
+
 
 
 static off_t stdin_plugin_read (input_plugin_t *this_gen, 
-				char *buf, off_t todo) {
+				char *buf, off_t len) {
 
   stdin_input_plugin_t  *this = (stdin_input_plugin_t *) this_gen;
-  off_t                  num_bytes, total_bytes;
+  off_t n, total;
 
-  nbc_check_buffers (this->nbc);
+  lprintf ("reading %lld bytes...\n", len);
 
-  total_bytes = 0;
+  total=0;
+  if (this->curpos < this->preview_size) {
+    n = this->preview_size - this->curpos;
+    if (n > (len - total))
+      n = len - total;
+    lprintf ("%lld bytes from preview (which has %lld bytes)\n", n, this->preview_size);
 
-#ifdef LOG
-  printf ("stdin: read %lld bytes...\n", todo);
-#endif
-
-  while (total_bytes < todo) {
-
-    if (this->preview_pos < this->preview_size) {
-
-      num_bytes = this->preview_size - this->preview_pos;
-      if (num_bytes > (todo - total_bytes)) 
-	num_bytes = todo - total_bytes;
-
-#ifdef LOG
-      printf ("stdin: %lld bytes from preview (which has %lld bytes)\n",
-	      num_bytes, this->preview_size);
-#endif
-
-      memcpy (&buf[total_bytes], &this->preview[this->preview_pos], num_bytes);
-
-      this->preview_pos += num_bytes;
-
-    } else {
-      num_bytes = read (this->fh, &buf[total_bytes], todo - total_bytes);
-#ifdef LOG
-      printf ("stdin: %lld bytes from file\n",
-	      num_bytes);
-#endif
-    }
-
-
-    if(num_bytes < 0) {
-
-      this->curpos += total_bytes;
-      return num_bytes;
-
-    } else if (!num_bytes) {
-
-      this->curpos += total_bytes;
-      return total_bytes;
-    }
-    total_bytes += num_bytes;
+    memcpy (&buf[total], &this->preview[this->curpos], n);
+    this->curpos += n;
+    total += n;
   }
 
-  this->curpos += total_bytes;
+  if( (len-total) > 0 ) {
+    n = _x_io_file_read (this->stream, this->fh, &buf[total], len - total);
 
-  return total_bytes;
+    lprintf ("got %lld bytes (%lld/%lld bytes read)\n", n,total,len);
+  
+    if (n < 0) {
+      _x_message(this->stream, XINE_MSG_READ_ERROR, NULL);
+      return 0;
+    }
+
+    this->curpos += n;
+    total += n;
+  }
+  return total;
 }
 
 static buf_element_t *stdin_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, 
@@ -154,50 +133,44 @@ static buf_element_t *stdin_plugin_read_block (input_plugin_t *this_gen, fifo_bu
   return buf;
 }
 
+/* forward reference */
 static off_t stdin_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
 
   stdin_input_plugin_t  *this = (stdin_input_plugin_t *) this_gen;
-  off_t dest = this->curpos;
 
-#ifdef LOG
-  printf ("stdin: seek %lld offset, %d origin...\n",
-	  offset, origin);
-#endif
+  lprintf ("seek %lld offset, %d origin...\n", offset, origin);
 
-  switch (origin) {
-  case SEEK_SET:
-    dest = offset;
-    break;
-  case SEEK_CUR:
-    dest = this->curpos + offset;
-    break;
-  case SEEK_END:
-    printf ("stdin: SEEK_END not implemented!\n");
-    return this->curpos;
-  default:
-    printf ("stdin: unknown origin in seek!\n");
-    return this->curpos;
+  if ((origin == SEEK_CUR) && (offset >= 0)) {
+
+    for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
+      if( !this_gen->read (this_gen, this->seek_buf, BUFSIZE) )
+        return this->curpos;
+    }
+
+    this_gen->read (this_gen, this->seek_buf, offset);
   }
 
-  if (this->curpos > dest) {
-    printf ("stdin: cannot seek back!\n");
-    return this->curpos;
-  }
+  if (origin == SEEK_SET) {
 
-  while (this->curpos<dest) {
+    if (offset < this->curpos) {
 
-    off_t n, diff;
+      if( this->curpos <= this->preview_size ) 
+        this->curpos = offset;
+      else
+        xprintf (this->xine, XINE_VERBOSITY_LOG, 
+                 _("stdin: cannot seek back! (%" PRIiMAX " > %" PRIiMAX ")\n"), 
+                 (intmax_t)this->curpos, (intmax_t)offset);
 
-    diff = dest - this->curpos;
+    } else {
+      offset -= this->curpos;
 
-    if (diff>1024)
-      diff = 1024;
+      for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
+        if( !this_gen->read (this_gen, this->seek_buf, BUFSIZE) )
+          return this->curpos;
+      }
 
-    n = stdin_plugin_read (this_gen, this->scratch, diff);
-
-    this->curpos += n;
-    if (n<diff)
-      return this->curpos;
+      this_gen->read (this_gen, this->seek_buf, offset);
+    }
   }
 
   return this->curpos;
@@ -236,7 +209,7 @@ static void stdin_plugin_dispose (input_plugin_t *this_gen ) {
   if (this->nbc) 
     nbc_close (this->nbc);
 
-  if (this->fh != STDIN_FILENO)
+  if ((this->fh != STDIN_FILENO) && (this->fh != -1))
     close(this->fh);
   
   free (this->mrl);
@@ -259,48 +232,74 @@ static int stdin_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
+static int stdin_plugin_open (input_plugin_t *this_gen ) {
+  stdin_input_plugin_t *this = (stdin_input_plugin_t *) this_gen;
 
-static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
-				    const char *data) {
+  lprintf ("trying to open '%s'...\n", this->mrl);
 
-  /* stdin_input_class_t  *cls = (stdin_input_class_t *) cls_gen; */
+  if (this->fh == -1) {
+    char *filename;
+
+    filename = (char *) &this->mrl[5];
+    this->fh = open (filename, O_RDONLY);
+
+    lprintf("filename '%s'\n", filename);
+
+    if (this->fh == -1) {
+      xprintf (this->xine, XINE_VERBOSITY_LOG, _("stdin: failed to open '%s'\n"), filename);
+      return 0;
+    }
+  }
+
+
+  /*
+   * mrl accepted and opened successfully at this point
+   *
+   * => create plugin instance
+   */
+
+  /*
+   * fill preview buffer
+   */
+
+  this->preview_size = stdin_plugin_read (&this->input_plugin, this->preview,
+					  MAX_PREVIEW_SIZE);
+  this->curpos          = 0;
+
+  return 1;
+}
+
+
+static input_plugin_t *stdin_class_get_instance (input_class_t *class_gen, 
+						 xine_stream_t *stream, const char *data) {
+
+  stdin_input_class_t  *class = (stdin_input_class_t *) class_gen;
   stdin_input_plugin_t *this;
   char                 *mrl = strdup(data);
   int                   fh;
 
 
-#ifdef LOG
-  printf ("input_stdin_fifo: trying to open '%s'...\n",
-	  mrl);
-#endif
-
-  if (!strncasecmp(mrl, "stdin:/", 7) 
-      || !strncmp(mrl, "-", 1)) {
+  if (!strncasecmp(mrl, "stdin:/", 7)
+      || !strncmp(mrl, "-", 1)
+      || !strncmp(mrl, "fd://0", 6)) {
 
     fh = STDIN_FILENO;
 
-  } else if (!strncasecmp (mrl, "fifo:", 5)) {
+  } else if (!strncasecmp (mrl, "fifo:/", 6)) {
     char *filename;
 
     filename = (char *) &mrl[5];
-    
-    fh = open (filename, O_RDONLY);
+    fh = -1;
+
+    lprintf("filename '%s'\n", filename);
 
-    printf("input_stdin_fifo: filename '%s'\n", filename);
-  
-    if (fh == -1) {
-      printf ("stdin: failed to open '%s'\n",
-	      filename);
-      free (mrl);
-      return NULL;
-    }
   } else {
     free (mrl);
     return NULL;
   }
-  
 
-  /* 
+
+  /*
    * mrl accepted and opened successfully at this point
    *
    * => create plugin instance
@@ -309,7 +308,12 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   this       = (stdin_input_plugin_t *) xine_xmalloc(sizeof(stdin_input_plugin_t));
 
   this->stream = stream;
+  this->curpos = 0;
+  this->mrl    = mrl;
+  this->fh     = fh;
+  this->xine   = class->xine;
 
+  this->input_plugin.open              = stdin_plugin_open;
   this->input_plugin.get_capabilities  = stdin_plugin_get_capabilities;
   this->input_plugin.read              = stdin_plugin_read;
   this->input_plugin.read_block        = stdin_plugin_read_block;
@@ -320,26 +324,13 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   this->input_plugin.get_mrl           = stdin_plugin_get_mrl;
   this->input_plugin.dispose           = stdin_plugin_dispose;
   this->input_plugin.get_optional_data = stdin_plugin_get_optional_data;
-  this->input_plugin.input_class       = cls_gen;
-
-  this->curpos          = 0;
-  this->mrl             = mrl;
-  this->fh              = fh;
+  this->input_plugin.input_class       = class_gen;
 
   /*
    * buffering control
    */
-
   this->nbc    = nbc_init (this->stream);
 
-  /*
-   * fill preview buffer
-   */
-
-  this->preview_size = stdin_plugin_read (&this->input_plugin, this->preview,
-					  PREVIEW_SIZE);
-  this->preview_pos  = 0;
-
   return &this->input_plugin;
 }
 
@@ -351,7 +342,7 @@ static char *stdin_class_get_description (input_class_t *this_gen) {
   return _("stdin streaming input plugin");
 }
 
-static char *stdin_class_get_identifier (input_class_t *this_gen) {
+static const char *stdin_class_get_identifier (input_class_t *this_gen) {
   return "stdin_fifo";
 }
 
@@ -369,7 +360,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   this->xine   = xine;
 
-  this->input_class.open_plugin        = open_plugin;
+  this->input_class.get_instance       = stdin_class_get_instance;
   this->input_class.get_identifier     = stdin_class_get_identifier;
   this->input_class.get_description    = stdin_class_get_description;
   this->input_class.get_dir            = NULL;
@@ -385,7 +376,7 @@ static void *init_class (xine_t *xine, void *data) {
  */
 
 plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 11, "stdin", XINE_VERSION_CODE, NULL, init_class },
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_INPUT, 15, "stdin", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
