@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine project
+ * Copyright (C) 2002-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mms.h,v 1.8 2002/12/13 08:56:53 f1rmb Exp $
+ * $Id: mms.h,v 1.11 2004/04/06 00:25:29 tmattern Exp $
  *
  * libmms public header
  */
@@ -37,7 +37,9 @@ int      mms_read (mms_t *this, char *data, int len);
 uint32_t mms_get_length (mms_t *this);
 void     mms_close (mms_t *this);
 
-int      mms_peek_header (mms_t *this, char *data);
+int      mms_peek_header (mms_t *this, char *data, int maxsize);
+
+off_t    mms_get_current_pos (mms_t *this);
 
 #endif
 
