@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.40 2003/01/11 11:29:22 esnel Exp $
+ * $Id: xine_decoder.c,v 1.53 2004/12/16 13:59:11 mroi Exp $
  *
  * stuff needed to turn libmad into a xine decoder plugin
  */
@@ -25,6 +25,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define LOG_MODULE "mad_decoder"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "audio_out.h"
 #include "buffer.h"
@@ -32,10 +38,6 @@
 #include "synth.h"
 #include "xineutils.h"
 
-/*
-#define LOG
-*/
-
 #define INPUT_BUF_SIZE  16384
 
 typedef struct {
@@ -77,6 +79,7 @@ static void mad_reset (audio_decoder_t *this_gen) {
 
   mad_synth_init  (&this->synth);
   mad_stream_init (&this->stream);
+  this->stream.options = MAD_OPTION_IGNORECRC;
   mad_frame_init  (&this->frame);
 }
 
@@ -125,13 +128,12 @@ static void mad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
   mad_decoder_t *this = (mad_decoder_t *) this_gen;
 
-#ifdef LOG
-  printf ("libmad: decode data, decoder_flags: %d\n", buf->decoder_flags);
-#endif  
+  lprintf ("decode data, decoder_flags: %d\n", buf->decoder_flags);
   
   if (buf->size>(INPUT_BUF_SIZE-this->bytes_in_buffer)) {
-    printf ("libmad: ALERT input buffer too small (%d bytes, %d avail)!\n",
-	    buf->size, INPUT_BUF_SIZE-this->bytes_in_buffer);
+    xprintf (this->xstream->xine, XINE_VERBOSITY_DEBUG,
+	     "libmad: ALERT input buffer too small (%d bytes, %d avail)!\n",
+	     buf->size, INPUT_BUF_SIZE-this->bytes_in_buffer);
     buf->size = INPUT_BUF_SIZE-this->bytes_in_buffer;
   }
   
@@ -188,30 +190,34 @@ static void mad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	    || (this->output_sampling_rate != this->frame.header.samplerate)
 	    || (this->output_mode != mode)) {
 
-#ifdef LOG
-	  printf ("libmad: audio sample rate %d mode %08x\n",
-		  this->frame.header.samplerate,
-		  mode);
-#endif
+	  lprintf ("audio sample rate %d mode %08x\n", this->frame.header.samplerate, mode);
 
-	  this->xstream->stream_info[XINE_STREAM_INFO_AUDIO_BITRATE] = this->frame.header.bitrate;
-	  switch (this->frame.header.layer) {
-	  case MAD_LAYER_I:
-	    this->xstream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-	      = strdup ("MPEG audio layer 1");
-	    break;
-	  case MAD_LAYER_II:
-	    this->xstream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-	      = strdup ("MPEG audio layer 2");
-	    break;
-	  case MAD_LAYER_III:
-	    this->xstream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-	      = strdup ("MPEG audio layer 3");
-	    break;
-	  default:
-	    this->xstream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-	      = strdup ("MPEG audio");
-	  }
+          /* the mpeg audio demuxer can set audio bitrate */
+          if (! _x_stream_info_get(this->xstream, XINE_STREAM_INFO_AUDIO_BITRATE)) {
+            _x_stream_info_set(this->xstream, XINE_STREAM_INFO_AUDIO_BITRATE,
+                               this->frame.header.bitrate);
+          }
+
+          /* the mpeg audio demuxer can set this meta info */
+          if (! _x_meta_info_get(this->xstream, XINE_META_INFO_AUDIOCODEC)) {
+            switch (this->frame.header.layer) {
+            case MAD_LAYER_I:
+              _x_meta_info_set_utf8(this->xstream, XINE_META_INFO_AUDIOCODEC, 
+                "MPEG audio layer 1 (lib: MAD)");
+              break;
+            case MAD_LAYER_II:
+              _x_meta_info_set_utf8(this->xstream, XINE_META_INFO_AUDIOCODEC, 
+                "MPEG audio layer 2 (lib: MAD)");
+              break;
+            case MAD_LAYER_III:
+              _x_meta_info_set_utf8(this->xstream, XINE_META_INFO_AUDIOCODEC, 
+                "MPEG audio layer 3 (lib: MAD)");
+              break;
+            default:
+              _x_meta_info_set_utf8(this->xstream, XINE_META_INFO_AUDIOCODEC, 
+                "MPEG audio (lib: MAD)");
+            }
+          }
 	  
 	  if (this->output_open) {
 	    this->xstream->audio_out->close (this->xstream->audio_out, this->xstream);
@@ -266,9 +272,8 @@ static void mad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	  buf->pts = 0;
 
 	}
-#ifdef LOG
-	printf ("libmad: decode worked\n"); 
-#endif
+
+	lprintf ("decode worked\n"); 
       }
     } 
 
@@ -295,7 +300,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 
   mad_decoder_t *this ;
 
-  this = (mad_decoder_t *) malloc (sizeof (mad_decoder_t));
+  this = (mad_decoder_t *) xine_xmalloc (sizeof (mad_decoder_t));
 
   this->audio_decoder.decode_data         = mad_decode_data;
   this->audio_decoder.reset               = mad_reset;
@@ -312,9 +317,9 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   mad_stream_init (&this->stream);
   mad_frame_init  (&this->frame);
 
-#ifdef LOG
-  printf ("libmad: init\n"); 
-#endif
+  this->stream.options = MAD_OPTION_IGNORECRC;
+
+  lprintf ("init\n"); 
 
   return &this->audio_decoder;
 }
@@ -339,7 +344,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   mad_class_t *this;
   
-  this = (mad_class_t *) malloc (sizeof (mad_class_t));
+  this = (mad_class_t *) xine_xmalloc (sizeof (mad_class_t));
 
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.get_identifier  = get_identifier;
@@ -355,11 +360,11 @@ static uint32_t audio_types[] = {
 
 static decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
-  5                    /* priority        */
+  7                    /* priority        */
 };
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_AUDIO_DECODER, 13, "mad", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
+  { PLUGIN_AUDIO_DECODER, 15, "mad", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
