@@ -1,4 +1,4 @@
-/*
+ /*
  * decode.c
  * Copyright (C) 2000-2002 Michel Lespinasse <walken@zoy.org>
  * Copyright (C) 1999-2000 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
@@ -25,32 +25,35 @@
  */
 
 #include "config.h"
-
 #include <stdio.h>
 #include <string.h>	/* memcpy/memset, try to remove */
 #include <stdlib.h>
 #include <inttypes.h>
 
+#define LOG_MODULE "decode"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "video_out.h"
 #include "mpeg2.h"
 #include "mpeg2_internal.h"
 #include "xineutils.h"
+#include "xvmc_vld.h"
 
 /*
 #define LOG_PAN_SCAN
 */
 
-/*
-#define LOG
-*/
-
 /* #define BUFFER_SIZE (224 * 1024) */
 #define BUFFER_SIZE (1194 * 1024) /* new buffer size for mpeg2dec 0.2.1 */
 
 static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer);
 
-void mpeg2_init (mpeg2dec_t * mpeg2dec)
+void mpeg2_init (mpeg2dec_t * mpeg2dec, 
+		 xine_video_port_t * output)
 {
   static int do_init = 1;
   uint32_t mm_accel;
@@ -71,10 +74,13 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec)
 						(void**)&mpeg2dec->picture_base);
 
     mpeg2dec->shift = 0xffffff00;
+    mpeg2dec->new_sequence = 0;
     mpeg2dec->is_sequence_needed = 1;
+    mpeg2dec->is_wait_for_ip_frames = 2;
     mpeg2dec->frames_to_drop = 0;
     mpeg2dec->drop_frame = 0;
     mpeg2dec->in_slice = 0;
+    mpeg2dec->output = output;
     mpeg2dec->chunk_ptr = mpeg2dec->chunk_buffer;
     mpeg2dec->code = 0xb4;
     mpeg2dec->seek_mode = 0;
@@ -83,13 +89,23 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec)
 
     /* initialize substructures */
     mpeg2_header_state_init (mpeg2dec->picture);
+
+    if ( output->get_capabilities(output) & VO_CAP_XXMC) {
+      printf("libmpeg2: output port has XxMC capability\n");
+      mpeg2dec->frame_format = XINE_IMGFMT_XXMC;
+    } else if( output->get_capabilities(output) & VO_CAP_XVMC_MOCOMP) {
+      printf("libmpeg2: output port has XvMC capability\n");
+      mpeg2dec->frame_format = XINE_IMGFMT_XVMC;
+    } else {
+      mpeg2dec->frame_format = XINE_IMGFMT_YV12;
+    }
 }
 
 static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
 {
   switch (mpeg2dec->picture->frame_rate_code) {
   case 1: /* 23.976 fps */
-    frame->duration      = 3913;
+    frame->duration      = 3754;  /* actually it's 3753.75 */
     break;
   case 2: /* 24 fps */
     frame->duration      = 3750;
@@ -107,10 +123,10 @@ static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
     frame->duration      = 1800;
     break;
   case 7: /* 59.94 fps */
-    frame->duration      = 1525;
+    frame->duration      = 1502;  /* actually it's 1501.5 */
     break;
   case 8: /* 60 fps */
-    frame->duration      = 1509;
+    frame->duration      = 1500;
     break;
   default:
        /* printf ("invalid/unknown frame rate code : %d \n",
@@ -128,8 +144,7 @@ static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
 
   if( ((mpeg2dec->rff_pattern & 0xff) == 0xaa ||
       (mpeg2dec->rff_pattern & 0xff) == 0x55) &&
-      !mpeg2dec->picture->progressive_sequence &&
-       mpeg2dec->picture->progressive_frame ) {
+      !mpeg2dec->picture->progressive_sequence ) {
     /* special case for ntsc 3:2 pulldown */
     frame->duration += frame->duration/4;
   }
@@ -137,82 +152,97 @@ static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
   {  
     if( frame->repeat_first_field ) {
       if( !mpeg2dec->picture->progressive_sequence &&
-           mpeg2dec->picture->progressive_frame ) {
+           frame->progressive_frame ) {
         /* decoder should output 3 fields, so adjust duration to
            count on this extra field time */
         frame->duration += frame->duration/2;     
       } else if( mpeg2dec->picture->progressive_sequence ) {
         /* for progressive sequences the output should repeat the
            frame 1 or 2 times depending on top_field_first flag. */
-        frame->duration *= (mpeg2dec->picture->top_field_first)?3:2;
+        frame->duration *= (frame->top_field_first)?3:2;
       }
     }
   }
-  
+  _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, frame->duration);
   /*printf("mpeg2dec: rff=%u\n",frame->repeat_first_field);*/
 } 
 
+static double get_aspect_ratio(mpeg2dec_t *mpeg2dec)
+{
+  double ratio;
+  picture_t * picture = mpeg2dec->picture;
+  double mpeg1_pel_ratio[16] = {1.0 /* forbidden */,
+    1.0, 0.6735, 0.7031, 0.7615, 0.8055, 0.8437, 0.8935, 0.9157,
+    0.9815, 1.0255, 1.0695, 1.0950, 1.1575, 1.2015, 1.0 /*reserved*/ };
+
+  if( !picture->mpeg1 ) {
+    /* these hardcoded values are defined on mpeg2 standard for
+     * aspect ratio. other values are reserved or forbidden.  */
+    switch(picture->aspect_ratio_information) {
+    case 2:
+      ratio = 4.0/3.0;
+      break;
+    case 3:
+      ratio = 16.0/9.0;
+      break;
+    case 4:
+      ratio = 2.11/1.0;
+      break;
+    case 1:
+    default:
+      ratio = (double)picture->coded_picture_width/(double)picture->coded_picture_height;
+      break;
+    }
+  } else {
+    /* mpeg1 constants refer to pixel aspect ratio */
+    ratio = (double)picture->coded_picture_width/(double)picture->coded_picture_height;
+    ratio /= mpeg1_pel_ratio[picture->aspect_ratio_information];
+  }
+
+  return ratio;
+}
+
 static void remember_metainfo (mpeg2dec_t *mpeg2dec) {
 
   picture_t * picture = mpeg2dec->picture;
 
-  mpeg2dec->stream->stream_info[XINE_STREAM_INFO_VIDEO_WIDTH]  = picture->frame_width;
-  mpeg2dec->stream->stream_info[XINE_STREAM_INFO_VIDEO_HEIGHT] = picture->frame_height;
-
-  switch (picture->aspect_ratio_information) {
-  case XINE_VO_ASPECT_PAN_SCAN:
-  case XINE_VO_ASPECT_ANAMORPHIC:
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000 * 16.0 /9.0;
-    break;
-  case XINE_VO_ASPECT_DVB:         /* 2.11:1 */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000 * 2.11/1.0;
-    break;
-  case XINE_VO_ASPECT_SQUARE:      /* square pels */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000;
-    break;
-  default:
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000 * 4.0 / 3.0;
-    break;
-  }
+  _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_WIDTH, picture->display_width);
+  _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_HEIGHT, picture->display_height);
+  _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_RATIO,
+    ((double)10000 * get_aspect_ratio(mpeg2dec)));
 
   switch (mpeg2dec->picture->frame_rate_code) {
   case 1: /* 23.976 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 3913;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 3913);
     break;
   case 2: /* 24 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 3750;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 3750);
     break;
   case 3: /* 25 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 3600;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 3600);
     break;
   case 4: /* 29.97 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 3003;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 3003);
     break;
   case 5: /* 30 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 3000;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 3000);
     break;
   case 6: /* 50 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 1800;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 1800);
     break;
   case 7: /* 59.94 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 1525;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 1525);
     break;
   case 8: /* 60 fps */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 1509;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 1509);
     break;
   default:
        /* printf ("invalid/unknown frame rate code : %d \n",
                frame->frame_rate_code); */
-    mpeg2dec->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION]      = 3000;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, 3000);
   }
 
-  if (mpeg2dec->picture->mpeg1) 
-    mpeg2dec->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
-      = strdup ("MPEG-1");
-  else
-    mpeg2dec->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
-      = strdup ("MPEG-2");
-
+  _x_meta_info_set_utf8(mpeg2dec->stream, XINE_META_INFO_VIDEOCODEC, "MPEG (libmpeg2)");
 }
 
 
@@ -229,10 +259,12 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	mpeg2dec->pts = 0;
 	return 0;
       }
-    } else if (mpeg2dec->is_frame_needed && (code != 0x00)) {
+    }
+    if (mpeg2dec->is_frame_needed) {
       /* printf ("libmpeg2: waiting for frame start\n");  */
       mpeg2dec->pts = 0;
-      return 0;
+      if (mpeg2dec->picture->current_frame)
+        mpeg2dec->picture->current_frame->bad_frame = 1;
     }
 
     mpeg2_stats (code, buffer);
@@ -245,6 +277,38 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     
     if (is_frame_done && picture->current_frame != NULL) {
 
+      /*
+       * This frame completion code will move to a separate libmpeg2_accel.c file?
+       * int libmpeg2_accel_frame_completion(mpeg2dec_t *, picture_t *, int);
+       */
+
+      if (mpeg2dec->frame_format == XINE_IMGFMT_XXMC) {
+	xine_xxmc_t *xxmc = (xine_xxmc_t *) 
+	  picture->current_frame->accel_data;
+	switch(picture->current_frame->format) {
+	case XINE_IMGFMT_XXMC:
+	  switch(xxmc->acceleration) {
+	  case XINE_XVMC_ACCEL_VLD:
+	    mpeg2_xxmc_vld_frame_complete(mpeg2dec, picture, code);
+	    break;
+	  case XINE_XVMC_ACCEL_IDCT:
+	  case XINE_XVMC_ACCEL_MOCOMP:
+	    xxmc->decoded = !picture->current_frame->bad_frame;
+	    xxmc->proc_xxmc_flush( picture->current_frame );
+	    break;
+	  default:
+	    break;
+	  }
+	default:
+	  break;
+	}
+      }
+
+      /*
+       * End of frame completion code.
+       */
+
+
 	if (((picture->picture_structure == FRAME_PICTURE) ||
 	     (picture->second_field)) ) {
 	  
@@ -289,15 +353,11 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	  switch (picture->picture_coding_type) {
 	  case B_TYPE:
 	    
-#ifdef LOG
-	    printf ("libmpeg2: B-Frame\n");
-#endif
+	    lprintf ("B-Frame\n");
 
 	    if (mpeg2dec->frames_to_drop>1) {
-#ifdef LOG
-	      printf ("libmpeg2: dropping b-frame because frames_to_drop==%d\n",
-		      mpeg2dec->frames_to_drop);
-#endif
+	      lprintf ("dropping b-frame because frames_to_drop==%d\n",
+		       mpeg2dec->frames_to_drop);
 	      mpeg2dec->drop_frame = 1;
 	    } else if (!picture->forward_reference_frame || picture->forward_reference_frame->bad_frame 
 		       || !picture->backward_reference_frame || picture->backward_reference_frame->bad_frame) {
@@ -315,21 +375,20 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 		printf ("fw ref frame not there)\n");
 #endif
 	      mpeg2dec->drop_frame = 1;
+	    } else if (mpeg2dec->is_wait_for_ip_frames > 0) {
+	      lprintf("dropping b-frame because refs are invalid\n");
+	      mpeg2dec->drop_frame = 1;
 	    }
 	    break;
 	    
 	  case P_TYPE:
 	    
-#ifdef LOG
-	    printf ("libmpeg2: P-Frame\n");
-#endif
+	    lprintf ("P-Frame\n");
 
 	    if (mpeg2dec->frames_to_drop>2) {
 	      mpeg2dec->drop_frame = 1;
-#ifdef LOG
-	      printf ("libmpeg2: dropping p-frame because frames_to_drop==%d\n",
-		      mpeg2dec->frames_to_drop);
-#endif
+	      lprintf ("dropping p-frame because frames_to_drop==%d\n",
+		       mpeg2dec->frames_to_drop);
 	    } else if (!picture->backward_reference_frame || picture->backward_reference_frame->bad_frame) {
 	      mpeg2dec->drop_frame = 1;
 #ifdef LOG
@@ -338,18 +397,25 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	      else
 		printf ("libmpeg2: dropping p-frame because ref %d is bad\n", picture->backward_reference_frame->id);
 #endif
-	    }
+	    } else if (mpeg2dec->is_wait_for_ip_frames > 1) {
+	      lprintf("dropping p-frame because ref is invalid\n");
+	      mpeg2dec->drop_frame = 1;
+	    } else if (mpeg2dec->is_wait_for_ip_frames)
+	      mpeg2dec->is_wait_for_ip_frames--;
+	    
 	    break;
 	    
 	  case I_TYPE:
-#ifdef LOG
-	    printf ("libmpeg2: I-Frame\n");
-#endif
+	    lprintf ("I-Frame\n");
 	    /* for the sake of dvd menus, never drop i-frames
 	    if (mpeg2dec->frames_to_drop>4) {
 	      mpeg2dec->drop_frame = 1;
 	    }
 	    */
+	    
+	    if (mpeg2dec->is_wait_for_ip_frames)
+	      mpeg2dec->is_wait_for_ip_frames--;
+	    
 	    break;
 	  }
 	}
@@ -367,6 +433,11 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	    break;
 	}
 	if (mpeg2dec->force_aspect) picture->aspect_ratio_information = mpeg2dec->force_aspect;
+
+	if (mpeg2dec->is_sequence_needed ) {
+	    mpeg2dec->new_sequence = 1;
+	}
+
 	if (mpeg2dec->is_sequence_needed 
 	    || (picture->frame_width != picture->coded_picture_width)
 	    || (picture->frame_height != picture->coded_picture_height)) {
@@ -381,8 +452,14 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	    data.width = picture->coded_picture_width;
 	    data.height = picture->coded_picture_height;
 	    data.aspect = picture->aspect_ratio_information;
+	    data.pan_scan = mpeg2dec->force_pan_scan;
 	    xine_event_send(mpeg2dec->stream, &event);
 
+            _x_stream_info_set(mpeg2dec->stream,XINE_STREAM_INFO_VIDEO_WIDTH,
+              picture->display_width);
+	    _x_stream_info_set(mpeg2dec->stream,XINE_STREAM_INFO_VIDEO_HEIGHT,
+              picture->display_height);
+
 	    if (picture->forward_reference_frame &&
 		picture->forward_reference_frame != picture->current_frame &&
 		picture->forward_reference_frame != picture->backward_reference_frame)
@@ -426,7 +503,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 
 	if (!(mpeg2dec->in_slice)) {
 	    mpeg2dec->in_slice = 1;
-     
+
 	    if (picture->second_field) {
 	      if (picture->current_frame)
 		picture->current_frame->field(picture->current_frame, 
@@ -434,28 +511,50 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	      else
 		mpeg2dec->drop_frame = 1;		
 	    } else {
+		int flags = picture->picture_structure;
+		if (!picture->mpeg1) flags |= VO_INTERLACED_FLAG;
+		if (mpeg2dec->force_pan_scan) flags |= VO_PAN_SCAN_FLAG;
+		if (mpeg2dec->new_sequence) flags |= VO_NEW_SEQUENCE_FLAG;
+
 		if ( picture->current_frame && 
 		     picture->current_frame != picture->backward_reference_frame &&
 		     picture->current_frame != picture->forward_reference_frame ) {
 			picture->current_frame->free (picture->current_frame);
 		}
-		
-		if (picture->picture_coding_type == B_TYPE)
+		if (picture->picture_coding_type == B_TYPE) {
 		    picture->current_frame =
 		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
 						     picture->coded_picture_width,
 						     picture->coded_picture_height,
-						     picture->aspect_ratio_information,
-						     XINE_IMGFMT_YV12,
-						     picture->picture_structure);
-		else {
+						     get_aspect_ratio(mpeg2dec),
+						     mpeg2dec->frame_format,
+						     flags);
+		    /*
+		     * Move to libmpeg2_accel.c
+		     * int libmpeg2_accel_new_frame(mpeg2dec_t *, picture_t *)
+		     */
+		    mpeg2_xxmc_choose_coding(mpeg2dec, picture, get_aspect_ratio(mpeg2dec),
+					     flags);
+		    /*
+		     * End of new frame accel code.
+		     */
+		} else {
 		    picture->current_frame =
 		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
 						     picture->coded_picture_width,
 						     picture->coded_picture_height,
-						     picture->aspect_ratio_information,
-						     XINE_IMGFMT_YV12,
-						     (VO_PREDICTION_FLAG | picture->picture_structure));
+						     get_aspect_ratio(mpeg2dec),
+						     mpeg2dec->frame_format,
+						     flags);
+		    /*
+		     * Move to libmpeg2_accel.c
+		     * int libmpeg2_accel_new_frame(mpeg2dec_t *, picture_t *)
+		     */
+		    mpeg2_xxmc_choose_coding(mpeg2dec, picture, 
+					     get_aspect_ratio(mpeg2dec), flags);
+		    /*
+		     * End of new frame accel code.
+		     */
 		    if (picture->forward_reference_frame &&
 		        picture->forward_reference_frame != picture->backward_reference_frame)
 		      picture->forward_reference_frame->free (picture->forward_reference_frame);
@@ -464,29 +563,111 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 			picture->backward_reference_frame;
 		    picture->backward_reference_frame = picture->current_frame;
 		}
+
+		/*
+		 * Move to libmpeg2_accel.c
+		 * int libmpeg2_accel_new_sequence(mpeg2dec_t *, picture_t *)
+		 */
+
+		if(mpeg2dec->new_sequence) {
+		  switch(mpeg2dec->frame_format) {
+		    case XINE_IMGFMT_XXMC:
+		    case XINE_IMGFMT_XVMC: {
+		      xine_xvmc_t *xvmc = (xine_xvmc_t *) 
+		        picture->current_frame->accel_data;
+		      picture->mc = xvmc->macroblocks;
+		      mpeg2dec->new_sequence = 0;
+		      break;
+		    }
+		    default:
+		      break;
+		  }
+		}
+		/*
+		 * End of new sequence accel code.
+		 */
+
 		picture->current_frame->bad_frame          = 1;
 		picture->current_frame->drawn              = 0;
 		picture->current_frame->pts                = mpeg2dec->pts;
                 picture->current_frame->top_field_first    = picture->top_field_first;
                 picture->current_frame->repeat_first_field = picture->repeat_first_field;
+                picture->current_frame->progressive_frame  = picture->progressive_frame;
+                picture->current_frame->crop_right         = picture->coded_picture_width - picture->display_width;
+                picture->current_frame->crop_bottom        = picture->coded_picture_height - picture->display_height;
 
-#ifdef LOG
-		printf ("libmpeg2: decoding frame %d, type %s\n",
-			picture->current_frame->id, picture->picture_coding_type == I_TYPE ? "I" :
-			picture->picture_coding_type == P_TYPE ? "P" : "B");
-#endif
+                switch( picture->picture_coding_type ) {
+                  case I_TYPE:
+                    picture->current_frame->picture_coding_type = XINE_PICT_I_TYPE;
+                    break;
+                  case P_TYPE:
+                    picture->current_frame->picture_coding_type = XINE_PICT_P_TYPE;
+                    break;
+                  case B_TYPE:
+                    picture->current_frame->picture_coding_type = XINE_PICT_B_TYPE;
+                    break;
+                  case D_TYPE:
+                    picture->current_frame->picture_coding_type = XINE_PICT_D_TYPE;
+                    break;
+                }
+
+		lprintf ("decoding frame %d, type %s\n",
+			 picture->current_frame->id, picture->picture_coding_type == I_TYPE ? "I" :
+			 picture->picture_coding_type == P_TYPE ? "P" : "B");
 		mpeg2dec->pts = 0;
+		/*printf("Starting to decode frame %d\n",picture->current_frame->id);*/
 	    }
 	}
 
 	if (!mpeg2dec->drop_frame && picture->current_frame != NULL) {
-	  mpeg2_slice (picture, code, buffer);
+#ifdef DEBUG_LOG
+	  printf("slice target %08x past %08x future %08x\n",picture->current_frame,picture->forward_reference_frame,picture->backward_reference_frame);
+	  fflush(stdout);
+#endif
+	  /*
+	   * The below accelerated slice function choice will move to libmpeg2_accel.c ?
+	   * int libmpeg2_accel_slice(mpeg2dec_t *, picture_t *, int , char *)
+	   */
+
+	  switch( mpeg2dec->frame_format ) {
+	    case XINE_IMGFMT_XXMC:
+	    {
+	      xine_xxmc_t *xxmc = (xine_xxmc_t *) 
+		picture->current_frame->accel_data;
+	      switch(picture->current_frame->format) {
+	      case XINE_IMGFMT_XXMC:
+		switch(xxmc->acceleration) {
+		case XINE_XVMC_ACCEL_VLD:
+		  mpeg2_xxmc_slice(mpeg2dec, picture, code, buffer);
+		  break;
+		case XINE_XVMC_ACCEL_IDCT:
+		case XINE_XVMC_ACCEL_MOCOMP:
+		  mpeg2_xvmc_slice (mpeg2dec, picture, code, buffer);
+		  break;
+		default:
+		  mpeg2_slice (picture, code, buffer);
+		  break;
+		}
+		break;
+	      default:
+		mpeg2_slice (picture, code, buffer);
+		break;
+	      }
+	      break;
+	    }
+	    case XINE_IMGFMT_XVMC:
+	      mpeg2_xvmc_slice (mpeg2dec, picture, code, buffer);
+	      break;
+	    default:
+	      mpeg2_slice (picture, code, buffer);
+	      break;
+	  }
+	  /*
+	   * End of acceleration code.
+	   */
 
 	  if( picture->v_offset > picture->limit_y ) { 
 	    picture->current_frame->bad_frame = 0;
-#ifdef LOG
-	    printf("libmpeg2: frame %d successfuly decoded\n",picture->current_frame->id); 
-#endif
 	  }
 	}
     }
@@ -529,6 +710,7 @@ static inline uint8_t * copy_chunk (mpeg2dec_t * mpeg2dec,
 	    }
 	}
 	mpeg2dec->code = byte;
+	mpeg2dec->chunk_size = chunk_ptr - mpeg2dec->chunk_buffer - 3;
 	mpeg2dec->chunk_ptr = mpeg2dec->chunk_buffer;
 	mpeg2dec->shift = 0xffffff00;
 	return current;
@@ -577,6 +759,15 @@ void mpeg2_discontinuity (mpeg2dec_t * mpeg2dec) {
     picture->forward_reference_frame->pts = 0;
   if ( picture->backward_reference_frame )
     picture->backward_reference_frame->pts = 0;
+  
+  /*
+   * Move to libmpeg2_accel.c
+   * int libmpeg2_accel_discontinuity(mpeg2dec_t *);
+   */
+  mpeg2dec->xvmc_last_slice_code=-1;
+  /*
+   * End of discontinuity accel code.
+   */
 }
 
 void mpeg2_reset (mpeg2dec_t * mpeg2dec) {
@@ -588,9 +779,18 @@ void mpeg2_reset (mpeg2dec_t * mpeg2dec) {
   
   mpeg2_discontinuity(mpeg2dec);
   
-  if( !picture->mpeg1 ) 
-    mpeg2dec->is_sequence_needed = 1;
-  else {
+  if( !picture->mpeg1 ) {
+    mpeg2dec->is_wait_for_ip_frames = 2;
+
+    /* mark current frames as bad so they won't make to screen */
+    if ( picture->current_frame )
+      picture->current_frame->bad_frame=1;
+    if (picture->forward_reference_frame )
+      picture->forward_reference_frame->bad_frame=1;
+    if (picture->backward_reference_frame)
+      picture->backward_reference_frame->bad_frame=1;
+
+  } else {
     /* to free reference frames one also needs to fix slice.c to 
      * abort when they are NULL. unfortunately it seems to break
      * DVD menus.
@@ -614,13 +814,10 @@ void mpeg2_reset (mpeg2dec_t * mpeg2dec) {
   }
   
   mpeg2dec->in_slice = 0;
+  mpeg2dec->seek_mode = 1;
 
 }
 
-/* flush must never allocate any frame (get_frame/duplicate_frame).
- * it is called from inside video_out loop and frame allocation
- * may cause some (rare) deadlocks.
- */
 void mpeg2_flush (mpeg2dec_t * mpeg2dec) {
 
   picture_t *picture = mpeg2dec->picture;
@@ -631,10 +828,8 @@ void mpeg2_flush (mpeg2dec_t * mpeg2dec) {
   if (picture->current_frame && !picture->current_frame->drawn &&
       !picture->current_frame->bad_frame) {
     
-#ifdef LOG
-    printf ("libmpeg2: blasting out current frame %d on flush\n",
-	    picture->current_frame->id);
-#endif
+    lprintf ("blasting out current frame %d on flush\n",
+	     picture->current_frame->id);
     
     picture->current_frame->drawn = 1;
     get_frame_duration(mpeg2dec, picture->current_frame);
@@ -663,9 +858,7 @@ void mpeg2_close (mpeg2dec_t * mpeg2dec)
  
     if ( picture->current_frame ) {
       if( !picture->current_frame->drawn ) {
-#ifdef LOG
-        printf ("libmpeg2: blasting out current frame on close\n");
-#endif
+        lprintf ("blasting out current frame on close\n");
         picture->current_frame->pts = 0;
         get_frame_duration(mpeg2dec, picture->current_frame);
         picture->current_frame->draw (picture->current_frame, mpeg2dec->stream);
@@ -687,9 +880,7 @@ void mpeg2_close (mpeg2dec_t * mpeg2dec)
     
     if (picture->backward_reference_frame) {
       if( !picture->backward_reference_frame->drawn) {
-#ifdef LOG
-        printf ("libmpeg2: blasting out backward reference frame on close\n");
-#endif
+        lprintf ("blasting out backward reference frame on close\n");
         picture->backward_reference_frame->pts = 0;
         get_frame_duration(mpeg2dec, picture->backward_reference_frame);
         picture->backward_reference_frame->draw (picture->backward_reference_frame, mpeg2dec->stream);
@@ -744,6 +935,8 @@ void mpeg2_find_sequence_header (mpeg2dec_t * mpeg2dec,
       if (mpeg2dec->is_sequence_needed) {
         xine_event_t event;
         xine_format_change_data_t data;
+
+	mpeg2dec->new_sequence = 1;
 	
 	mpeg2dec->is_sequence_needed = 0;
 	picture->frame_width  = picture->coded_picture_width;
@@ -758,7 +951,13 @@ void mpeg2_find_sequence_header (mpeg2dec_t * mpeg2dec,
 	data.width = picture->coded_picture_width;
 	data.height = picture->coded_picture_height;
 	data.aspect = picture->aspect_ratio_information;
+	data.pan_scan = mpeg2dec->force_pan_scan;
 	xine_event_send(mpeg2dec->stream, &event);
+
+	_x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_WIDTH,
+          picture->display_width);
+	_x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_HEIGHT,
+          picture->display_height);
       }
     } else if (code == 0xb5) {	/* extension_start_code */
       if (mpeg2_header_extension (picture, mpeg2dec->chunk_buffer)) {
@@ -791,7 +990,7 @@ static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer)
       xine_format_change_data_t data;
       
       /* open the closed caption decoder first */
-      mpeg2dec->cc_dec = get_spu_decoder(mpeg2dec->stream, (BUF_SPU_CC >> 16) & 0xff);
+      mpeg2dec->cc_dec = _x_get_spu_decoder(mpeg2dec->stream, (BUF_SPU_CC >> 16) & 0xff);
       
       /* send a frame format event so that the CC decoder knows the initial image size */
       event.type = XINE_EVENT_FRAME_FORMAT_CHANGE;
@@ -801,7 +1000,13 @@ static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer)
       data.width = mpeg2dec->picture->coded_picture_width;
       data.height = mpeg2dec->picture->coded_picture_height;
       data.aspect = mpeg2dec->picture->aspect_ratio_information;
+      data.pan_scan = mpeg2dec->force_pan_scan;
       xine_event_send(mpeg2dec->stream, &event);
+
+      _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_WIDTH,
+        mpeg2dec->picture->display_width);
+      _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_HEIGHT,
+        mpeg2dec->picture->display_height);
     }
     
     if (mpeg2dec->cc_dec) {
