@@ -21,12 +21,20 @@
 
 /* Structure for the mpeg2dec decoder */
 
+#ifndef MPEG2_H
+#define MPEG2_H
+
 typedef struct mpeg2dec_s {
+    xine_video_port_t * output;
+    uint32_t frame_format;
+
     /* this is where we keep the state of the decoder */
     struct picture_s * picture, *picture_base;
     
     uint32_t shift;
+    int new_sequence;
     int is_sequence_needed;
+    int is_wait_for_ip_frames;
     int frames_to_drop, drop_frame;
     int in_slice;
     int seek_mode, is_frame_needed;
@@ -40,21 +48,25 @@ typedef struct mpeg2dec_s {
     uint8_t * chunk_ptr;
     /* last start code ? */
     uint8_t code;
+    uint32_t chunk_size;
 
     int64_t pts;
     uint32_t rff_pattern; 
     int force_aspect;
+    int force_pan_scan;
 
     xine_stream_t *stream;
     
     /* a spu decoder for possible closed captions */
     spu_decoder_t *cc_dec;
-    
+    int xvmc_last_slice_code;
+    unsigned xxmc_mb_pic_height;
 } mpeg2dec_t ;
 
 
 /* initialize mpegdec with a opaque user pointer */
-void mpeg2_init (mpeg2dec_t * mpeg2dec);
+void mpeg2_init (mpeg2dec_t * mpeg2dec, 
+		 xine_video_port_t * output);
 
 /* destroy everything which was allocated, shutdown the output */
 void mpeg2_close (mpeg2dec_t * mpeg2dec);
@@ -74,3 +86,5 @@ void mpeg2_discontinuity (mpeg2dec_t * mpeg2dec);
  * currently
  */
 /* void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer); */
+
+#endif
