@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: ldt_keeper.c,v 1.5 2002/12/18 04:00:53 guenter Exp $
+ * $Id: ldt_keeper.c,v 1.13 2004/09/21 19:35:57 hadess Exp $
  *
  *
  * contents:
@@ -66,6 +66,11 @@
 #ifdef __linux__
 #include <asm/unistd.h>
 #include <asm/ldt.h>
+/* 2.5.xx+ calls this user_desc: */
+#include <linux/version.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,47)
+#define modify_ldt_ldt_s user_desc
+#endif
 /* prototype it here, so we won't depend on kernel headers */
 #ifdef  __cplusplus
 extern "C" {
@@ -124,17 +129,19 @@ struct modify_ldt_ldt_s {
 /* user level (privilege level: 3) ldt (1<<2) segment selector */
 #define       LDT_SEL(idx) ((idx) << 3 | 1 << 2 | 3)
 
-/* i got this value from wine sources, it's the first free LDT entry */
+/*
+ * linuxthreads can use all LDT entries from 0 to PTHREAD_THREADS_MAX-1.
+ * by default PTHREAD_THREADS_MAX = 1024, so unless one has recompiled
+ * it's own glibc/linuxthreads this should be a safe value.
+ */
 #ifndef       TEB_SEL_IDX
-#define       TEB_SEL_IDX     17
+#define       TEB_SEL_IDX     1024
 #endif
 
-#define       TEB_SEL LDT_SEL(TEB_SEL_IDX)
+static unsigned int teb_sel = LDT_SEL(TEB_SEL_IDX);
 
-/**
- * here is a small logical problem with Restore for multithreaded programs -
- * in C++ we use static class for this...
- */
+static ldt_fs_t global_ldt_fs;
+static int      global_usage_count = 0;
 
 #ifdef __cplusplus
 extern "C"
@@ -142,49 +149,25 @@ extern "C"
 void Setup_FS_Segment(void)
 {
     __asm__ __volatile__(
-	"movl %0,%%eax; movw %%ax, %%fs" : : "i" (TEB_SEL)
+	"movl %0,%%eax; movw %%ax, %%fs" : : "r" (teb_sel) : "%eax"
     );
 }
 
-/* (just in case someday we need dynamic entry indexes) 
-__ASM_GLOBAL_FUNC( __set_fs, "movl 4(%esp),%eax\n\tmovw %ax,%fs\n\tret" )
-*/
-
-/* we don't need this - use modify_ldt instead */
-#if 0
-#ifdef __linux__
-/* XXX: why is this routine from libc redefined here? */
-/* NOTE: the redefined version ignores the count param, count is hardcoded as 16 */
-static int LDT_Modify( int func, struct modify_ldt_ldt_s *ptr,
-		       unsigned long count )
+void Check_FS_Segment(void)
 {
-    int res;
-#ifdef __PIC__
-    __asm__ __volatile__( "pushl %%ebx\n\t"
-			  "movl %2,%%ebx\n\t"
-			  "int $0x80\n\t"
-			  "popl %%ebx"
-			  : "=a" (res)
-			  : "0" (__NR_modify_ldt),
-			  "r" (func),
-			  "c" (ptr),
-			  "d"(16)//sizeof(*ptr) from kernel point of view
-			  :"esi"     );
-#else
-    __asm__ __volatile__("int $0x80"
-			 : "=a" (res)
-			 : "0" (__NR_modify_ldt),
-			 "b" (func),
-			 "c" (ptr),
-			 "d"(16)
-			 :"esi");
-#endif  /* __PIC__ */
-    if (res >= 0) return res;
-    errno = -res;
-    return -1;
+    int fs;
+     __asm__ __volatile__(
+	"movw %%fs,%%ax; mov %%eax,%0" : "=r" (fs) :: "%eax"
+    );
+    fs = fs & 0xffff;
+    
+    if( fs != teb_sel ) {
+      printf("ldt_keeper: FS segment is not set or has being lost!\n");
+      printf("            Please report this error to xine-devel@lists.sourceforge.net\n");
+      printf("            Aborting....\n");
+      abort();
+    }
 }
-#endif
-#endif
 
 #if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)
 static void LDT_EntryToBytes( unsigned long *buffer, const struct modify_ldt_ldt_s *content )
@@ -202,43 +185,11 @@ static void LDT_EntryToBytes( unsigned long *buffer, const struct modify_ldt_ldt
 }
 #endif
 
-void* fs_seg=0;
-
-ldt_fs_t* Setup_LDT_Keeper(void)
+static int _modify_ldt(struct modify_ldt_ldt_s array)
 {
-    struct modify_ldt_ldt_s array;
     int ret;
-    ldt_fs_t* ldt_fs = (ldt_fs_t*) malloc(sizeof(ldt_fs_t));
 
-    if (!ldt_fs)
-	return NULL;
-
-    ldt_fs->fd = open("/dev/zero", O_RDWR);
-    if(ldt_fs->fd<0){
-        perror( "Cannot open /dev/zero for READ+WRITE. Check permissions! error: ");
-	return NULL;
-    }
-    fs_seg=
-    ldt_fs->fs_seg = mmap(NULL, getpagesize(), PROT_READ | PROT_WRITE, MAP_PRIVATE,
-			  ldt_fs->fd, 0);
-    if (ldt_fs->fs_seg == (void*)-1)
-    {
-	perror("ERROR: Couldn't allocate memory for fs segment");
-        close(ldt_fs->fd);
-        free(ldt_fs);
-	return NULL;
-    }
-    *(void**)((char*)ldt_fs->fs_seg+0x18) = ldt_fs->fs_seg;
-    array.base_addr=(int)ldt_fs->fs_seg;
-    array.entry_number=TEB_SEL_IDX;
-    array.limit=array.base_addr+getpagesize()-1;
-    array.seg_32bit=1;
-    array.read_exec_only=0;
-    array.seg_not_present=0;
-    array.contents=MODIFY_LDT_CONTENTS_DATA;
-    array.limit_in_pages=0;
 #ifdef __linux__
-    //ret=LDT_Modify(0x1, &array, sizeof(struct modify_ldt_ldt_s));
     ret=modify_ldt(0x1, &array, sizeof(struct modify_ldt_ldt_s));
     if(ret<0)
     {
@@ -252,7 +203,13 @@ ldt_fs_t* Setup_LDT_Keeper(void)
         unsigned long d[2];
 
         LDT_EntryToBytes( d, &array );
+#if defined(__FreeBSD__) && defined(LDT_AUTO_ALLOC)
+        ret = i386_set_ldt(LDT_AUTO_ALLOC, (union descriptor *)d, 1);
+        array.entry_number = ret;
+        teb_sel = LDT_SEL(ret);
+#else
         ret = i386_set_ldt(array.entry_number, (union descriptor *)d, 1);
+#endif
         if (ret < 0)
         {
             perror("install_fs");
@@ -266,9 +223,9 @@ ldt_fs_t* Setup_LDT_Keeper(void)
 #if defined(__svr4__)
     {
 	struct ssd ssd;
-	ssd.sel = TEB_SEL;
+	ssd.sel = teb_sel;
 	ssd.bo = array.base_addr;
-	ssd.ls = array.limit - array.base_addr;
+	ssd.ls = array.limit;
 	ssd.acc1 = ((array.read_exec_only == 0) << 1) |
 	    (array.contents << 2) |
 	    0xf0;   /* P(resent) | DPL3 | S */
@@ -280,22 +237,131 @@ ldt_fs_t* Setup_LDT_Keeper(void)
     }
 #endif
 
+    return ret;
+}
+
+ldt_fs_t* Setup_LDT_Keeper(void)
+{
+    struct modify_ldt_ldt_s array;
+    int ret;
+    int ldt_already_set = 0;
+    ldt_fs_t* ldt_fs = (ldt_fs_t*) malloc(sizeof(ldt_fs_t));
+
+    if (!ldt_fs)
+	return NULL;
+
+#ifdef __linux__
+    /*
+     * LDT might be shared by different threads, so we must
+     * check it here to avoid filling the segment descriptor again.
+     */
+    {
+        unsigned char *ldt = malloc((TEB_SEL_IDX+1)*8);
+        unsigned int limit;
+
+        memset (ldt, 0, (TEB_SEL_IDX+1)*8);
+        modify_ldt(0, ldt, (TEB_SEL_IDX+1)*8);
+/*        
+        printf("ldt_keeper: old LDT entry = [%x] [%x]\n",
+                *(unsigned int *) (&ldt[TEB_SEL_IDX*8]),
+                *(unsigned int *) (&ldt[TEB_SEL_IDX*8+4]) );
+*/                
+        limit = ((*(unsigned int *) (&ldt[TEB_SEL_IDX*8])) & 0xffff) |
+                ((*(unsigned int *) (&ldt[TEB_SEL_IDX*8+4])) & 0xf0000);
+        
+        if( limit ) {
+            if( limit == getpagesize()-1 ) {
+                ldt_already_set = 1;
+            } else {
+#ifdef LOG
+                printf("ldt_keeper: LDT entry seems to be used by someone else. [%x] [%x]\n",
+                       *(unsigned int *) (&ldt[TEB_SEL_IDX*8]),
+                       *(unsigned int *) (&ldt[TEB_SEL_IDX*8+4]) );
+                printf("            Please report this message to xine-devel@lists.sourceforge.net\n");
+#endif        
+            }
+        }
+        free(ldt);
+    }
+#endif /*linux*/
+
+    if( !ldt_already_set )
+    {
+#ifdef LOG
+        printf("ldt_keeper: creating a new segment descriptor.\n");
+#endif        
+        ldt_fs->fd = open("/dev/zero", O_RDWR);
+        if(ldt_fs->fd<0){
+            perror( "Cannot open /dev/zero for READ+WRITE. Check permissions! error: ");
+	    return NULL;
+        }
+    
+        ldt_fs->fs_seg = mmap(NULL, getpagesize(), PROT_READ | PROT_WRITE, MAP_PRIVATE,
+			      ldt_fs->fd, 0);
+        if (ldt_fs->fs_seg == (void*)-1)
+        {
+	    perror("ERROR: Couldn't allocate memory for fs segment");
+            close(ldt_fs->fd);
+            free(ldt_fs);
+	    return NULL;
+        }
+        *(void**)((char*)ldt_fs->fs_seg+0x18) = ldt_fs->fs_seg;
+        memset (&array, 0, sizeof (array));
+        array.base_addr=(int)ldt_fs->fs_seg;
+        array.entry_number=TEB_SEL_IDX;
+        array.limit=getpagesize()-1;
+        array.seg_32bit=1;
+        array.read_exec_only=0;
+        array.seg_not_present=0;
+        array.contents=MODIFY_LDT_CONTENTS_DATA;
+        array.limit_in_pages=0;
+    
+        ret = _modify_ldt(array);
+        
+        ldt_fs->prev_struct = (char*)malloc(sizeof(char) * 8);
+        *(void**)array.base_addr = ldt_fs->prev_struct;
+        
+        memcpy( &global_ldt_fs, ldt_fs, sizeof(ldt_fs_t) );
+    } else {
+#ifdef LOG
+        printf("ldt_keeper: LDT entry already set, reusing.\n");
+#endif        
+        global_usage_count++;
+        memcpy( ldt_fs, &global_ldt_fs, sizeof(ldt_fs_t) );
+    }
+    
     Setup_FS_Segment();
-
-    ldt_fs->prev_struct = (char*)malloc(sizeof(char) * 8);
-    *(void**)array.base_addr = ldt_fs->prev_struct;
-
+    
     return ldt_fs;
 }
 
 void Restore_LDT_Keeper(ldt_fs_t* ldt_fs)
 {
+    struct modify_ldt_ldt_s array;
+    
     if (ldt_fs == NULL || ldt_fs->fs_seg == 0)
 	return;
-    if (ldt_fs->prev_struct)
-	free(ldt_fs->prev_struct);
-    munmap((char*)ldt_fs->fs_seg, getpagesize());
-    ldt_fs->fs_seg = 0;
-    close(ldt_fs->fd);
+
+    if( global_usage_count ) {
+#ifdef LOG
+        printf("ldt_keeper: shared LDT, restore does nothing.\n");
+#endif        
+        /* shared LDT. only the last user can free. */
+        global_usage_count--; 
+    } else {
+#ifdef LOG
+        printf("ldt_keeper: freeing LDT entry.\n");
+#endif        
+        if (ldt_fs->prev_struct)
+            free(ldt_fs->prev_struct);
+        munmap((char*)ldt_fs->fs_seg, getpagesize());
+        ldt_fs->fs_seg = 0;
+        close(ldt_fs->fd);
+    
+        /* mark LDT entry as free again */
+        memset(&array, 0, sizeof(struct modify_ldt_ldt_s));
+        array.entry_number=TEB_SEL_IDX;
+        _modify_ldt(array);
+    }
     free(ldt_fs);
 }
