@@ -130,8 +130,10 @@ static WIN_BOOL MODULE_InitDll( WINE_MODREF *wm, DWORD type, LPVOID lpReserved )
 {
     WIN_BOOL retv = TRUE;
 
+#ifdef LOG
     static LPCSTR typeName[] = { "PROCESS_DETACH", "PROCESS_ATTACH",
                                  "THREAD_ATTACH", "THREAD_DETACH" };
+#endif
     assert( wm );
 
 
@@ -142,8 +144,12 @@ static WIN_BOOL MODULE_InitDll( WINE_MODREF *wm, DWORD type, LPVOID lpReserved )
         return TRUE;
 
 
+#ifdef LOG
     TRACE("(%s,%s,%p) - CALL\n", wm->modname, typeName[type], lpReserved );
-
+#else
+    TRACE("(%s,%p) - CALL\n", wm->modname, lpReserved );
+#endif
+    
     /* Call the initialization routine */
     switch ( wm->type )
     {
@@ -164,8 +170,12 @@ static WIN_BOOL MODULE_InitDll( WINE_MODREF *wm, DWORD type, LPVOID lpReserved )
     /* The state of the module list may have changed due to the call
        to PE_InitDLL. We cannot assume that this module has not been
        deleted.  */
+#ifdef LOG
     TRACE("(%p,%s,%p) - RETURN %d\n", wm, typeName[type], lpReserved, retv );
-
+#else
+    TRACE("(%p,%p) - RETURN %d\n", wm, lpReserved, retv );
+#endif
+    
     return retv;
 }
 
@@ -353,7 +363,6 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 {
 	WINE_MODREF *wm = 0;
 	char* listpath[] = { "", "", "/usr/lib/win32", "/usr/local/lib/win32", 0 };
-	extern char* win32_def_path;
 	char path[512];
 	char checked[2000];
         int i = -1;
@@ -373,6 +382,8 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 
 	while (wm == 0 && listpath[++i])
 	{
+	    memset (&path, 0, sizeof (path));
+
 	    if (i < 2)
 	    {
 		if (i == 0)
@@ -420,6 +431,44 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 	if (!wm)
 	    printf("wine/module: Win32 LoadLibrary failed to load: %s\n", checked);
 
+        // remove a few divs in the VP codecs that make trouble
+        if (strstr(libname,"vp5vfw.dll") && wm)
+        {
+          int i;
+          if (PE_FindExportedFunction(wm, "DriverProc", TRUE)==(void*)0x10003930) {
+            for (i=0;i<3;i++) ((char*)0x10004e86)[i]=0x90;
+            for (i=0;i<3;i++) ((char*)0x10005a23)[i]=0x90;
+            for (i=0;i<3;i++) ((char*)0x10005bff)[i]=0x90;
+          } else {
+            printf("wine/module: Unsupported VP5 version\n");
+            return 0;
+          }
+        }
+
+        if (strstr(libname,"vp6vfw.dll") && wm)
+        {
+          int i;
+          if (PE_FindExportedFunction(wm, "DriverProc", TRUE)==(void*)0x10003ef0) {
+            // looks like VP 6.1.0.2
+            for (i=0;i<6;i++) ((char*)0x10007268)[i]=0x90;
+            for (i=0;i<6;i++) ((char*)0x10007e83)[i]=0x90;
+            for (i=0;i<6;i++) ((char*)0x1000806a)[i]=0x90;
+          } else if (PE_FindExportedFunction(wm, "DriverProc", TRUE)==(void*)0x10004120) {
+            // looks like VP 6.2.0.10
+            for (i=0;i<6;i++) ((char*)0x10007688)[i]=0x90;
+            for (i=0;i<6;i++) ((char*)0x100082c3)[i]=0x90;
+            for (i=0;i<6;i++) ((char*)0x100084aa)[i]=0x90;
+          } else if (PE_FindExportedFunction(wm, "DriverProc", TRUE)==(void*)0x10003e70) {
+            // looks like VP 6.0.7.3
+            for (i=0;i<6;i++) ((char*)0x10007559)[i]=0x90;
+            for (i=0;i<6;i++) ((char*)0x100081c3)[i]=0x90;
+            for (i=0;i<6;i++) ((char*)0x1000839e)[i]=0x90;
+          } else {
+            printf("wine/module: Unsupported VP6 version\n");
+            return 0;
+          }
+        }
+
 	if (strstr(libname,"QuickTime.qts") && wm)
 	{
 	    void** ptr;
@@ -428,10 +477,10 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 
 //	    dispatch_addr = GetProcAddress(wm->module, "theQuickTimeDispatcher", TRUE);
 	    dispatch_addr = PE_FindExportedFunction(wm, "theQuickTimeDispatcher", TRUE);
-	    if (dispatch_addr == 0x62924c30)
+	    if (dispatch_addr == (void *)0x62924c30)
 	    {
 	        printf ("wine/module: QuickTime5 DLLs found\n");
-		ptr = 0x62b75ca4; // dispatch_ptr
+		ptr = (void **)0x62b75ca4; // dispatch_ptr
 	        for (i=0;i<5;i++)  ((char*)0x6299e842)[i]=0x90; // make_new_region ?
 	        for (i=0;i<28;i++) ((char*)0x6299e86d)[i]=0x90; // call__call_CreateCompatibleDC ?
 		for (i=0;i<5;i++)  ((char*)0x6299e898)[i]=0x90; // jmp_to_call_loadbitmap ?
@@ -456,20 +505,29 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 		((char *)0x6288e0ae)[0] = 0xc3; // font/dc remover
 		for (i=0;i<24;i++) ((char*)0x6287a1ad)[i]=0x90; // destroy window
 #endif
-	    } else if (dispatch_addr == 0x6693b330)
+	    } else if (dispatch_addr == (void *)0x6693b330)
 	    {
 	        printf ("wine/module: QuickTime6 DLLs found\n");
-		ptr = 0x66bb9524; // dispatcher_ptr
+		ptr = (void **)0x66bb9524; // dispatcher_ptr
 		for (i=0;i<5;i++)  ((char *)0x66a730cc)[i]=0x90; // make_new_region
 		for (i=0;i<28;i++) ((char *)0x66a730f7)[i]=0x90; // call__call_CreateCompatibleDC
 		for (i=0;i<5;i++)  ((char *)0x66a73122)[i]=0x90; // jmp_to_call_loadbitmap
 		for (i=0;i<9;i++)  ((char *)0x66a73131)[i]=0x90; // call__calls_OLE_shit
 		for (i=0;i<96;i++) ((char *)0x66aac852)[i]=0x90; // disable threads
+	    } else if (dispatch_addr == (void *)0x6693c3e0)
+	    {
+    		printf ("wine/module: QuickTime6.3 DLLs found\n");
+		ptr = (void **)0x66bca01c; // dispatcher_ptr
+		for (i=0;i<5;i++)  ((char *)0x66a68f6c)[i]=0x90; // make_new_region
+		for (i=0;i<28;i++) ((char *)0x66a68f97)[i]=0x90; // call__call_CreateCompatibleDC
+		for (i=0;i<5;i++)  ((char *)0x66a68fc2)[i]=0x90; // jmp_to_call_loadbitmap
+		for (i=0;i<9;i++)  ((char *)0x66a68fd1)[i]=0x90; // call__calls_OLE_shit
+		for (i=0;i<96;i++) ((char *)0x66ab4722)[i]=0x90; // disable threads
 	    } else
 	    {
-	        printf ("wine/module: Unsupported QuickTime version (0x%x)\n",
+	        printf ("wine/module: Unsupported QuickTime version (%p)\n",
 			dispatch_addr);
-		return NULL;
+		return 0;
 	    }
 
 	    printf ("wine/module: QuickTime.qts patched!!! old entry=%p\n",ptr[0]);
@@ -722,15 +780,15 @@ static int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_
   // memory management:
   case 0x150011: //NewPtrClear
   case 0x150012: //NewPtrSysClear
-      reg->eax=malloc(((u_int32_t *)stack_base)[1]);
-      memset(reg->eax,0,((u_int32_t *)stack_base)[1]);
+      reg->eax=(u_int32_t)malloc(((u_int32_t *)stack_base)[1]);
+      memset((void *)reg->eax,0,((u_int32_t *)stack_base)[1]);
 #ifdef DEBUG_QTX_API
       printf("%*sLEAVE(%d): EMULATED! 0x%X\n",ret_i*2,"",ret_i, reg->eax);
 #endif
       return 1;
   case 0x15000F: //NewPtr
   case 0x150010: //NewPtrSys
-      reg->eax=malloc(((u_int32_t *)stack_base)[1]);
+      reg->eax=(u_int32_t)malloc(((u_int32_t *)stack_base)[1]);
 #ifdef DEBUG_QTX_API
       printf("%*sLEAVE(%d): EMULATED! 0x%X\n",ret_i*2,"",ret_i, reg->eax);
 #endif
@@ -739,7 +797,7 @@ static int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_
       if(((u_int32_t *)stack_base)[1]>=0x60000000)
           printf("WARNING! Invalid Ptr handle!\n");
       else
-          free(((u_int32_t *)stack_base)[1]);
+          free((void *)(((u_int32_t *)stack_base)[1]));
       reg->eax=0;
 #ifdef DEBUG_QTX_API
       printf("%*sLEAVE(%d): EMULATED! 0x%X\n",ret_i*2,"",ret_i, reg->eax);
@@ -840,8 +898,10 @@ static int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_
 
 static int report_func_ret(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags)
 {
+#ifdef DEBUG_QTX_API
   int i;
   short err;
+#endif
 
   // restore ret addr:
   --ret_i;
@@ -939,7 +999,7 @@ FARPROC MODULE_GetProcAddress(
 #ifdef DEBUG_QTX_API
 #define DECL_COMPONENT(sname,name,type) \
     if(!strcmp(function,name)){ \
-	fprintf(stderr,name "dispatcher catched -> %p\n",retproc); \
+	fprintf(stderr,name "dispatcher caught -> %p\n",retproc); \
 	real_ ## sname = retproc; retproc = fake_ ## sname; \
     }
 #include "qt_comp.h"
@@ -950,11 +1010,11 @@ FARPROC MODULE_GetProcAddress(
 //      || !strcmp(function,"_CallComponentFunctionWithStorage")
 //      || !strcmp(function,"_CallComponent")
       ){
-	fprintf(stderr,"theQuickTimeDispatcher catched -> %p\n",retproc);
+	fprintf(stderr,"theQuickTimeDispatcher caught -> %p\n",retproc);
       report_entry = report_func;
       report_ret   = report_func_ret;
-      wrapper_target=retproc;
-      retproc=wrapper;
+      wrapper_target=(void *)retproc;
+      retproc=(void *)wrapper;
     }
 
     }
