@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -7,17 +7,17 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * xine is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer.h,v 1.98 2003/01/26 23:36:46 f1rmb Exp $
+ * $Id: buffer.h,v 1.145 2004/12/12 22:01:30 mroi Exp $
  *
  *
  * contents:
@@ -44,9 +44,17 @@ extern "C" {
 
 #include <stdio.h>
 #include <pthread.h>
-#include <inttypes.h>
 #include <sys/types.h>
-#include "attributes.h"
+
+#ifdef XINE_COMPILE
+#  include <inttypes.h>
+#  include "attributes.h"
+#else
+#  include <xine/os_types.h>
+#  include <xine/attributes.h>
+#endif
+
+#define BUF_MAX_CALLBACKS 5
 
 /*
  * buffer types
@@ -142,7 +150,24 @@ extern "C" {
 #define BUF_VIDEO_RV40		0x02390000
 #define BUF_VIDEO_PSX_MDEC	0x023A0000
 #define BUF_VIDEO_YUV_FRAMES	0x023B0000 /* uncompressed YUV, delivered by v4l input plugin */
-
+#define BUF_VIDEO_HUFFYUV	0x023C0000
+#define BUF_VIDEO_IMAGE		0x023D0000
+#define BUF_VIDEO_THEORA        0x023E0000
+#define BUF_VIDEO_4XM           0x023F0000
+#define BUF_VIDEO_I420		0x02400000
+#define BUF_VIDEO_VP4           0x02410000
+#define BUF_VIDEO_VP5           0x02420000
+#define BUF_VIDEO_VP6           0x02430000
+#define BUF_VIDEO_VMD		0x02440000
+#define BUF_VIDEO_MSZH		0x02450000
+#define BUF_VIDEO_ZLIB		0x02460000
+#define BUF_VIDEO_8BPS		0x02470000
+#define BUF_VIDEO_ASV1		0x02480000
+#define BUF_VIDEO_ASV2		0x02490000
+#define BUF_VIDEO_BITPLANE	0x024A0000 /* Amiga typical picture and animation format */
+#define BUF_VIDEO_BITPLANE_BR1	0x024B0000 /* the same with Bytrun compression 1 */
+#define BUF_VIDEO_FLV1		0x024C0000
+#define BUF_VIDEO_H264		0x024D0000
 
 /* audio buffer types:  (please keep in sync with buffer_types.c) */
 
@@ -156,7 +181,7 @@ extern "C" {
 #define BUF_AUDIO_DTS		0x03050000
 #define BUF_AUDIO_MSADPCM	0x03060000
 #define BUF_AUDIO_MSIMAADPCM	0x03070000
-#define BUF_AUDIO_MSGSM		0x03080000 
+#define BUF_AUDIO_MSGSM		0x03080000
 #define BUF_AUDIO_VORBIS        0x03090000
 #define BUF_AUDIO_IMC           0x030a0000
 #define BUF_AUDIO_LH            0x030b0000
@@ -193,24 +218,57 @@ extern "C" {
 #define BUF_AUDIO_DIALOGIC_IMA	0x032A0000
 #define BUF_AUDIO_NSF		0x032B0000
 #define BUF_AUDIO_FLAC		0x032C0000
+#define BUF_AUDIO_DV		0x032D0000
+#define BUF_AUDIO_WMAV		0x032E0000
+#define BUF_AUDIO_SPEEX		0x032F0000
+#define BUF_AUDIO_RAWPCM	0x03300000
+#define BUF_AUDIO_4X_ADPCM	0x03310000
+#define BUF_AUDIO_VMD		0x03320000
+#define BUF_AUDIO_XAN_DPCM	0x03330000
+#define BUF_AUDIO_ALAC		0x03340000
 
 /* spu buffer types:    */
- 
+
 #define BUF_SPU_BASE		0x04000000
 #define BUF_SPU_DVD		0x04000000
 #define BUF_SPU_TEXT            0x04010000
 #define BUF_SPU_CC              0x04020000
 #define BUF_SPU_DVB             0x04030000
+#define BUF_SPU_SVCD            0x04040000
+#define BUF_SPU_CVD             0x04050000
+#define BUF_SPU_OGM             0x04060000
+#define BUF_SPU_CMML            0x04070000
 
 /* demuxer block types: */
 
 #define BUF_DEMUX_BLOCK		0x05000000
 
-/* to access extra_info_t contents one have to include xine_internal.h */
-#ifndef EXTRA_INFO
-#define EXTRA_INFO
 typedef struct extra_info_s extra_info_t;
-#endif
+
+/*
+ * extra_info_t is used to pass information from input or demuxer plugins
+ * to output frames (past decoder). new data must be added after the existing
+ * fields for backward compatibility.
+ */
+  
+struct extra_info_s {
+
+  int                   input_normpos; /* remember where this buf came from in
+                                        * the input source (0..65535). can be
+                                        * either time or offset based. */
+  int                   input_time;    /* time offset in miliseconds from
+                                        * beginning of stream */
+  uint32_t              frame_number;  /* number of current frame if known */
+  
+  int                   seek_count;    /* internal engine use */
+  int64_t               vpts;          /* set on output layers only */ 
+  
+  int                   invalid;       /* do not use this extra info to update anything */
+  int                   total_time;    /* duration in miliseconds of the stream */
+};
+
+
+#define BUF_NUM_DEC_INFO 4
 
 typedef struct buf_element_s buf_element_t;
 struct buf_element_s {
@@ -220,17 +278,19 @@ struct buf_element_s {
   unsigned char        *content;   /* start of raw content in mem (without header etc) */
 
   int32_t               size ;     /* size of _content_                                     */
-  int32_t               max_size;  /* size of pre-allocated memory pointed to by "mem"      */ 
+  int32_t               max_size;  /* size of pre-allocated memory pointed to by "mem"      */
   uint32_t              type;
   int64_t               pts;       /* presentation time stamp, used for a/v sync            */
   int64_t               disc_off;  /* discontinuity offset                                  */
 
   extra_info_t         *extra_info; /* extra info will be passed to frames */
-  
+
   uint32_t              decoder_flags; /* stuff like keyframe, is_header ... see below      */
 
-  uint32_t              decoder_info[4]; /* additional decoder flags and other dec-spec. stuff */
-  void                 *decoder_info_ptr[4]; /* pointers to dec-spec. stuff */
+                        /* additional decoder flags and other dec-spec. stuff */
+  uint32_t              decoder_info[BUF_NUM_DEC_INFO]; 
+                        /* pointers to dec-spec. stuff */
+  void                 *decoder_info_ptr[BUF_NUM_DEC_INFO];
 
   void (*free_buffer) (buf_element_t *buf);
 
@@ -239,19 +299,48 @@ struct buf_element_s {
 
 } ;
 
+/* keyframe should be set whenever possible (that is, when demuxer
+ * knows about frames and keyframes).                                 */
 #define BUF_FLAG_KEYFRAME    0x0001
+
+/* frame start/end. BUF_FLAG_FRAME_END is sent on last buf of a frame */
 #define BUF_FLAG_FRAME_START 0x0002
 #define BUF_FLAG_FRAME_END   0x0004
+
+/* any out-of-band data needed to initialize decoder must have
+ * this flag set.                                                     */
 #define BUF_FLAG_HEADER      0x0008
+
+/* preview buffers are normal data buffers that must not produce any
+ * output in decoders (may be used to sneak details about the stream
+ * to come).                                                          */
 #define BUF_FLAG_PREVIEW     0x0010
+
+/* set when user stop the playback                                    */
 #define BUF_FLAG_END_USER    0x0020
+
+/* set when stream finished naturaly                                  */
 #define BUF_FLAG_END_STREAM  0x0040
+
+/* decoder_info[0] carries the frame step (1/90000).                  */
 #define BUF_FLAG_FRAMERATE   0x0080
+
+/* hint to metronom that seeking has occurred                         */
 #define BUF_FLAG_SEEK        0x0100
+
+/* special information inside, see below.                             */
 #define BUF_FLAG_SPECIAL     0x0200
-#define BUF_FLAG_NO_VIDEO    0x0400
-/* do not decode the buffer contents, detect framing only */
-#define BUF_FLAG_FRAMING     0x0800 
+
+/* header use standard xine_bmiheader or xine_waveformatex structs.
+ * xine_waveformatex is actually optional since the most important
+ * information for audio init is available from decoder_info[].
+ * note: BUF_FLAG_HEADER must also be set.                            */
+#define BUF_FLAG_STDHEADER   0x0400 
+
+/* decoder_info[1] carries numerator for display aspect ratio
+ * decoder_info[2] carries denominator for display aspect ratio       */
+#define BUF_FLAG_ASPECT      0x0800
+
 
 /* Special buffer types:
  * Sometimes there is a need to relay special information from a demuxer
@@ -262,7 +351,7 @@ struct buf_element_s {
  *
  * The interface was designed in a way that did not require an API
  * version bump. To send a special buffer type, set a buffer's flags field
- * to BUF_SPECIAL_PALETTE. Set the buffer's decoder_info[1] field to a
+ * to BUF_FLAG_SPECIAL. Set the buffer's decoder_info[1] field to a
  * number according to one of the special buffer subtypes defined below.
  * The second and third decoder_info[] fields are defined according to
  * your buffer type's requirements.
@@ -275,7 +364,7 @@ struct buf_element_s {
 
 /* these are the types of special buffers */
 
-/* 
+/*
  * In a BUF_SPECIAL_PALETTE buffer:
  * decoder_info[1] = BUF_SPECIAL_PALETTE
  * decoder_info[2] = number of entries in palette table
@@ -289,23 +378,14 @@ struct buf_element_s {
  */
 #define BUF_SPECIAL_PALETTE  1
 
-/*
- * In a BUF_SPECIAL_IDCIN_HUFFMAN_TABLE buffer:
- * decoder_info[1] = BUF_SPECIAL_IDCIN_HUFFMAN_TABLE
- * decoder_info[2] = 65536 (size of data)
- * decoder_info_ptr[2] = pointer to a 65536-element byte array containing the
- *  Huffman tables from an Id CIN file
- * This buffer is used to transport the Huffman tables from an Id CIN
- * file to the Id CIN decoder. A decoder should not count on the byte array
- * to exist for the duration of the program's execution and should copy the
- * data into its own private structures.
- */
-#define BUF_SPECIAL_IDCIN_HUFFMAN_TABLE  2
+
+/* special buffer type 2 used to be defined but is now available for use */
+
 
 /*
  * In a BUF_SPECIAL_ASPECT buffer:
  * decoder_info[1] = BUF_SPECIAL_ASPECT
- * decoder_info[2] = aspect ratio code
+ * decoder_info[2] = MPEG2 aspect ratio code
  * decoder_info[3] = stream scale prohibitions
  * This buffer is used to force mpeg decoders to use a certain aspect.
  * Currently xine-dvdnav uses this, because it has more accurate information
@@ -320,7 +400,7 @@ struct buf_element_s {
  * decoder_info[1] = BUF_SPECIAL_DECODER_CONFIG
  * decoder_info[2] = data size
  * decoder_info_ptr[2] = pointer to data
- * This buffer is used to pass config information from  .mp4 files 
+ * This buffer is used to pass config information from  .mp4 files
  * (atom esds) to decoders. both mpeg4 and aac streams use that.
  */
 #define BUF_SPECIAL_DECODER_CONFIG  4
@@ -328,9 +408,14 @@ struct buf_element_s {
 /*
  * In a BUF_SPECIAL_STSD_ATOM buffer:
  * decoder_info[1] = BUF_SPECIAL_STSD_ATOM
- * decoder_info[2] = size of the stsd atom
- * decoder_info_ptr[2] = pointer to stsd atom
- * binary-only quicktime decoders need this, sent by qt demuxer
+ * decoder_info[2] = size of the ImageDescription atom, minus the
+ *   four length bytes at the beginning
+ * decoder_info_ptr[2] = pointer to ImageDescription atom, starting with
+ *   the codec fourcc
+ * Some Quicktime decoders need information contained within the 
+ * ImageDescription atom inside a Quicktime file's stsd atom. This
+ * special buffer carries the ImageDescription atom from the QT demuxer
+ * to an A/V decoder.
  */
 #define BUF_SPECIAL_STSD_ATOM  5
 
@@ -344,8 +429,16 @@ struct buf_element_s {
  */
 #define BUF_SPECIAL_LPCM_CONFIG 6
 
-
-/* special buffer type 7 used to be defined but is now available for use */
+/*
+ * In a BUF_SPECIAL_CHARSET_ENCODING buffer:
+ * decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING
+ * decoder_info[2] = size of charset encoding string
+ * decoder_info_ptr[2] = pointer to charset encoding string
+ * This is used mostly with subtitle buffers when encoding is
+ * known at demuxer level (take precedence over xine config
+ * settings such as subtitles.separate.src_encoding)
+ */
+#define BUF_SPECIAL_CHARSET_ENCODING 7
 
 
 /*
@@ -356,23 +449,35 @@ struct buf_element_s {
  * This buffer is pass SPU subtypes from DVDs
  */
 #define BUF_SPECIAL_SPU_DVD_SUBTYPE 8
+
+
 #define SPU_DVD_SUBTYPE_CLUT		1
 #define SPU_DVD_SUBTYPE_PACKAGE		2
-#define SPU_DVD_SUBTYPE_SUBP_CONTROL	3
+#define SPU_DVD_SUBTYPE_VOBSUB_PACKAGE	3
 #define SPU_DVD_SUBTYPE_NAV		4
 
-
 /* In a BUF_SPECIAL_SPU_DVB_DESCRIPTOR
  * decoder_info[1] = BUF_SPECIAL_SPU_DVB_DESCRIPTOR
  * decoder_info[2] = size of spu_dvb_descriptor_t
  * decoder_info_ptr[2] = pointer to spu_dvb_descriptor_t, or NULL
- * decoder_info[3] = 
+ * decoder_info[3] =
  *
  * This buffer is used to tell a DVBSUB decoder when the stream
  * changes.  For more information on how to write a DVBSUB decoder,
  * see the comment at the top of src/demuxers/demux_ts.c
  **/
 #define BUF_SPECIAL_SPU_DVB_DESCRIPTOR 9
+
+/*
+ * In a BUF_SPECIAL_RV_CHUNK_TABLE:
+ * decoder_info[1] = BUF_SPECIAL_RV_CHUNK_TABLE
+ * decoder_info[2] = number of entries in chunk table
+ * decoder_info_ptr[2] = pointer to the chunk table
+ *
+ * This buffer transports the chunk table associated to each RealVideo frame.
+ */
+#define BUF_SPECIAL_RV_CHUNK_TABLE 10
+
 typedef struct spu_dvb_descriptor_s spu_dvb_descriptor_t;
 struct spu_dvb_descriptor_s
 {
@@ -391,7 +496,10 @@ typedef struct fifo_buffer_s fifo_buffer_t;
 struct fifo_buffer_s
 {
   buf_element_t  *first, *last;
+
   int             fifo_size;
+  uint32_t        fifo_data_size;
+  void            *fifo_empty_cb_data;
 
   pthread_mutex_t mutex;
   pthread_cond_t  not_empty;
@@ -401,80 +509,97 @@ struct fifo_buffer_s
    */
 
   void (*put) (fifo_buffer_t *fifo, buf_element_t *buf);
-  
+
   buf_element_t *(*get) (fifo_buffer_t *fifo);
 
   void (*clear) (fifo_buffer_t *fifo) ;
 
   int (*size) (fifo_buffer_t *fifo);
 
+  int (*num_free) (fifo_buffer_t *fifo);
+
+  uint32_t (*data_size) (fifo_buffer_t *fifo);
+
   void (*dispose) (fifo_buffer_t *fifo);
 
-  /* 
-   * alloc buffer for this fifo from global buf pool 
+  /*
+   * alloc buffer for this fifo from global buf pool
    * you don't have to use this function to allocate a buffer,
    * an input plugin can decide to implement it's own
    * buffer allocation functions
    */
 
-  buf_element_t *(*buffer_pool_alloc) (fifo_buffer_t *this);
-  
-  
+  buf_element_t *(*buffer_pool_alloc) (fifo_buffer_t *self);
+
+
   /*
    * special functions, not used by demuxers
-   */ 
-  
+   */
+
   /* the same as buffer_pool_alloc but may fail if none is available */
-  buf_element_t *(*buffer_pool_try_alloc) (fifo_buffer_t *this);
-  
-  /* the samme as put but insert at the head of the fifo */
+  buf_element_t *(*buffer_pool_try_alloc) (fifo_buffer_t *self);
+
+  /* the same as put but insert at the head of the fifo */
   void (*insert) (fifo_buffer_t *fifo, buf_element_t *buf);
-  
+
+  /* callbacks */
+  void (*register_alloc_cb) (fifo_buffer_t *fifo, void (*cb)(fifo_buffer_t *fifo, void *), void *cb_data);
+  void (*register_put_cb) (fifo_buffer_t *fifo, void (*cb)(fifo_buffer_t *fifo, buf_element_t *buf, void *), void *cb_data);
+  void (*register_get_cb) (fifo_buffer_t *fifo, void (*cb)(fifo_buffer_t *fifo, buf_element_t *buf, void *), void *cb_data);
+  void (*unregister_alloc_cb) (fifo_buffer_t *fifo, void (*cb)(fifo_buffer_t *fifo, void *));
+  void (*unregister_put_cb) (fifo_buffer_t *fifo, void (*cb)(fifo_buffer_t *fifo, buf_element_t *buf, void *));
+  void (*unregister_get_cb) (fifo_buffer_t *fifo, void (*cb)(fifo_buffer_t *fifo, buf_element_t *buf, void *));
 
   /*
    * private variables for buffer pool management
    */
-
   buf_element_t   *buffer_pool_top;    /* a stack actually */
   pthread_mutex_t  buffer_pool_mutex;
   pthread_cond_t   buffer_pool_cond_not_empty;
   int              buffer_pool_num_free;
-  int		   buffer_pool_capacity;
-  int		   buffer_pool_buf_size;
+  int              buffer_pool_capacity;
+  int              buffer_pool_buf_size;
   void            *buffer_pool_base; /*used to free mem chunk */
+  void           (*alloc_cb[BUF_MAX_CALLBACKS])(fifo_buffer_t *fifo, void *data_cb);
+  void           (*put_cb[BUF_MAX_CALLBACKS])(fifo_buffer_t *fifo, buf_element_t *buf, void *data_cb);
+  void           (*get_cb[BUF_MAX_CALLBACKS])(fifo_buffer_t *fifo, buf_element_t *buf, void *data_cb);
+  void            *alloc_cb_data[BUF_MAX_CALLBACKS];
+  void            *put_cb_data[BUF_MAX_CALLBACKS];
+  void            *get_cb_data[BUF_MAX_CALLBACKS];
 } ;
 
 /*
  * allocate and initialize new (empty) fifo buffer,
  * init buffer pool for it:
- * allocate num_buffers of buf_size bytes each 
+ * allocate num_buffers of buf_size bytes each
  */
 
-fifo_buffer_t *fifo_buffer_new (int num_buffers, uint32_t buf_size);
+fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size);
+fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size);
 
 
 /* return BUF_VIDEO_xxx given the fourcc
  * fourcc_int must be read in machine endianness
  * example: fourcc_int = *(uint32_t *)fourcc_char;
  */
-uint32_t fourcc_to_buf_video( uint32_t fourcc_int );
+uint32_t _x_fourcc_to_buf_video( uint32_t fourcc_int );
 
 /* return codec name given BUF_VIDEO_xxx */
-char * buf_video_name( uint32_t buf_type );
+char * _x_buf_video_name( uint32_t buf_type );
 
 /* return BUF_VIDEO_xxx given the formattag */
-uint32_t formattag_to_buf_audio( uint32_t formattag );
+uint32_t _x_formattag_to_buf_audio( uint32_t formattag );
 
 /* return codec name given BUF_VIDEO_xxx */
-char * buf_audio_name( uint32_t buf_type );
+char * _x_buf_audio_name( uint32_t buf_type );
 
 
 #ifndef ATTRIBUTE_PACKED
 /* no attribute packed? let's try with pragma pack as a last resort */
 #pragma pack(2)
 #endif
-/* this is xine version of BITMAPINFOHEADER 
- * - should be safe to compile on 64bits machines 
+/* this is xine version of BITMAPINFOHEADER
+ * - should be safe to compile on 64bits machines
  * - will always use machine endian format, so demuxers reading
  *   stuff from win32 formats must use the function below.
  */
@@ -509,10 +634,10 @@ typedef struct __attribute__((__packed__)) {
 #endif
 
 /* convert xine_bmiheader struct from little endian */
-void xine_bmiheader_le2me( xine_bmiheader *bih );
+void _x_bmiheader_le2me( xine_bmiheader *bih );
 
 /* convert xine_waveformatex struct from little endian */
-void xine_waveformatex_le2me( xine_waveformatex *wavex );
+void _x_waveformatex_le2me( xine_waveformatex *wavex );
 
 #ifdef __cplusplus
 }
