@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: metronom.h,v 1.43 2003/01/13 02:15:07 miguelfreitas Exp $
+ * $Id: metronom.h,v 1.62 2004/09/26 22:54:52 valtri Exp $
  *
  * metronom: general pts => virtual calculation/assoc
  *                   
@@ -47,16 +47,28 @@
 extern "C" {
 #endif
 
-#include <inttypes.h>
-#include <sys/time.h>
 #include <pthread.h>
-#include "video_out.h"
-#include "xine.h"
+
+#ifdef XINE_COMPILE
+#  include "video_out.h"
+#  include "xine.h"
+#else
+#  include <xine/video_out.h>
+#  include <xine.h>
+#endif
 
 typedef struct metronom_s metronom_t ;
 typedef struct metronom_clock_s metronom_clock_t;
 typedef struct scr_plugin_s scr_plugin_t;
 
+/* metronom prebuffer can be adjusted with XINE_PARAM_METRONOM_PREBUFFER.
+ * it sets how much the first video/audio frame should be delayed to
+ * have some prebuffering at the output layers. reducing this value (about
+ * 1/8 sec) may result in faster seeking (good to simulate play backwards,
+ * for example).
+ */
+#define PREBUFFER_PTS_OFFSET  12000
+
   /* see below */
 #define DISC_STREAMSTART 0 
 #define DISC_RELATIVE    1     
@@ -71,7 +83,7 @@ struct metronom_s {
    *
    * parameter pts_per_smpls : 1/90000 sec per 65536 samples
    */
-  void (*set_audio_rate) (metronom_t *this, int64_t pts_per_smpls);
+  void (*set_audio_rate) (metronom_t *self, int64_t pts_per_smpls);
 
   /*
    * called by video output driver for *every* frame
@@ -85,7 +97,7 @@ struct metronom_s {
    * 
    */
   
-  void (*got_video_frame) (metronom_t *this, vo_frame_t *frame);
+  void (*got_video_frame) (metronom_t *self, vo_frame_t *frame);
 
   /*
    * called by audio output driver whenever audio samples are delivered to it
@@ -100,7 +112,7 @@ struct metronom_s {
    *
    */
 
-  int64_t (*got_audio_samples) (metronom_t *this, int64_t pts, 
+  int64_t (*got_audio_samples) (metronom_t *self, int64_t pts, 
 				int nsamples); 
 
   /*
@@ -113,7 +125,7 @@ struct metronom_s {
    * due to the lack of regularity on spu packets)
    */
 
-  int64_t (*got_spu_packet) (metronom_t *this, int64_t pts);
+  int64_t (*got_spu_packet) (metronom_t *self, int64_t pts);
 
   /*
    * tell metronom about discontinuities.
@@ -134,42 +146,40 @@ struct metronom_s {
    *                    applies the necessary vpts offset for the seek in
    *                    metronom, but keeps the vpts difference between
    *                    audio and video, so that metronom doesn't cough
-   *
-   * for DISC_RELATIVE and DISC_ABSOLUTE metronom will enter a
-   * special discontinuity mode which means that it will ignore
-   * pts values for some time (about 1sec) to ignore any held-back
-   * reference frames that are flushed out of decoders containing
-   * pts values that do not mach the new offset. Then it will
-   * just switch to the new disc_offset and resume synced operation.
-   *                    
    */
-  void (*handle_audio_discontinuity) (metronom_t *this, int type, int64_t disc_off);
-  void (*handle_video_discontinuity) (metronom_t *this, int type, int64_t disc_off);
+  void (*handle_audio_discontinuity) (metronom_t *self, int type, int64_t disc_off);
+  void (*handle_video_discontinuity) (metronom_t *self, int type, int64_t disc_off);
 
   /*
    * set/get options for metronom, constants see below
    */
-  void (*set_option) (metronom_t *this, int option, int64_t value);
-  int64_t (*get_option) (metronom_t *this, int option);
+  void (*set_option) (metronom_t *self, int option, int64_t value);
+  int64_t (*get_option) (metronom_t *self, int option);
   
-  void (*exit) (metronom_t *this);
-
   /*
-   * pointer to current xine stream object. 
+   * set a master metronom
+   * this is currently useful to sync independently generated streams
+   * (e.g. by post plugins) to the discontinuity domain of another
+   * metronom
    */
-  xine_stream_t *stream;
+  void (*set_master) (metronom_t *self, metronom_t *master);
   
-  metronom_clock_t *clock;
+  void (*exit) (metronom_t *self);
 
+#ifdef METRONOM_INTERNAL
   /*
    * metronom internal stuff
    */
+  xine_t         *xine;
+  
+  metronom_t     *master;
 
   int64_t         pts_per_smpls;
 
   int64_t         video_vpts;
   int64_t         spu_vpts;
   int64_t         audio_vpts;
+  int64_t         audio_vpts_rmndr;  /* the remainder for integer division */
 
   int64_t         vpts_offset;
 
@@ -179,11 +189,13 @@ struct metronom_s {
   int             audio_samples;
   int64_t         audio_drift_step;
 
+  int64_t         prebuffer;
   int64_t         av_offset;
   int64_t         spu_offset;
 
   pthread_mutex_t lock;
 
+  int             have_video;
   int             have_audio;
   int             video_discontinuity_count;
   int             audio_discontinuity_count;
@@ -198,7 +210,10 @@ struct metronom_s {
   int64_t         img_duration;
   int             img_cpt;
   int64_t         last_video_pts;
-  
+  int64_t         last_audio_pts;
+
+  int             video_mode;
+#endif
 };
 
 /*
@@ -209,81 +224,106 @@ struct metronom_s {
 #define METRONOM_ADJ_VPTS_OFFSET  3
 #define METRONOM_FRAME_DURATION   4
 #define METRONOM_SPU_OFFSET       5
+#define METRONOM_VPTS_OFFSET      6
+#define METRONOM_PREBUFFER        7
 
-metronom_t *metronom_init (int have_audio, xine_stream_t *stream);
+metronom_t *_x_metronom_init (int have_video, int have_audio, xine_t *xine);
 
+/* FIXME: reorder this structure on the next cleanup to remove the dummies */
 struct metronom_clock_s {
 
   /*
    * set/get options for clock, constants see below
    */
-  void (*set_option) (metronom_clock_t *this, int option, int64_t value);
-  int64_t (*get_option) (metronom_clock_t *this, int option);
+  void (*set_option) (metronom_clock_t *self, int option, int64_t value);
+  int64_t (*get_option) (metronom_clock_t *self, int option);
 
   /*
    * system clock reference (SCR) functions
    */
 
+#ifdef METRONOM_CLOCK_INTERNAL
   /*
    * start clock (no clock reset)
    * at given pts
    */
-  void (*start_clock) (metronom_clock_t *this, int64_t pts);
+  void (*start_clock) (metronom_clock_t *self, int64_t pts);
 
 
   /*
    * stop metronom clock
    */
-  void (*stop_clock) (metronom_clock_t *this);
+  void (*stop_clock) (metronom_clock_t *self);
 
 
   /*
    * resume clock from where it was stopped
    */
-  void (*resume_clock) (metronom_clock_t *this);
+  void (*resume_clock) (metronom_clock_t *self);
+#else
+  void *dummy1;
+  void *dummy2;
+  void *dummy3;
+#endif
 
 
   /*
    * get current clock value in vpts
    */
-  int64_t (*get_current_time) (metronom_clock_t *this);
+  int64_t (*get_current_time) (metronom_clock_t *self);
 
 
   /*
    * adjust master clock to external timer (e.g. audio hardware)
    */
-  void (*adjust_clock) (metronom_clock_t *this, int64_t desired_pts);
-
+  void (*adjust_clock) (metronom_clock_t *self, int64_t desired_pts);
 
+#ifdef METRONOM_CLOCK_INTERNAL
   /*
    * set clock speed
    * for constants see xine_internal.h
    */
 
-  int (*set_speed) (metronom_clock_t *this, int speed);
+  int (*set_fine_speed) (metronom_clock_t *self, int speed);
+#else
+  void *dummy4;
+#endif
 
   /*
    * (un)register a System Clock Reference provider at the metronom
    */
-  int    (*register_scr) (metronom_clock_t *this, scr_plugin_t *scr);
-  void (*unregister_scr) (metronom_clock_t *this, scr_plugin_t *scr);
+  int    (*register_scr) (metronom_clock_t *self, scr_plugin_t *scr);
+  void (*unregister_scr) (metronom_clock_t *self, scr_plugin_t *scr);
 
-  void (*exit) (metronom_clock_t *this);
+#ifdef METRONOM_CLOCK_INTERNAL
+  void (*exit) (metronom_clock_t *self);
   
-  scr_plugin_t*   scr_master;
-  scr_plugin_t**  scr_list;
+  xine_t         *xine;
+  
+  scr_plugin_t   *scr_master;
+  scr_plugin_t  **scr_list;
   pthread_t       sync_thread;
   int             thread_running;
   int             scr_adjustable;
+#else
+  void *dummy5;
+  void *dummy6;
+  void *dummy7;
+  void *dummy8;
+  pthread_t dummy9;
+  int dummy10;
+  int dummy11;
+#endif
   
   int speed;
-  
+
+#ifdef METRONOM_CLOCK_INTERNAL
   pthread_mutex_t lock;
   pthread_cond_t  cancel;
-
+#endif
 };
 
-metronom_clock_t *metronom_clock_init(void);
+metronom_clock_t *_x_metronom_clock_init(xine_t *xine);
 
 /*
  * clock options
@@ -299,7 +339,7 @@ struct scr_plugin_s
 {
   int interface_version;
 
-  int (*get_priority) (scr_plugin_t *this);
+  int (*get_priority) (scr_plugin_t *self);
 
   /* 
    * set/get clock speed 
@@ -308,15 +348,15 @@ struct scr_plugin_s
    * returns actual speed
    */
 
-  int (*set_speed) (scr_plugin_t *this, int speed);
+  int (*set_fine_speed) (scr_plugin_t *self, int speed);
 
-  void (*adjust) (scr_plugin_t *this, int64_t vpts);
+  void (*adjust) (scr_plugin_t *self, int64_t vpts);
 
-  void (*start) (scr_plugin_t *this, int64_t start_vpts);
+  void (*start) (scr_plugin_t *self, int64_t start_vpts);
 
-  int64_t (*get_current) (scr_plugin_t *this);
+  int64_t (*get_current) (scr_plugin_t *self);
 
-  void (*exit) (scr_plugin_t *this);
+  void (*exit) (scr_plugin_t *self);
 
   metronom_clock_t *clock;
 };
