@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: resample.c,v 1.4 2002/10/23 17:12:34 guenter Exp $
+ * $Id: resample.c,v 1.8 2003/12/09 00:02:37 f1rmb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,16 +25,17 @@
 #endif
 
 #include <inttypes.h>
+#include "resample.h"
 
 /* contributed by paul flinders */
 
-void audio_out_resample_mono(int16_t* input_samples, uint32_t in_samples, 
-			     int16_t* output_samples, uint32_t out_samples)
+void _x_audio_out_resample_mono(int16_t* input_samples, uint32_t in_samples, 
+				int16_t* output_samples, uint32_t out_samples)
 {
   int osample;
   /* 16+16 fixed point math */
   uint32_t isample = 0;
-  uint32_t istep = (in_samples << 16)/out_samples;
+  uint32_t istep = ((in_samples-2) << 16)/(out_samples-2);
 
 #ifdef VERBOSE
   printf ("Audio : resample %d samples to %d\n",
@@ -59,13 +60,13 @@ void audio_out_resample_mono(int16_t* input_samples, uint32_t in_samples,
   output_samples[out_samples-1] = input_samples[in_samples-1];
 }
 
-void audio_out_resample_stereo(int16_t* input_samples, uint32_t in_samples, 
-			       int16_t* output_samples, uint32_t out_samples)
+void _x_audio_out_resample_stereo(int16_t* input_samples, uint32_t in_samples, 
+				  int16_t* output_samples, uint32_t out_samples)
 {
   int osample;
   /* 16+16 fixed point math */
   uint32_t isample = 0;
-  uint32_t istep = (in_samples << 16)/out_samples;
+  uint32_t istep = ((in_samples-2) << 16)/(out_samples-2);
 
 #ifdef VERBOSE
   printf ("Audio : resample %d samples to %d\n",
@@ -97,13 +98,13 @@ void audio_out_resample_stereo(int16_t* input_samples, uint32_t in_samples,
 }
 
 
-void audio_out_resample_4channel(int16_t* input_samples, uint32_t in_samples, 
-				 int16_t* output_samples, uint32_t out_samples)
+void _x_audio_out_resample_4channel(int16_t* input_samples, uint32_t in_samples, 
+				    int16_t* output_samples, uint32_t out_samples)
 {
   int osample;
   /* 16+16 fixed point math */
   uint32_t isample = 0;
-  uint32_t istep = (in_samples << 16)/out_samples;
+  uint32_t istep = ((in_samples-2) << 16)/(out_samples-2);
 
 #ifdef VERBOSE
   printf ("Audio : resample %d samples to %d\n",
@@ -151,13 +152,13 @@ void audio_out_resample_4channel(int16_t* input_samples, uint32_t in_samples,
 }
 
 
-void audio_out_resample_5channel(int16_t* input_samples, uint32_t in_samples, 
-				 int16_t* output_samples, uint32_t out_samples)
+void _x_audio_out_resample_5channel(int16_t* input_samples, uint32_t in_samples, 
+				    int16_t* output_samples, uint32_t out_samples)
 {
   int osample;
   /* 16+16 fixed point math */
   uint32_t isample = 0;
-  uint32_t istep = (in_samples << 16)/out_samples;
+  uint32_t istep = ((in_samples-2) << 16)/(out_samples-2);
 
 #ifdef VERBOSE
   printf ("Audio : resample %d samples to %d\n",
@@ -212,13 +213,13 @@ void audio_out_resample_5channel(int16_t* input_samples, uint32_t in_samples,
 }
 
 
-void audio_out_resample_6channel(int16_t* input_samples, uint32_t in_samples, 
-				 int16_t* output_samples, uint32_t out_samples)
+void _x_audio_out_resample_6channel(int16_t* input_samples, uint32_t in_samples, 
+				    int16_t* output_samples, uint32_t out_samples)
 {
   int osample;
   /* 16+16 fixed point math */
   uint32_t isample = 0;
-  uint32_t istep = (in_samples << 16)/out_samples;
+  uint32_t istep = ((in_samples-2) << 16)/(out_samples-2);
 
 #ifdef VERBOSE
   printf ("Audio : resample %d samples to %d\n",
@@ -279,8 +280,8 @@ void audio_out_resample_6channel(int16_t* input_samples, uint32_t in_samples,
   output_samples[out_samples*6-1] = input_samples[in_samples*6-1];
 }
 
-void audio_out_resample_8to16(int8_t* input_samples, 
-                              int16_t* output_samples, uint32_t samples)
+void _x_audio_out_resample_8to16(int8_t* input_samples, 
+				 int16_t* output_samples, uint32_t samples)
 {
   while( samples-- ) {
     int16_t os;
@@ -291,8 +292,8 @@ void audio_out_resample_8to16(int8_t* input_samples,
   }
 }
 
-void audio_out_resample_16to8(int16_t* input_samples, 
-                              int8_t* output_samples, uint32_t samples)
+void _x_audio_out_resample_16to8(int16_t* input_samples, 
+				 int8_t* output_samples, uint32_t samples)
 {
   while( samples-- ) {
     int16_t os;
@@ -303,8 +304,8 @@ void audio_out_resample_16to8(int16_t* input_samples,
   }
 }
 
-void audio_out_resample_monotostereo(int16_t* input_samples, 
-                                     int16_t* output_samples, uint32_t frames)
+void _x_audio_out_resample_monotostereo(int16_t* input_samples, 
+					int16_t* output_samples, uint32_t frames)
 {
   while( frames-- ) {
     int16_t os;
@@ -315,8 +316,8 @@ void audio_out_resample_monotostereo(int16_t* input_samples,
   }
 }
 
-void audio_out_resample_stereotomono(int16_t* input_samples, 
-                                     int16_t* output_samples, uint32_t frames)
+void _x_audio_out_resample_stereotomono(int16_t* input_samples, 
+					int16_t* output_samples, uint32_t frames)
 {
   while( frames-- ) {
     int16_t os;
